<?xml version="1.0" encoding="ISO-8859-1"?>

<xsl:stylesheet xmlns:fo="http://www.w3.org/1999/XSL/Format"
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">

  <xsl:output method="xml"/>
  
  <xsl:template match="article">
    <fo:root>
      <!-- defines the layout master -->
      <fo:layout-master-set>
	<fo:simple-page-master master-name="first" 
	  height="29.7cm" width="21cm" 
	  margin-top="1cm" 
	  margin-bottom="2cm" 
	  margin-left="2.5cm" 
	  margin-right="2.5cm">
	  <fo:region-before extent="3cm"/>
	  <fo:region-body margin-top="3cm"/>
	  <fo:region-after extent="3cm"/>
	</fo:simple-page-master>
      </fo:layout-master-set>
      
      <!-- starts actual layout -->
      <fo:page-sequence master-name="first">
	
	<!-- Inserts a header with the page number -->
	<fo:static-content flow-name="xsl-region-before">
	  <fo:table>
	    <fo:table-column column-width="7.5cm"/>
	    <fo:table-column column-width="8.5cm"/>
	    <fo:table-body>
	      <fo:table-row>
		<fo:table-cell>
		  <fo:block text-align="start" font-size="10pt" font-family="serif" line-height="14pt">
		    <xsl:value-of select="head/time"/>
		  </fo:block>
		</fo:table-cell>
		<fo:table-cell>
		  <fo:block text-align="end" font-size="10pt" font-family="serif" line-height="14pt">
		    <xsl:value-of select="head/title"/> - p. <fo:page-number/>
		  </fo:block>
		</fo:table-cell>
	      </fo:table-row>
	    </fo:table-body>
	  </fo:table>   
	</fo:static-content>
	
	<fo:flow flow-name="xsl-region-body">
	  <!-- title -->
	  <fo:block font-size="24pt" 
	    font-family="sans-serif" 
	    line-height="28pt" 
	    space-before.optimum="3pt" 
	    space-after.optimum="3pt" 
	    font-weight="bold">
	    <xsl:value-of select="head/title"/>
	  </fo:block>
	  <xsl:apply-templates select="body"/>
	  <fo:block font-size="12pt" 
	    font-family="sans-serif" 
	    line-height="15pt" 
	    space-before.optimum="3pt" 
	    space-after.optimum="3pt" 
	    font-weight="bold">
	    <xsl:value-of select="head/author"/>
	  </fo:block>
	</fo:flow>
      </fo:page-sequence>
    </fo:root>
  </xsl:template>
  
  <!-- Body/Intro -->
  
  <!-- Default: applyTemplates -->
  
  <!-- Paragraph -->
  <xsl:template match="paragraph">
    <fo:block font-size="12pt" 
      font-family="sans-serif" 
      line-height="15pt" 
      space-after.optimum="3pt" >
      <xsl:apply-templates/>
    </fo:block>
  </xsl:template>
  
  <xsl:template match="section">
    <fo:block font-size="16pt" 
      font-family="sans-serif" 
      line-height="17pt" 
      space-after.optimum="3pt"
      font-weight="bold" >
      <xsl:apply-templates/>
    </fo:block>
  </xsl:template>
  
  <xsl:template match="subsection">
    <fo:block font-size="14pt" 
      font-family="sans-serif" 
      line-height="17pt" 
      space-after.optimum="3pt" >
      <xsl:apply-templates/>
    </fo:block>
  </xsl:template>
  
  <xsl:template match="emphasized">
    <fo:inline font-weight="bold">
      <xsl:apply-templates/>
    </fo:inline>
  </xsl:template>
  
  <xsl:template match="emphasized2">
    <fo:inline font-style="italic">
      <xsl:apply-templates/>
    </fo:inline>
  </xsl:template>
  
  <xsl:template match="emphasized3">
    <fo:inline font-style="italic" font-weight="bold">
      <xsl:apply-templates/>
    </fo:inline>
  </xsl:template>
  
  <xsl:template match="ul">
    <fo:list-block>
      <xsl:for-each select="paragraph">
	<fo:list-item>
	  <!-- insert a bullet -->
	  <fo:list-item-label>
	    <fo:block font-size="12pt" 
	      font-family="sans-serif" 
	      line-height="15pt" 
	      space-after.optimum="3pt" >
	      &#x2022;
	    </fo:block>
	  </fo:list-item-label>
	  <!-- list text --> 
	  <fo:list-item-body>
	    <fo:block font-size="12pt" 
	      font-family="sans-serif" 
	      line-height="15pt" 
	      space-after.optimum="3pt" >
	      <xsl:apply-templates/>
	    </fo:block>
	  </fo:list-item-body>
	</fo:list-item>
      </xsl:for-each>
    </fo:list-block>
  </xsl:template>

  <xsl:template match="ol">
    <fo:list-block>
      <xsl:for-each select="paragraph">
	<fo:list-item>
	  <!-- insert a bullet -->
	  <fo:list-item-label>
	    <fo:block font-size="12pt" 
	      font-family="sans-serif" 
	      line-height="15pt" >
	      <xsl:value-of select="concat(position(),'.)')"/>
	    </fo:block>
	  </fo:list-item-label>
	  <!-- list text --> 
	  <fo:list-item-body>
	    <fo:block font-size="12pt" 
	      font-family="sans-serif" 
	      line-height="15pt" >
	      <xsl:apply-templates/>
	    </fo:block>
	  </fo:list-item-body>
	</fo:list-item>
      </xsl:for-each>
    </fo:list-block>
  </xsl:template>

  <!--<xsl:template match="image">
<fo:block text-align="centered">
<fo:external-graphic href="{@src}"/>
</fo:block>
</xsl:template> -->
  
</xsl:stylesheet>
