<?xml version="1.0" encoding="ISO-8859-1"?>

<!--
  Dieses Style wandelt einen Artikel im stimme-XML-Format
  nach XSL um.
  2000/03/08 Max Kellermann
-->

<xsl:stylesheet xmlns:fo="http://www.w3.org/1999/XSL/Format"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
    version="1.0"
    xmlns:lxslt="http://xml.apache.org/xslt"
    xmlns:my-ext="ext1"
    extension-element-prefixes="my-ext">
<xsl:output method="xml"/>

<lxslt:component prefix="my-ext" functions="calcsum">
  <lxslt:script lang="javascript"><![CDATA[
    
    function lineValue(line) {
      price = 0; count = 1;
      for (node=line.getFirstChild(); node; node=node.getNextSibling()) {
        if (node.getNodeName().equals("price"))
	  price = parseInt(node.getFirstChild().getNodeValue());
	else if (node.getNodeName().equals("count"))
	  count = parseInt(node.getFirstChild().getNodeValue());
      }
      return price*count;
    }

    function calcsum(lines) {
      sum = 0;
      for (z=0; z<lines.getLength(); z++)
        sum += lineValue(lines.item(z));
      return sum;
    }

]]></lxslt:script>
</lxslt:component>

<xsl:template match="invoice">
  <fo:root xmlns:fo="http://www.w3.org/1999/XSL/Format">
    <!-- defines the layout master -->
    <fo:layout-master-set>
      <fo:simple-page-master page-master-name="first" 
                             height="29.7cm" width="21cm" 
                             margin-top="1cm" 
                             margin-bottom="1cm" 
                             margin-left="2cm" 
                             margin-right="2cm">
        <fo:region-before extent="5cm"/>
        <fo:region-body margin-top="5cm"/>
        <fo:region-after extent="1.5cm"/>
      </fo:simple-page-master>
    </fo:layout-master-set>

    <!-- starts actual layout -->
    <fo:page-sequence>

      <!-- applies layout master -->
      <fo:sequence-specification>
        <fo:sequence-specifier-single page-master-name="first"/>
      </fo:sequence-specification>

      <!-- Inserts a header with the page number -->
      <fo:static-content flow-name="xsl-before">
        <fo:block text-indent="9cm" font-size="12pt" font-family="serif"
		  line-height="14pt" space-before.maximum="0pt">
        Xtensible Publishing System
	</fo:block>
	<fo:block text-indent="9cm" font-size="12pt" font-family="serif"
		  line-height="14pt" space-before.optimum="20pt">
	  Max Kellermann
        </fo:block>
	<fo:block text-indent="9cm" font-size="12pt" font-family="serif"
		  line-height="14pt">
	  von-Andreae Str. 14 
        </fo:block>
	<fo:block text-indent="9cm" font-size="12pt" font-family="serif"
		  line-height="14pt">
	  51427 Bergisch Gladbach
	</fo:block>
	<fo:block text-indent="9cm" font-size="12pt" font-family="serif"
		  line-height="14pt">
	 Tel.: +49 2204 609050
	</fo:block>
	<fo:block text-indent="9cm" font-size="12pt" font-family="serif"
		  line-height="14pt">
	 Fax.: +49 2204 23311
	</fo:block>
	<fo:block text-indent="9cm" font-size="12pt" font-family="serif"
		  line-height="14pt">
	 e-mail: max.kellermann@netcologne.de
	</fo:block>
      </fo:static-content>

      <fo:static-content flow-name="xsl-after">
        <fo:block text-align="centered" font-size="12pt" font-family="serif"
		  line-height="14pt">
        Mein Konto: Bankleitzahl 370xxxx Konto Nr.: 130912345
	</fo:block>
      </fo:static-content>

      <fo:flow flow-name="xsl-body">
        
        
        <fo:block font-size="14pt" 
                  font-family="sans-serif" 
                  line-height="18pt" 
                  space-before.optimum="3pt" 
                  space-after.optimum="3pt" 
                  font-weight="bold" 
                  start-indent="15pt">
          <xsl:value-of select="head/title"/>
        </fo:block>
        <fo:block space-before.optimum="30pt">
	</fo:block>
        <xsl:apply-templates select="header/customer"/>
        <fo:block text-indent="9cm" font-size="12pt" font-family="serif" line-height="14pt">
          <xsl:variable name="d" select="header/date"/>
	  <xsl:value-of select="substring($d,9,2)"/>
          <xsl:text>.</xsl:text>
	  <xsl:value-of select="substring($d,6,2)"/>
	  <xsl:text>.</xsl:text>
          <xsl:value-of select="substring($d,1,4)"/>
	  um <xsl:value-of select="substring($d,11,5)"/> Uhr

        </fo:block>
        <!-- h1> -->  
<!--	<fo:block>
          <xsl:value-of select="header/title"/>
	</fo:block>-->
        <fo:block space-before.optimum="50pt">
          <xsl:value-of select="header/intro"/>
        </fo:block>	
	<fo:block space-before.optimum="20pt" space-after.optimum="30pt">
        Rechnung
	</fo:block>
        <fo:table>
	  <fo:table-column column-width="7cm"/>
          <fo:table-column column-width="4cm"/>
       	  <fo:table-body>
            <xsl:apply-templates select="body"/>
	    <xsl:variable name="sum" select="my-ext:calcsum(body/line)"/>
	    <fo:table-row>
	      <fo:table-cell/> 
	      <fo:table-cell>
	        <fo:block text-align="end">
                  <xsl:text>---------------</xsl:text>
		</fo:block>
	      </fo:table-cell>
	    </fo:table-row>
	    <fo:table-row>
	      <fo:table-cell/> 
	      <fo:table-cell>
	        <fo:block text-align="end">
                  <xsl:value-of select="format-number($sum,'0,00')"/>
	          <xsl:text> </xsl:text>
                  <xsl:value-of select="/invoice/header/currency"/>
		</fo:block>
	      </fo:table-cell>
	    </fo:table-row>
  	    <xsl:if test="count(header/vat)>0">
	      <xsl:variable name="mwst" select="header/vat"/>
              <fo:table-row>
		<fo:table-cell>
		  <fo:block>	
                    <xsl:value-of select="$mwst"/>
	            <xsl:text>% MWST</xsl:text>
		  </fo:block>
                </fo:table-cell>
		<fo:table-cell>
		  <fo:block text-align="end">
                    <xsl:value-of select="format-number(($sum*$mwst) div 100,'0,00')"/>
	            <xsl:text> </xsl:text>
                    <xsl:value-of select="/invoice/header/currency"/>
		  </fo:block>  
		</fo:table-cell>  
	      </fo:table-row>
	      <fo:table-row>	
	        <fo:table-cell/> 
	        <fo:table-cell>
	          <fo:block text-align="end">
                    <xsl:text>---------------</xsl:text>
	   	  </fo:block>
	        </fo:table-cell>
	      </fo:table-row>
	      <fo:table-row>
                <fo:table-cell/>
                <fo:table-cell>
		  <fo:block text-align="end">
	            <xsl:value-of select="format-number(($sum*($mwst+100)) div 100,'0,00')"/>
                    <xsl:text> </xsl:text>
                    <xsl:value-of select="/invoice/header/currency"/>
		  </fo:block>  
	        </fo:table-cell>
	      </fo:table-row>
	    </xsl:if>
	  </fo:table-body>  
        </fo:table> 
        <fo:block space-before.optimum="50pt">
          <xsl:text> 
            Wir bitten um Ueberweisung des Rechnungsbetrags auf unser Konto. 
	  </xsl:text>
	</fo:block>
	<fo:block space-before.optimum="30pt">
          <xsl:text> 
             Mit Dank,
	  </xsl:text>
	</fo:block>
	<fo:block space-before.optimum="60pt">
          <xsl:value-of select="header/extro"/>
        </fo:block>
      </fo:flow>
    </fo:page-sequence>
  </fo:root>
</xsl:template>

<!-- Body/Intro -->

  <!-- Default: applyTemplates -->

<xsl:template match="customer">
  <fo:block>
    <xsl:value-of select="salutation"/>
  </fo:block>
  <fo:block>
    <xsl:value-of select="name"/>
  </fo:block>
  <fo:block>
    <xsl:value-of select="address1"/>
  </fo:block>
  <fo:block>
    <xsl:value-of select="address2"/>
    <!--<xsl:value-of select="address3"/>-->
  </fo:block>
</xsl:template>

<xsl:template match="line">
  <fo:table-row>
    <fo:table-cell>
      <fo:block>
        <xsl:value-of select="count"/>
        <xsl:text> </xsl:text>
        <xsl:value-of select="text"/>
        <xsl:if test="count!=''">
          <xsl:text>  </xsl:text>
          <xsl:value-of select="format-number(price,'0,00')"/>
          <xsl:text> </xsl:text>
          <xsl:value-of select="/invoice/header/currency"/>
        </xsl:if>
      </fo:block>	
    </fo:table-cell>
    <fo:table-cell>
      <fo:block text-align="end">    
        <xsl:choose>
          <xsl:when test="count!=''">
            <xsl:value-of select="format-number(price*count,'0,00')"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="format-number(price,'0,00')"/>
          </xsl:otherwise>
        </xsl:choose>
        <xsl:text> </xsl:text>
        <xsl:value-of select="/invoice/header/currency"/>
      </fo:block>    	
    </fo:table-cell>
  </fo:table-row>


</xsl:template>
<!--\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\-->
<!-- Paragraph -->
<xsl:template match="intro/paragraph">
  <fo:block font-size="12pt" 
            font-family="sans-serif" 
            line-height="15pt" 
            space-after.optimum="3pt" >
    <xsl:apply-templates/>
  </fo:block>
</xsl:template>

<xsl:template match="paragraph">
  <fo:block font-size="12pt" 
            font-family="sans-serif" 
            line-height="15pt" 
            space-after.optimum="3pt" >
    <xsl:apply-templates/>
  </fo:block>
</xsl:template>

<xsl:template match="section">
  <fo:block font-size="24pt" 
            font-family="sans-serif" 
            line-height="27pt" 
            space-after.optimum="3pt" >
    <xsl:apply-templates/>
  </fo:block>
</xsl:template>

<xsl:template match="subsection">
  <fo:block font-size="18pt" 
            font-family="sans-serif" 
            line-height="21pt" 
            space-after.optimum="3pt" >
    <xsl:apply-templates/>
  </fo:block>
</xsl:template>

<xsl:template match="emphasized">
  <fo:inline font-weight="bold">
    <xsl:apply-templates/>
  </fo:inline>
</xsl:template>

<xsl:template match="emphasized2">
  <fo:inline font-style="italic">
    <xsl:apply-templates/>
  </fo:inline>
</xsl:template>

<xsl:template match="emphasized3">
  <fo:inline font-style="italic" font-weight="bold">
    <xsl:apply-templates/>
  </fo:inline>
</xsl:template>

<xsl:template match="note"/>

</xsl:stylesheet>











