<?xml version="1.0" encoding="ISO-8859-1"?>

<!--
  Dieses Style wandelt eine XML-Rechnung nach HTML um. (German)
  1999/12/10 Max Kellermann
-->

<xsl:stylesheet 
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
    version="1.0"
    xmlns:lxslt="http://xml.apache.org/xslt"
    xmlns:my-ext="ext1"
    extension-element-prefixes="my-ext">

<xsl:output method="html"/>

<lxslt:component prefix="my-ext" functions="calcsum">
  <lxslt:script lang="javascript"><![CDATA[
    
    function lineValue(line) {
      price = 0; count = 1;
      for (node=line.getFirstChild(); node; node=node.getNextSibling()) {
        if (node.getNodeName().equals("price"))
	  price = parseInt(node.getFirstChild().getNodeValue());
	else if (node.getNodeName().equals("count"))
	  count = parseInt(node.getFirstChild().getNodeValue());
      }
      return price*count;
    }

    function calcsum(lines) {
      sum = 0;
      for (z=0; z<lines.getLength(); z++)
        sum += lineValue(lines.item(z));
      return sum;
    }

]]></lxslt:script>
</lxslt:component>

<xsl:template match="invoice">
  <html>
    <head>
      <title>Rechnung vom <xsl:value-of select="header/date"/></title>
    </head>
    <body bgcolor="white">
      <xsl:apply-templates select="header/customer"/>
      <p align="right">
        <xsl:variable name="d" select="header/date"/>
	<xsl:value-of select="substring($d,9,2)"/>
	<xsl:text>.</xsl:text>
	<xsl:value-of select="substring($d,6,2)"/>
	<xsl:text>.</xsl:text>
	<xsl:value-of select="substring($d,1,4)"/>
      </p>
      <h1><xsl:value-of select="header/title"/></h1>
      <p><xsl:value-of select="header/intro"/></p>
      <table width="100%">
        <tr>
          <td></td>
          <th align="left">Beschreibung</th>
          <th align="right">Preis</th>
        </tr>
        <xsl:apply-templates select="body"/>
	<xsl:variable name="sum" select="my-ext:calcsum(body/line)"/>
	<!--<xsl:for-each select="body/line">
	  <xsl:variable name="lpr" select="price"/>
	  <xsl:if test="count!=''">
	    <xsl:variable name="lpr" select="count*$lpr"/>
	  </xsl:if>
	  <xsl:variable name="sum" select="$sum+$lpr"/>
	</xsl:for-each>-->
	<tr>
          <td></td>
          <td></td>
          <td align="right">
	    <hr></hr>
            <xsl:value-of select="format-number($sum, '0,00')"/>
	    <xsl:text> </xsl:text>
            <xsl:value-of select="/invoice/header/currency"/>
	  </td>
	</tr>
	<xsl:if test="count(header/vat)>0">
	  <xsl:variable name="mwst" select="header/vat"/>
          <tr>
            <td></td>
            <td>
              <xsl:value-of select="$mwst"/>
	      <xsl:text>% MWST</xsl:text>
            </td>
            <td align="right">
              <xsl:value-of select="format-number(($sum*$mwst) div 100, '0,00')"/>
	      <xsl:text> </xsl:text>
              <xsl:value-of select="/invoice/header/currency"/>
	    </td>
	  </tr>
          <tr>
            <td></td>
            <td></td>
            <td align="right">
	      <hr></hr>
              <xsl:value-of select="format-number(($sum*($mwst+100)) div 100, '0,00')"/>
	      <xsl:text> </xsl:text>
              <xsl:value-of select="/invoice/header/currency"/>
	    </td>
	  </tr>
	</xsl:if>
      </table>
      <p><xsl:value-of select="header/extro"/></p>
    </body>
  </html>
</xsl:template>

<xsl:template match="customer">
  <p>
    <xsl:value-of select="salutation"/><br></br>
    <xsl:value-of select="name"/><br></br>
    <xsl:value-of select="address1"/><br></br>
    <xsl:value-of select="address2"/><br></br>
    <!--<xsl:value-of select="address3"/><br></br>-->
  </p>
</xsl:template>

<xsl:template match="line">
  <tr>
    <th align="right">
      <xsl:value-of select="prefix"/>
    </th>
    <td>
      <xsl:value-of select="count"/>
      <xsl:text> </xsl:text>
      <xsl:value-of select="text"/>
      <xsl:if test="count!=''">
        <xsl:text>  </xsl:text>
        <xsl:value-of select="format-number(price, '0,00')"/>
        <xsl:text> </xsl:text>
        <xsl:value-of select="/invoice/header/currency"/>
      </xsl:if>
    </td>
    <td align="right">
      <xsl:choose>
        <xsl:when test="count!=''">
          <xsl:value-of select="format-number(price*count, '0,00')"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="format-number(price, '0,00')"/>
        </xsl:otherwise>
      </xsl:choose>
      <xsl:text> </xsl:text>
      <xsl:value-of select="/invoice/header/currency"/>
    </td>
  </tr>
</xsl:template>

</xsl:stylesheet>
