/*
 *
 *  LeanEdit
 *  Copyright (c) 1999-2001 Max Kellermann (max.kellermann@epost.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: AboutDialog.java,v 1.9 2002/03/12 21:19:51 max Exp $
 *
 */

package org.leanedit;

import java.awt.Frame;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.io.IOException;
import java.util.ResourceBundle;
import javax.swing.Action;
import javax.swing.AbstractAction;
import javax.swing.JScrollPane;
import javax.swing.JEditorPane;
import javax.swing.Box;
import javax.swing.JLabel;
import javax.swing.BorderFactory;
import make.swing.bt.FrameDescription;
import make.swing.bt.ButtonDescription;
import make.swing.panel.InterfacePanel;
import make.swing.panel.PanelContent;
import make.swing.panel.DialogInterfacePanel;

class LicenseDialog implements PanelContent {
    /*static*****************************/
    protected static final FrameDescription frameDescription
        = FrameDescription.closeDescription;
    /*properties************************/
    protected InterfacePanel interfacePanel;
    protected JScrollPane scrollPane;
    /*Constructor*********************/
    public LicenseDialog(InterfacePanel _ip) {
        interfacePanel = _ip;
        interfacePanel.setTitle("License");
        interfacePanel.addFrameDescription(frameDescription);
        try {
            JEditorPane ep = new JEditorPane(getClass().getResource("COPYING"));
            scrollPane = new JScrollPane(ep);
            scrollPane.setPreferredSize(new Dimension(530, 500));
        } catch (IOException e) {
            e.printStackTrace();
        }
        interfacePanel.setContent(this);
    }
    /*PanelContent**********************/
    public Component getComponent() {
        return scrollPane;
    }
    public boolean panelClosing() {
        return true;
    }
    public void panelClosed() {
        interfacePanel.removeFrameDescription(frameDescription);
    }
}

class JavaDialog implements PanelContent {
    /*static*****************************/
    protected static final FrameDescription frameDescription
        = FrameDescription.closeDescription;
    /*properties************************/
    protected InterfacePanel interfacePanel;
    protected Box box = Box.createVerticalBox();
    /*Constructor*********************/
    public JavaDialog(InterfacePanel _ip) {
        interfacePanel = _ip;
        interfacePanel.setTitle("Java info");
        interfacePanel.addFrameDescription(frameDescription);
        box.add(new JLabel("Hardware Architecture: '"+
                           System.getProperty("os.arch")+
                           "'"));
        box.add(new JLabel("Operating System (OS): '"+
                           System.getProperty("os.name")+
                           "' version '"+
                           System.getProperty("os.version")+
                           "'"));
        box.add(new JLabel("Java Virtual Machine (JVM): '"+
                           System.getProperty("java.vendor")+
                           "' version '"+
                           System.getProperty("java.version")+
                           "'"));
        interfacePanel.setContent(this);
        box.getParent().setBackground(Color.white);
    }
    /*PanelContent**********************/
    public Component getComponent() {
        return box;
    }
    public boolean panelClosing() {
        return true;
    }
    public void panelClosed() {
        interfacePanel.removeFrameDescription(frameDescription);
    }
}

public class AboutDialog implements PanelContent {
    /*CONST************************************/
    protected static final ResourceBundle version
        = ResourceBundle.getBundle("org.leanedit.Version");
    private static final String contentText
        = "<html>"+
        "<p>LeanEdit v"+version.getString("version")+"<br>"+
        "\u00a9 1999-2001 Max Kellermann (max.kellermann@epost.de)<br>"+
        "<p>LeanEdit comes with ABSOLUTELY NO WARRANTY<br>"+
        "This is free software, and you are welcome to redistribute it<br>"+
        "under certain conditions; click on `License' for details."+
        "<p>This software uses the following libraries:<ul>"+
        "<li>Xerces 1.4.3 \u00a9 The Apache Software Foundation</li>"+
        "<li>Xalan 2.1 \u00a9 The Apache Software Foundation</li>"+
        "<li>FOP 0.20.1 \u00a9 The Apache Software Foundation</li>"+
        "<li>BSF 2.1 \u00a9 International Business Machines</li>"+
        "<li>Rhino \u00a9 Netscape Communications</li>"+
        "</ul>"+
        "<p>All trademarks are property of their respective owners."+
        "<p><br>Many thanks to The Apache Software Foundation, "+
        "IBM alphaworks & developerworks and the Linux community."+
        "<p>For feedback contact max@leanedit.org."+
        "</html>";
    /*static*****************************/
    protected static final ButtonDescription[] buttons = {
        new ButtonDescription("license", "License"),
        new ButtonDescription("java", "Java"),
        new ButtonDescription("close-frame", "Close"),
    };
    protected static final FrameDescription frameDescription;
    static {
        frameDescription = new FrameDescription();
        frameDescription.setButtonBox(buttons);
    }
    /*properties************************/
    protected InterfacePanel interfacePanel;
    protected JLabel text;
    /*Constructor***************************/
    public AboutDialog(InterfacePanel _ip) {
        interfacePanel = _ip;
        interfacePanel.setTitle("About LeanEdit");
        interfacePanel.addFrameDescription(frameDescription);
        interfacePanel.getButtonManager().installActions(actions);
        text = new JLabel(contentText);
        text.setBorder(BorderFactory.createEmptyBorder(10, 10, 20, 0));
        interfacePanel.setContent(this);
        text.getParent().setBackground(Color.white);
    }
    /*PanelContent**********************/
    public Component getComponent() {
        return text;
    }
    public boolean panelClosing() {
        return true;
    }
    public void panelClosed() {
        interfacePanel.removeFrameDescription(frameDescription);
        interfacePanel.getButtonManager().uninstallActions(actions);
    }
    /*Actions***************************/
    protected Action[] actions = new Action[]{
        new LicenseAction(),
        new JavaAction(),
    };
    class LicenseAction extends AbstractAction {
        LicenseAction() { super("license"); }
        public void actionPerformed(ActionEvent evt) {
            InterfacePanel ip = new DialogInterfacePanel(interfacePanel.getFrame());
            new LicenseDialog(ip);
            ip.setVisible(true);
        }
    }
    class JavaAction extends AbstractAction {
        JavaAction() { super("java"); }
        public void actionPerformed(ActionEvent evt) {
            InterfacePanel ip = new DialogInterfacePanel(interfacePanel.getFrame());
            new JavaDialog(ip);
            ip.setVisible(true);
        }
    }
}
