/*
 *
 *  LeanEdit
 *  Copyright (c) 1999-2001 Max Kellermann (max.kellermann@epost.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: DefaultPanel.java,v 1.7 2002/03/12 21:19:19 max Exp $
 *
 */

package org.leanedit;

import java.awt.Frame;
import java.awt.Component;
import java.util.Enumeration;
import make.swing.SwingSession;
import make.swing.panel.InterfacePanel;
import make.swing.panel.PanelContent;

/**
 * DefaultPanel is an abstract PanelContent implementation 
 * that belongs to a SwingSession.
 */
public abstract class DefaultPanel implements PanelContent {
    /*properties******************************/
    private SwingSession session;
    private Component component;
    private Enumeration sessionActions;
    private InterfacePanel interfacePanel;
    /*Constructor*****************************/
    public DefaultPanel(SwingSession _sess, Component _component, 
                        InterfacePanel _ip) {
        session = _sess;
        component = _component;
        interfacePanel = _ip;
        // install actions from session
        sessionActions = _sess.getActions();
        if (sessionActions!=null) {
            interfacePanel.getButtonManager().installActions(sessionActions);
            sessionActions = _sess.getActions();
        }
    }
    /*DefaultPanel************************/
    public final SwingSession getSwingSession() { 
        return session; 
    }
    public final InterfacePanel getInterfacePanel() { 
        return interfacePanel;
    }
    public final Frame getFrame() { 
        return interfacePanel.getFrame(); 
    }
    /*PanelContent**************************/
    public Component getComponent() {
        return component;
    }
    public boolean panelClosing() { 
        return true; 
    }
    public void panelClosed() {
        if (sessionActions!=null) {
            interfacePanel.getButtonManager().uninstallActions(sessionActions);
            sessionActions = null;
        }
        interfacePanel = null;
        component = null;
    }
}
