/*
 *
 *  LeanEdit
 *  Copyright (c) 1999-2001 Max Kellermann (max.kellermann@epost.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: EditorApplication.java,v 1.9 2001/09/20 20:20:53 max Exp $
 *
 */

package org.leanedit;

import java.util.Enumeration;
import org.xml.sax.EntityResolver;
import make.xml.SchemaManager;
import org.leanedit.plugins.PluginManager;
import org.leanedit.module.ModuleManager;
import org.leanedit.convert.ConverterManager;

/**
 * EditorApplication represents a whole instance of
 * LeanEdit.
 */
public interface EditorApplication {
    /**
     * Returns all existing sessions.
     */
    Enumeration getSessions();
    /**
     * Adds a new session.
     */
    void addSession(EditorSession session);
    /**
     * Removes a session.
     */
    void removeSession(EditorSession session);
    /**
     * returns the application's PluginManager instance
     */
    PluginManager getPluginManager();
    /**
     * returns the application's SchemaManager instance
     */
    SchemaManager getSchemaManager();
    /**
     * returns the application's ModuleManager instance
     */
    ModuleManager getModuleManager();
    /**
     * returns the application's ConverterManager instance
     */
    ConverterManager getConverterManager();
    /**
     * returns the application's EntityResolver. It is
     * responsible to load external entities.
     */
    EntityResolver getEntityResolver();
}
