/*
 *
 *  LeanEdit
 *  Copyright (c) 1999-2001 Max Kellermann (max.kellermann@epost.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: EditorDocumentView.java,v 1.4 2001/09/20 20:20:53 max Exp $
 *
 */

package org.leanedit;

import java.util.Hashtable;
import org.w3c.dom.Element;
import org.w3c.dom.Attr;
import make.xml.DOMUtil;
import make.xml.ConfigPath;
import make.util.Debug;
import org.leanedit.plugins.PluginManager;
import org.leanedit.plugins.EditorPlugin;

public class EditorDocumentView {
    /***properties***********************/
    protected ConfigPath configPath;
    protected PluginManager pluginManager;
    protected EditorDocumentType documentType;
    protected String name, title, plugin, description;
    protected Hashtable cache = new Hashtable();
    /***Constructor********************/
    public EditorDocumentView(PluginManager _pm) {
	configPath = new ConfigPath(null, null, null);
	pluginManager = _pm;
	name = "tree"; title = "Tree view";
	plugin = "tree";
    }
    public EditorDocumentView(EditorDocumentType _dt, Element vd) {
	documentType = _dt;
	configPath = documentType.getConfigPath().getPath(vd.getAttribute("name"));
	pluginManager = documentType.getApplication().getPluginManager();
	name = vd.getAttribute("name");
	title = DOMUtil.getElementString(vd, "title");
	if (title==null)
	    title = name;
	plugin = DOMUtil.getElementString(vd, "plugin");
	if (plugin==null) {
	    if (Debug.DEBUG)
		System.err.println("WARNING: no plugin specified");
	    plugin = "tree";
	}
	description = DOMUtil.getElementString(vd, "description");
    }
    /***Info****************************/
    public String getName() { return name; }
    public String getTitle() { return title; }
    public String getDescription() { return description; }
    public EditorPlugin loadPlugin() {
	EditorPlugin pl = pluginManager.loadPlugin(plugin);
	if (pl==null)
	    pl = pluginManager.loadPlugin("tree");
	return pl;
    }
    public ConfigPath getConfigPath() { return configPath; }
}
