/*
 *
 *  LeanEdit
 *  Copyright (c) 1999-2001 Max Kellermann (max.kellermann@epost.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: EditorFileChooser.java,v 1.5 2002/03/14 19:05:03 max Exp $
 *
 */

package org.leanedit;

import java.io.File;
import java.util.ResourceBundle;
import javax.swing.JFileChooser;
import javax.swing.filechooser.FileFilter;

/**
 * This is a JFileChooser that lets the user choose an
 * XML file or any other file that LeanEdit can import.
 */
public class EditorFileChooser extends JFileChooser {
    /*static**************************/
    protected static final ResourceBundle resources
        = ResourceBundle.getBundle("org.leanedit.EditorFrame");
    /*Filters******************************************/
    public static class XmlFileFilter extends FileFilter {
        /*FileFilter******/
        public boolean accept(File f) {
            if (f!=null) {
                if (f.isDirectory()) return true;
                String name = f.getName();
                return name.endsWith(".xml") || 
                    name.endsWith(".wml");
            }
            return false;
        }
        public String getDescription() {
            return EditorFileChooser.resources.getString("filter.xml.label")+" (*.xml)";
        }
    }
    public static class ZipFileFilter extends FileFilter {
        /*FileFilter*******/
        public boolean accept(File f) {
            if (f!=null) {
                if (f.isDirectory()) return true;
                String name = f.getName();
                return name.endsWith(".zip");
            }
            return false;
        }
        public String getDescription() {
            return EditorFileChooser.resources.getString("filter.zip.label")+" (*.zip)";
        }
    }
    /*Constructor****************************/
    public EditorFileChooser() {
        FileFilter xmlFilter = new XmlFileFilter();
        addChoosableFileFilter(xmlFilter);
        addChoosableFileFilter(new ZipFileFilter());
        setFileFilter(xmlFilter);
    }
}
