/*
 *
 *  LeanEdit
 *  Copyright (c) 1999-2001 Max Kellermann (max.kellermann@epost.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: EditorFrame.java,v 1.9 2001/10/12 20:03:13 max Exp $
 *
 */

package org.leanedit;

import java.awt.Frame;
import java.beans.PropertyChangeListener;
import make.swing.panel.InterfacePanel;
import make.swing.panel.PanelContent;
import org.leanedit.plugins.EditorComponent;
import org.leanedit.plugins.CompositeComponent;

/**
 * EditorFrame is an object displaying a DOM tree using
 * an org.leanedit.plugins.EditorComponent. 
 * Actually, in most cases, it is not a frame, 
 * but rather a make.swing.PanelContent.
 */
public interface EditorFrame extends PanelContent {
    /*CONST***********************************/
    // property names
    String VIEW = "view";
    String COMPONENT = "component";
    String DOCUMENT = "document";
    // component names
    String COMPONENT_ROOT = "cpn_root";
    /*EditorFrame*****************************/
    /**
     * returns the session object this frame belongs to
     */
    EditorSession getSession();
    /**
     * returns the InterfacePanel object this frame is displayed in
     */
    InterfacePanel getInterfacePanel();
    /**
     * returns the view that is used to display the document
     */
    EditorDocumentView getView();
    /**
     * registers the specified named component 
     */
    void registerComponent(String key, CompositeComponent ec);
    /**
     * unregisters the specified named component 
     */
    void unregisterComponent(String key, CompositeComponent ec);
    /**
     * returns all components that are known under a specified name
     */
    EditorComponent[] getComponents(String key);
    /**
     * returns the document that is currently being displayed.
     * If the user made changes which cannot be committed into
     * the document, the user gets an error message on the screen
     * and this method returns null.
     */
    EditorDocument getDocument();
    /**
     * asks the object to notify the listener whenever a property changes
     */
    void addPropertyChangeListener(PropertyChangeListener l);
    /**
     * removes the specified listener from the list
     */
    void removePropertyChangeListener(PropertyChangeListener l);
    /**
     * Closes this frame.
     * @param interactive if true, the user is asked about
     *                    saving data etc.
     * @return true if the session was closed
     */
    boolean close(boolean interactive);
}
