/*
 *
 *  LeanEdit
 *  Copyright (c) 1999-2001 Max Kellermann (max.kellermann@epost.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: EditorSession.java,v 1.8 2001/10/12 20:03:13 max Exp $
 *
 */

package org.leanedit;

import java.util.Enumeration;
import java.io.IOException;
import javax.swing.Action;
import org.xml.sax.SAXException;
import make.swing.SwingSession;
import make.swing.panel.InterfacePanel;
import make.datamodel.embedded.EmbeddedManager;

/**
 * An EditorSession object describes a session which the user
 * is working on. Currently, there is only one session in
 * one instance of LeanEdit, but this may change later.
 */
public interface EditorSession extends SwingSession {
    /**
     * Returns all existing frames.
     * A frame is an PanelContent instance,
     * may also be an EditorFrame instance (which extends PanelContent).
     */
    Enumeration getFrames();
    /**
     * returns the application object
     */
    EditorApplication getApplication();
    /**
     * opens the specified document in a new window.
     * If no InterfacePanel is specified, a new one is created.
     */
    EditorFrame open(EmbeddedManager em, InterfacePanel ip) 
	throws IOException, SAXException;
    /**
     * Adds the specified actions to the session.
     */
    void addActions(Enumeration enum);
    /**
     * Removes the specified actions from the session.
     */
    void removeActions(Enumeration enum);
    /**
     * Closes this session and all frames which belong to it.
     * @param interactive if true, the user is asked about
     *                    saving data etc.
     * @return true if the session was closed
     */
    boolean close(boolean interactive);
}
