/*
 *
 *  LeanEdit
 *  Copyright (c) 1999-2001 Max Kellermann (max.kellermann@epost.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: SubEditorFrame.java,v 1.8 2001/10/12 20:03:13 max Exp $
 *
 */

package org.leanedit;

import java.util.Enumeration;
import make.swing.bt.FrameDescription;
import make.swing.panel.InterfacePanel;
import org.leanedit.module.EditorModule;
import org.leanedit.module.ModuleManager;

/**
 * SubEditorFrame is an EditorFrame implementation that 
 * acts as a child for another EditorFrame object.
 * e.g. this may be a dialog that is opened by its parent frame.
 */
public class SubEditorFrame extends EditorPanel {
    /*properties******************************/
    private EditorFrame original;
    private FrameDescription frameDescription;
    /*Constructor*****************************/
    public SubEditorFrame(EditorFrame _or, InterfacePanel _ip,
			  FrameDescription _fd) {
	super(_or.getSession(), _ip);
	original = _or;
	frameDescription = _fd;
	if (frameDescription!=null)
	    getInterfacePanel().addFrameDescription(frameDescription);
	ModuleManager moduleManager = getApplication().getModuleManager();
	for (Enumeration enum=moduleManager.getModuleNames(); enum.hasMoreElements();) {
	    String name = (String)enum.nextElement();
	    EditorModule module = moduleManager.getModule(name);
	    module.install(this);
	}
    }
    /*SubEditorFrame**********************/
    /**
     * returns the parent EditorFrame object of this frame
     */
    public EditorFrame getParent() {
	return original;
    }
    /*PanelContent**************************/
    public void panelClosed() {
	ModuleManager moduleManager = getApplication().getModuleManager();
	for (Enumeration enum=moduleManager.getModuleNames(); enum.hasMoreElements();) {
	    String name = (String)enum.nextElement();
	    EditorModule module = moduleManager.getModule(name);
	    module.uninstall(this);
	}
	if (frameDescription!=null)
	    getInterfacePanel().removeFrameDescription(frameDescription);
	super.panelClosed();
    }
}
