/*
 *
 *  LeanEdit
 *  Copyright (c) 1999-2001 Max Kellermann (max.kellermann@epost.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: SessionImpl.java,v 1.5 2001/12/22 12:14:58 max Exp $
 *
 */

package org.leanedit.applet;

import java.util.Vector;
import java.io.IOException;
import org.xml.sax.SAXException;
import make.datamodel.embedded.EmbeddedManager;
import make.swing.bt.FrameDescription;
import make.swing.bt.FrameDescriptionCache;
import make.swing.progress.ProgressIndicatorFactory;
import make.swing.progress.FrameProgressIndicatorFactory;
import make.swing.panel.InterfacePanel;
import make.swing.panel.FrameInterfacePanel;
import make.swing.panel.PanelContent;
import org.leanedit.EditorApplication;
import org.leanedit.AbstractEditorSession;
import org.leanedit.EditorFrame;
import org.leanedit.TopEditorFrame;

public class SessionImpl extends AbstractEditorSession {
    /*properties*************************/
    protected FrameDescription frameDescription;
    protected ProgressIndicatorFactory progressIndicatorFactory
        = new FrameProgressIndicatorFactory();
    private Vector installedFrames = new Vector();
    /*Constructor*********************************/
    public SessionImpl(EditorApplication _app) {
        super(_app);
        frameDescription = FrameDescriptionCache.getFrameDescription
            (SessionImpl.class, "frame.xml", getLocale());
    }
    /*AbstractEditorSession public************************************/
    public EditorFrame createNewFrame(InterfacePanel ip) {
        EditorFrame frame = new TopEditorFrame(this, ip);
        ip.setContent(frame);
        if (frameDescription!=null)
            ip.addFrameDescription(frameDescription);
        installedFrames.addElement(frame);
        return frame;
    }
    /*AbstractEditorSession protected**********************************/
    protected void removeFrame(PanelContent frame) {
        // remove FrameDescription
        if (installedFrames.contains(frame)) {
            EditorFrame editorFrame = (EditorFrame)frame;
            InterfacePanel interfacePanel = editorFrame.getInterfacePanel();
            if (frameDescription!=null)
                interfacePanel.removeFrameDescription(frameDescription);
            installedFrames.remove(frame);
        }
        // call super class
        super.removeFrame(frame);
    }
    /*SwingSession*************************************/
    public InterfacePanel createInterfacePanel() {
        return wrapInterfacePanel(new FrameInterfacePanel());
    }
    public ProgressIndicatorFactory getProgressIndicatorFactory() {
        return progressIndicatorFactory;
    }
    /*EditorSession**********************************/
    public EditorFrame open(EmbeddedManager em, InterfacePanel ip)
        throws IOException, SAXException {
        if (ip==null)
            ip = createInterfacePanel();
        EditorFrame frame = createNewFrame(ip);
        ((TopEditorFrame)frame).readFrom(em);
        ip.setVisible(true);
        return frame;
    }
}
