/*
 *
 *  LeanEdit
 *  Copyright (c) 1999-2001 Max Kellermann (max.kellermann@epost.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: ControlPanel.java,v 1.4 2001/10/12 20:03:13 max Exp $
 *
 */

package org.leanedit.control;

import java.util.Vector;
import java.util.Enumeration;
import java.text.ParseException;
import java.awt.Component;
import java.awt.event.ActionEvent;
import javax.swing.Action;
import javax.swing.AbstractAction;
import javax.swing.JTabbedPane;
import javax.swing.JOptionPane;
import make.util.ErrorUtil;
import make.swing.bt.FrameDescription;
import make.swing.panel.PanelContent;
import make.swing.panel.InterfacePanel;
import org.leanedit.EditorApplication;
import org.leanedit.EditorSession;

/**
 * ControlPanel is the configuration interface
 * for LeanEdit.
 */
public class ControlPanel implements PanelContent {
    protected static final FrameDescription frameDescription
	= FrameDescription.okCancelDescription;
    /*properties***************************/
    protected InterfacePanel interfacePanel;
    protected EditorApplication application;
    protected EditorSession session;
    protected JTabbedPane tabbedPane = new JTabbedPane();
    protected Vector tabs = new Vector();
    /*Constructor*****************************/
    public ControlPanel(EditorSession _session, InterfacePanel _ip) {
	session = _session;
	interfacePanel = _ip;
	application = session.getApplication();
	tabs.addElement(new ModuleTab(application.getModuleManager()));
	tabs.addElement(new SessionTab(application));
	tabs.addElement(new FrameTab(session));
	for (Enumeration enum=tabs.elements(); enum.hasMoreElements();) {
	    ControlTab tab = (ControlTab)enum.nextElement();
	    tabbedPane.addTab(tab.getDisplayName(), tab.getComponent());
	}
	interfacePanel.setTitle("Control panel");
	interfacePanel.addFrameDescription(frameDescription);
	interfacePanel.getButtonManager().installActions(actions);
    }
    /*PanelContent******************************/
    public Component getComponent() {
	return tabbedPane;
    }
    public boolean panelClosing() {
	return true;
    }
    public void panelClosed() {
	for (Enumeration enum=tabs.elements(); enum.hasMoreElements();) {
	    ControlTab tab = (ControlTab)enum.nextElement();
	    tab.dispose();
	}
	tabs.removeAllElements();
	tabs = null;
	interfacePanel.removeFrameDescription(frameDescription);
	interfacePanel.getButtonManager().uninstallActions(actions);
    }
    /*Actions***********************************/
    protected Action[] actions = {
	new OkAction(),
	new CancelAction(),
    };
    class OkAction extends AbstractAction {
	public OkAction() {
	    super("ok");
	}
	public void actionPerformed(ActionEvent evt) {
	    try {
		for (Enumeration enum=tabs.elements(); enum.hasMoreElements();) {
		    ControlTab tab = (ControlTab)enum.nextElement();
		    tab.apply();
		}
		interfacePanel.close();
	    } catch (ParseException e) {
		JOptionPane.showMessageDialog
		    (interfacePanel.getFrame(), ErrorUtil.format(e),
		     "Error", JOptionPane.ERROR_MESSAGE, null);
	    }
	}
    }
    class CancelAction extends AbstractAction {
	public CancelAction() {
	    super("cancel");
	}
	public void actionPerformed(ActionEvent evt) {
	    interfacePanel.close();
	}
    }
}
