/*
 *
 *  LeanEdit
 *  Copyright (c) 1999-2001 Max Kellermann (max.kellermann@epost.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: Converter.java,v 1.8 2001/09/20 20:20:53 max Exp $
 *
 */

package org.leanedit.convert;

import java.util.Hashtable;
import org.w3c.dom.Document;
import make.mime.Message;
import make.xml.DocumentType;
import make.datamodel.embedded.EmbeddedManager;

/**
 * A Converter is an object whose job is to convert an XML document
 * into a MIME message, e.g. a PDF file, LaTeX or similar.
 * Its job is also to do the opposite: read an XML document from a MIME message.
 * This can be used to export or import files.
 *
 * FIXME: this API needs more parameters about user configuration and schema 
 *        selection etc.
 */
public interface Converter {
    // info about me

    /**
     * Returns true if this object can be used to import documents
     */
    boolean importPossible();

    /**
     * Returns true if this object can be used to export documents
     */
    boolean exportPossible();

    // identify messages

    /**
     * Returns true if this object recognizes that the specified
     * system identifier can be imported or exported here.
     */
    boolean identify(String sysId);
    /**
     * Returns true if this objectrecognizes that the specified
     * message can be imported or exported here.
     */
    boolean identify(Message msg);

    /**
     * Returns true if this object can import the specified MIME message.
     * This is quite more than just returning true in identify.
     * This method may not load the actual data, but it may look at
     * the header values.
     */
    boolean importPossible(Message msg);

    /**
     * Returns true if this object can convert the specified MIME message
     * to the specified schema.
     * This method may not load the actual data, but it may look at
     * the header values.
     */
    boolean importPossible(Message msg, DocumentType schema);

    /**
     * Returns true if this object can export a document of the
     * specified schema.
     */
    boolean exportPossible(DocumentType schema);

    // the actual job

    /**
     * Reads a document from the specified MIME message.
     * @param src the message which shall be imported
     * @param schema the schema to which the message shall be imported
     * @param config the configuration hashtable.
     *               This method may modify this configuration to include hints
     *               which will later be passed to exportDocument.
     * @param dest the EmbeddedManager the imported document will be saved into
     */
    void importDocument(Message src, DocumentType schema,
			Hashtable config, EmbeddedManager dest)
	throws ConverterException;

    /**
     * Converts a document to a MIME message.
     * @param doc the DOM document which will be exported
     * @param config the configuration hashtable that was built in importDocument
     */
    Message exportDocument(EmbeddedManager src, DocumentType schema,
			   Hashtable config)
	throws ConverterException;
}
