/*
 *
 *  LeanEdit
 *  Copyright (c) 1999-2001 Max Kellermann (max.kellermann@epost.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: ConverterChooser.java,v 1.4 2001/10/12 20:03:13 max Exp $
 *
 */

package org.leanedit.convert;

import java.awt.Frame;
import java.awt.Component;
import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.Iterator;
import java.util.Vector;
import java.util.Hashtable;
import java.util.ResourceBundle;
import javax.swing.Action;
import javax.swing.AbstractAction;
import javax.swing.JList;
import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.JPanel;
import javax.swing.ListCellRenderer;
import javax.swing.JOptionPane;
import make.util.ErrorUtil;
import make.swing.bt.ButtonDescription;
import make.swing.bt.FrameDescription;
import make.swing.panel.PanelContent;
import make.swing.panel.InterfacePanel;
import make.swing.panel.DialogInterfacePanel;

public class ConverterChooser 
    implements PanelContent, ListCellRenderer {
    /*static******************************/
    private static final ResourceBundle resources
	= ResourceBundle.getBundle("make.swing.Dialog");
    protected static final ButtonDescription[] buttons = {
	new ButtonDescription("configure", "Configure..."),
	new ButtonDescription("ok", resources.getString("ok.label")),
	new ButtonDescription("close-frame", resources.getString("cancel.label")),
    };
    public static final FrameDescription frameDescription;
    static {
	frameDescription = new FrameDescription("Converter chooser");
	frameDescription.setButtonBox(buttons);
    }
    /*properties*************************/
    protected InterfacePanel interfacePanel;
    protected JPanel panel = new JPanel(new BorderLayout(), false);
    protected JList list;
    protected Hashtable config;
    protected Converter converter;
    protected ListCellRenderer oldRenderer;
    /*Constructor************************/
    protected ConverterChooser(InterfacePanel _ip,
			       Iterator converters,
			       Hashtable _config) {
	interfacePanel = _ip;
	config = _config;
	Vector vc = new Vector();
	while (converters.hasNext()) {
	    vc.addElement(converters.next());
	}
	panel.add("North", new JLabel("Please choose a converter from the list:"));
	list = new JList(vc);
	oldRenderer = list.getCellRenderer();
	list.setCellRenderer(this);
	list.addMouseListener(new MouseAdapter() {
		public void mouseClicked(MouseEvent evt) {
		    if (evt.getClickCount()==2)
			confirm((ConverterLoader)list.getSelectedValue());
		}
	    });
	panel.add("Center", new JScrollPane(list));
	interfacePanel.getButtonManager().installActions(actions);
	interfacePanel.addFrameDescription(frameDescription);
	interfacePanel.setContent(this);
    }
    public Converter getConverter() {
	return converter;
    }
    public boolean confirm(ConverterLoader loader) {
	if (loader==null)
	    return false;
	try {
	    converter = loader.loadConverter();
	} catch (Exception e) {
	    JOptionPane.showMessageDialog
		(interfacePanel.getFrame(), ErrorUtil.format(e),
		 "Converter chooser",
		 JOptionPane.ERROR_MESSAGE, null);
	    return false;
	}
	if (converter==null)
	    return false;
	interfacePanel.close();
	return true;
    }
    /*PanelContent**********************/
    public Component getComponent() {
	return panel;
    }
    public boolean panelClosing() {
	return true;
    }
    public void panelClosed() {
	interfacePanel.removeFrameDescription(frameDescription);
	interfacePanel.getButtonManager().uninstallActions(actions);
    }
    /*ListCellRenderer*********************/
    public Component getListCellRendererComponent(JList list,
						  Object value,
						  int index,
						  boolean isSelected,
						  boolean cellHasFocus) {
	value = ((ConverterLoader)value).getDisplayName();
	return oldRenderer.getListCellRendererComponent
	    (list, value, index, isSelected, cellHasFocus);
    }
    /*Actions**************************/
    protected final Action[] actions = {
	new OkAction(),
    };
    class OkAction extends AbstractAction {
	OkAction() { super("ok"); }
        public void actionPerformed(ActionEvent evt) {
	    confirm((ConverterLoader)list.getSelectedValue());
	}
    }
    /*Static******************************/
    public static Converter chooseConverter(Frame frame,
					    Iterator converters,
					    Hashtable config) {
	InterfacePanel ip = new DialogInterfacePanel(frame);
	ConverterChooser dlg = new ConverterChooser
	    (ip, converters, config);
	ip.setVisible(true);
	return dlg.getConverter();
    }
}
