/*
 *
 *  LeanEdit
 *  Copyright (c) 1999-2001 Max Kellermann (max.kellermann@epost.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: ConverterLoader.java,v 1.2 2001/09/20 20:20:53 max Exp $
 *
 */

package org.leanedit.convert;

/**
 * This class is the loader object for a Converter instance.
 * This way, only converters which are actually needed are
 * loaded from hard disk.
 */
public final class ConverterLoader {
    /*properties******************/
    protected String displayName, description;
    protected ClassLoader classLoader;
    protected String className, dialogClassName;
    protected Converter instance;
    protected ConverterDialog dialogInstance;
    /*Constructor********************/
    public ConverterLoader(String _dn, String _desc,
			   ClassLoader _cl, String _cn) {
	displayName = _dn;
	description = _desc;
	classLoader = _cl; className = _cn;
    }
    public ConverterLoader(String _dn, String _desc,
			   Converter _instance) {
	displayName = _dn;
	description = _desc;
	instance = _instance;
    }
    public String toString() {
	return "[Converter class="+className+"]";
    }
    /*ConverterLoader**********************/
    /**
     * Returns the name of this converter presented to the user.
     */
    public String getDisplayName() {
	return displayName;
    }
    /**
     * Returns a short description.
     */
    public String getDescription() {
	return description;
    }
    /**
     * Returns the shared Converter instance
     * or null if this converter was not initialized before.
     */
    public Converter getConverter() {
	return instance;
    }
    /**
     * Loads the converter class and instantiates a shared instance.
     */
    public Converter loadConverter() 
	throws ClassNotFoundException, InstantiationException,
	       IllegalAccessException {
	// first check cache
	if (instance!=null)
	    return instance;
	// load the class
	Class cls = classLoader==null
	    ? Class.forName(className)
	    : classLoader.loadClass(className);

	// create a new instance
	instance = (Converter)cls.newInstance();
	return instance;
    }
    public boolean hasConverterDialog() {
	return dialogClassName!=null;
    }
    public ConverterDialog loadConverterDialog()
	throws ClassNotFoundException, InstantiationException,
	       IllegalAccessException {
	if (dialogClassName==null)
	    return null;
	// first check cache
	if (dialogInstance!=null)
	    return dialogInstance;
	// load the class
	Class cls = classLoader==null
	    ? Class.forName(dialogClassName)
	    : classLoader.loadClass(dialogClassName);

	// create a new instance
	dialogInstance = (ConverterDialog)cls.newInstance();
	return dialogInstance;
    }
}
