/*
 *
 *  LeanEdit
 *  Copyright (c) 1999-2001 Max Kellermann (max.kellermann@epost.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: XMLConverter.java,v 1.3 2001/09/20 20:20:53 max Exp $
 *
 */

package org.leanedit.convert;

import java.util.Hashtable;
import java.io.IOException;
import java.io.ByteArrayOutputStream;
import org.w3c.dom.Node;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;
import make.mime.Message;
import make.mime.ArrayMessage;
import make.mime.MIMEUtil;
import make.xml.DOMUtil;
import make.xml.DOMWriter;
import make.xml.DocumentType;
import make.datamodel.embedded.EmbeddedManager;

/**
 * This is the simplest Converter class:
 * it uses make.xml.DOMWriter to write a document to a stream.
 */
public class XMLConverter implements Converter {
    /*Converter******************************/
    public boolean importPossible() {
	return true;
    }
    public boolean exportPossible() {
	return true;
    }
    public boolean identify(String sysId) {
	return sysId.toLowerCase().endsWith(".xml");
    }
    public boolean identify(Message msg) {
	return MIMEUtil.isMediaType(msg, "unknown") ||
	    MIMEUtil.isMediaType(msg, "text");
    }
    public boolean importPossible(Message msg) {
	return identify(msg);
    }
    public boolean importPossible(Message msg, DocumentType schema) {
	// always false - we don't translate to another schema
	return false;
    }
    public boolean exportPossible(DocumentType schema) {
	return true;
    }
    public void importDocument(Message src, DocumentType schema,
			       Hashtable config, EmbeddedManager dest)
	throws ConverterException {
	try {
	    Document doc = DOMUtil.parse(src.getInputStream());
	    if (doc==null || doc.getDocumentElement()==null)
		throw new ConverterException("The file is empty.");
	    dest.replaceDocument(doc.getDocumentElement());
	} catch (IOException e) {
	    throw new ConverterException(e);
	} catch (SAXException e) {
	    throw new ConverterException(e);
	}
    }
    public Message exportDocument(EmbeddedManager src, DocumentType schema,
				  Hashtable config)
	throws ConverterException {
	try {
	    ByteArrayOutputStream baos = new ByteArrayOutputStream();
	    Node node = src.getDocument();
	    if (node==null)
		throw new ConverterException("The document is empty.");
	    // FIXME: change encoding
	    DOMWriter dw = new DOMWriter(baos, "ISO-8859-1");
	    dw.print(node.getOwnerDocument());
	    dw.flush();
	    byte[] data = baos.toByteArray();
	    return new ArrayMessage(data, "text/xml");
	} catch (IOException e) {
	    throw new ConverterException(e);
	} catch (SAXException e) {
	    throw new ConverterException(e);
	}
    }
}
