/*
 *
 *  LeanEdit
 *  Copyright (c) 1999-2001 Max Kellermann (max.kellermann@epost.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: TransformEmbeddedManager.java,v 1.7 2001/09/20 20:20:53 max Exp $
 *
 */

package org.leanedit.embedded;

import java.io.IOException;
import java.io.InputStream;
import java.util.Hashtable;
import org.w3c.dom.Node;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.stream.StreamSource;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.dom.DOMResult;
import make.xml.DOMUtil;
import make.xml.SchemaManager;
import make.xml.DocumentType;
import make.datamodel.embedded.EmbeddedManager;
import make.datamodel.embedded.ProxyEmbeddedManager;

public class TransformEmbeddedManager 
    implements EmbeddedManager {
    /*properties******************************/
    protected TransformerFactory tFactory = TransformerFactory.newInstance();
    protected SchemaManager schemaManager;
    protected EmbeddedManager original;
    protected String storeSchema, displaySchema;
    /*Constructor*****************************/
    public TransformEmbeddedManager(SchemaManager _schemaManager,
				    EmbeddedManager _org, 
				    String _sfmt, String _dfmt) { 
	schemaManager = _schemaManager;
	original = _org; 
	storeSchema = _sfmt; displaySchema = _dfmt;
    }
    public String toString() { return "[XSL:"+original.toString()+"]"; }
    /*Internal*********************************/
    protected InputStream findExportStyle(String src, String dest) {
	DocumentType type = schemaManager.getSchema(src);
	if (type==null) return null;
	return type.getConfigPath().open("export_"+dest+".xsl");
    }
    protected InputStream findImportStyle(String src, String dest) {
	DocumentType type = schemaManager.getSchema(dest);
	if (type==null) return null;
	return type.getConfigPath().open("import_"+src+".xsl");
    }
    protected InputStream findStyle(String src, String dest) {
	InputStream is;
	is = findExportStyle(src, dest);
	if (is==null)
	    return findImportStyle(src, dest);
	else
	    return is;
    }
    /*ProxyEmbeddedManager*************************/
    public EmbeddedManager getOriginal() {
	return original;
    }
    /*EmbeddedManager***************************/
    public String[] getNames() throws IOException {
	return original.getNames();
    }
    public Hashtable getSizes() throws IOException {
	return original.getSizes();
    }
    public boolean contains(String name) { 
	return original.contains(name);
    }
    public void replaceEntry(String name, InputStream data)
	throws IOException {
	original.replaceEntry(name, data);
    }
    public InputStream getEntry(String name) throws IOException {
	return original.getEntry(name);
    }
    public void replaceDocument(Node n) throws IOException {
	if (storeSchema==null)
	    throw new InternalError();
	if (n!=null) {
	    // source
	    Document src = n.getOwnerDocument();
	    // style
	    String schemaName = schemaManager.getSchemaName(src);
	    InputStream xslIS = findStyle(schemaName, storeSchema);
	    if (xslIS==null)
		throw new IOException("Kein passender Exportfilter gefunden.");
	    // target
	    Document target = DOMUtil.createDocument();
	    // transform
	    try {
		Transformer transformer
		    = tFactory.newTransformer(new StreamSource(xslIS));
		transformer.transform(new DOMSource(src),
				      new DOMResult(target));
	    } catch (TransformerException e) {
		throw new IOException(e.getLocalizedMessage());
	    }
	    n = target.getDocumentElement();
	}
	original.replaceDocument(n);
    }
    public Node getDocument() throws IOException, SAXException {
	if (displaySchema==null)
	    throw new InternalError();
	// source
	Node n = original.getDocument();
	if (n==null)
	    return null;
	Document src = n.getOwnerDocument();
	// style
	String schemaName = schemaManager.getSchemaName(src);
	InputStream xslIS = findStyle(schemaName, storeSchema);
	if (xslIS==null)
	    throw new IOException("Kein passender Exportfilter gefunden.");
	// target
	Document target = DOMUtil.createDocument();
	// transform
	try {
	    Transformer transformer
		= tFactory.newTransformer(new StreamSource(xslIS));
	    transformer.transform(new DOMSource(src),
				  new DOMResult(target));
	} catch (TransformerException e) {
	    throw new IOException(e.getLocalizedMessage());
	}
	return target.getDocumentElement();
    }
    public boolean equals(Object o) { 
	if (o instanceof TransformEmbeddedManager)
	    return original.equals(((TransformEmbeddedManager)o).original);
	else
	    return original.equals(o);
    }
}
