/*
 *
 *  LeanEdit
 *  Copyright (c) 1999-2001 Max Kellermann (max.kellermann@epost.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: EmbeddedField.java,v 1.7 2001/10/18 14:50:41 max Exp $
 *
 */

package org.leanedit.field;

import java.util.ResourceBundle;
import java.io.File;
import java.io.FilenameFilter;
import java.io.IOException;
import java.io.InputStream;
import java.io.FileInputStream;
import java.io.OutputStream;
import java.io.FileOutputStream;
import java.awt.Component;
import java.awt.BorderLayout;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.text.ParseException;
import javax.swing.JPanel;
import javax.swing.JFileChooser;
import javax.swing.JButton;
import javax.swing.JOptionPane;
import javax.swing.ImageIcon;
import javax.swing.filechooser.FileFilter;
import make.datamodel.embedded.EmbeddedManager;
import make.io.Streams;

class EmbeddedField extends EmbeddedSelectField {
    /*static**********************************/
    protected static final ResourceBundle resources
	= ResourceBundle.getBundle("org.leanedit.field.EmbeddedField");
    protected static final ResourceBundle editorFrameResources
	= ResourceBundle.getBundle("org.leanedit.EditorFrame");
    protected static JFileChooser fileChooser;
    static {
	try {
	    fileChooser = new JFileChooser();
	    fileChooser.setFileFilter(new FileFilter() {
		    public boolean accept(File f) {
			if (f!=null) {
			    if (f.isDirectory()) 
				return true;
			    else if (f.isFile()) {
				String name = f.getName().toLowerCase();
				return name.endsWith(".jpg") || name.endsWith(".jpeg") ||
				    name.endsWith(".gif") || name.endsWith(".png");
			    }
			}
			return false;
		    }
		    public String getDescription() {
			return "Images (*.jpg, *.jpeg, *.gif, *.png)";
		    }
		});
	} catch (SecurityException e) {
	    // we're in an applet and access to hard disk is forbidden
	    fileChooser = null;
	}
    }
    /*properties*************/
    protected JButton open, save;
    /*Constructor*************/
    public EmbeddedField(EmbeddedManager _em, FilenameFilter _flt) {
	super(_em, _flt);
	if (fileChooser!=null) {
	    JPanel panel = new JPanel(new BorderLayout(), false);
	    panel.add("Center", super.getComponent());
	    JPanel p = new JPanel(new BorderLayout(), false);
	    open = new JButton
		(new ImageIcon(getClass().getResource("/org/leanedit/resources/open.gif")));
	    open.setActionCommand("open");
	    open.addActionListener(this);
	    p.add("West", open);
	    save = new JButton
		(new ImageIcon(getClass().getResource("/org/leanedit/resources/save.gif")));
	    save.setActionCommand("save");
	    p.add("East", save);
	    panel.add("East", p);
	    save.addActionListener(this);
	    setComponent(panel);
	}
    }
    public void errorMsg(String msg, String title) {
	JOptionPane.showMessageDialog(null, msg, title,
				      JOptionPane.ERROR_MESSAGE);
    }
    /*ActionListener*******************************/
    public void actionPerformed(ActionEvent evt) {
	// choose
	if (evt.getSource()==open) {
	    fileChooser.getUI().rescanCurrentDirectory(fileChooser);
	    final int chooseRet = fileChooser.showOpenDialog(null);
	    if (chooseRet!=fileChooser.APPROVE_OPTION)
		return;
	    File f = fileChooser.getSelectedFile();
	    if (!f.exists()) return;
	    if (f.isDirectory()) {
		errorMsg(resources.getString("error.is_directory"),
			 resources.getString("operation.open"));
		return;
	    }
	    if (!f.canRead()) {
		errorMsg(resources.getString("error.access_denied"),
			 resources.getString("operation.open"));
		return;
	    }
	    // copy
	    if (!embeddedManager.contains(f.getName())) //TODO: anderer Pfad?
		try {
		    InputStream fis = new FileInputStream(f);
		    embeddedManager.replaceEntry(f.getName(), fis);
		} catch (IOException e) {
		    errorMsg(resources.getString("error.error_reading_file")+
			     "\n"+e.getLocalizedMessage(),
			     resources.getString("operation.open"));
		    return;
		}
	    // embed
	    reloadList();
	    try {
		setValue(f.getName());
	    } catch (ParseException e) {
	    }
	} else if (evt.getSource()==save) {
	    fileChooser.getUI().rescanCurrentDirectory(fileChooser);
	    String name = null;
	    try {
		name = (String)getValue(); 
	    } catch (ParseException e) {
	    }
	    if (name==null || !embeddedManager.contains(name)) {
		errorMsg(resources.getString("error.choose_file"),
			 resources.getString("operation.save"));
		return;
	    }
	    final int chooseRet = fileChooser.showSaveDialog(null);
	    if (chooseRet!=fileChooser.APPROVE_OPTION)
		return;
	    File f = fileChooser.getSelectedFile();
	    if (f.exists()) {
		if (JOptionPane.showConfirmDialog
		    (null,
		     editorFrameResources.getString("ask.overwrite"),
		     resources.getString("operation.save"),
		     JOptionPane.YES_NO_OPTION)!=JOptionPane.YES_OPTION)
		    return;
		if (f.isDirectory()) {
		    errorMsg(resources.getString("error.is_directory"),
			     resources.getString("operation.save"));
		    return;
		}
		if (!f.canWrite()) {
		    errorMsg(resources.getString("error.acess_denied"),
			     resources.getString("operation.save"));
		    return;
		}
	    }
	    // copy
	    try {
		InputStream is = embeddedManager.getEntry(name);
		if (is==null) return;
		OutputStream os = new FileOutputStream(f);
		Streams.copy(is, os);
		is.close(); os.close();
	    } catch (IOException e) {
		errorMsg(resources.getString("error.copy_file")+
			 "\n"+e.getLocalizedMessage(), 
			 resources.getString("operation.save"));
	    }
	} else
	    super.actionPerformed(evt);
    }
}
