/*
 *
 *  LeanEdit
 *  Copyright (c) 1999-2001 Max Kellermann (max.kellermann@epost.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: EmbeddedSelectField.java,v 1.2 2001/09/20 20:20:53 max Exp $
 *
 */

package org.leanedit.field;

import java.io.FilenameFilter;
import java.io.IOException;
import java.util.Vector;
import java.awt.Component;
import java.text.ParseException;
import javax.swing.AbstractListModel;
import javax.swing.JList;
import javax.swing.JComboBox;
import javax.swing.ComboBoxModel;
import javax.swing.AbstractListModel;
import javax.swing.ListCellRenderer;
import make.datamodel.embedded.EmbeddedManager;
import make.swing.field.DefaultField;

public class EmbeddedSelectField extends DefaultField
    implements ListCellRenderer {
    class Model extends AbstractListModel implements ComboBoxModel {
	public int getSize() {
	    return selectedItem==null || selectedItem==nullObject || selectedItemInFile
		?names.length+1
		:names.length+2;
	}	    
	public Object getElementAt(int index) {
	    if (index==0)
		return nullObject;
	    index--;
	    return index==names.length
		?selectedItem
		:names[index];
	}
	public Object getSelectedItem() {
	    return selectedItem;
	}
	public void setSelectedItem(Object anItem) {
	    if (selectedItem!=null && selectedItem!=nullObject && !selectedItemInFile)
		fireIntervalRemoved(this, names.length+1, names.length+1);
	    selectedItem = null;
	    selectedItemInFile = true;
	    if (anItem==null || anItem==nullObject)
		selectedItem = nullObject;
	    else {
		for (int z=0; z<names.length; z++)
		    if (names[z].equals(anItem)) {
			selectedItem = names[z];
			break;
		    }
		if (selectedItem==null) {
		    selectedItem = anItem;
		    selectedItemInFile = false;
		    fireIntervalAdded(this, names.length+1, names.length+1);
		}
	    }
	    fireContentsChanged(this, -1, -1);
	}
    }
    /***properties*************/
    protected final Object nullObject = this;
    protected FilenameFilter nameFilter;
    protected EmbeddedManager embeddedManager;
    protected JComboBox field;
    protected ListCellRenderer renderer;
    protected Object selectedItem;
    protected boolean selectedItemInFile = true;
    protected String[] names;
    protected Model model = new Model();
    /***Constructor*************/
    public EmbeddedSelectField(EmbeddedManager _em, FilenameFilter _flt) {
	embeddedManager = _em; nameFilter = _flt;
	reloadList();
	field = new JComboBox(model);
	field.addActionListener(this);
	renderer = field.getRenderer();
	field.setRenderer(this);
	setComponent(field);
    }
    public void reloadList() {
	try {
	    String[] n = embeddedManager.getNames();
	    Vector vc = new Vector(n.length);
	    for (int z=0; z<n.length; z++)
		if (!n[z].equals("_dummy") && 
		    (nameFilter==null ||
		     nameFilter.accept(null, n[z])))
		    vc.addElement(n[z]);
	    names = new String[vc.size()];
	    for (int z=0; z<names.length; z++)
		names[z] = (String)vc.elementAt(z);
	} catch (IOException e) {
	    names = new String[0];
	}
    }
    /***Slot*********************************/
    public void setValue(Object v) throws ParseException {
	field.setSelectedItem(v);
    }
    public Object getValue() throws ParseException {
	return selectedItem==nullObject
	    ?null
	    :selectedItem;
    }
    public void setEnabled(boolean _e) {
        field.setEnabled(_e);
    }
    public boolean getEnabled() {
	return field.isEnabled();
    }
    /***ListCellRenderer*************************/
    public Component getListCellRendererComponent(JList _list, Object value,
						  int index,
						  boolean isSelected,
						  boolean cellHasFocus) {
	Object vDisplay;
	if (value==nullObject || value==null)
	    vDisplay = "[Leer]";
	else
	    vDisplay = value;
	return renderer.getListCellRendererComponent
	    (_list, vDisplay, index,
	     isSelected, cellHasFocus);
    }
}
