/*
 *
 *  LeanEdit
 *  Copyright (c) 1999-2001 Max Kellermann (max.kellermann@epost.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: PluginInputLink.java,v 1.9 2001/12/22 12:14:59 max Exp $
 *
 */

package org.leanedit.field;

import java.util.Locale;
import java.text.ParseException;
import java.awt.Component;
import javax.swing.JComponent;
import org.w3c.dom.Element;
import javax.xml.transform.TransformerException;
import make.swing.il.DefaultInputLink;
import make.swing.il.InputLinkFactory;
import make.xml.DOMUtil;
import org.leanedit.EditorDocument;
import org.leanedit.EditorFrame;
import org.leanedit.plugins.EditorComponent;
import org.leanedit.plugins.ComponentListener;
import org.leanedit.plugins.EditorPlugin;
import org.leanedit.plugins.ContainerComponent;
import org.leanedit.plugins.PluginManager;

public class PluginInputLink extends DefaultInputLink {
    /*properties*************************/
    protected Component component;
    protected EditorComponent editorComponent;
    protected EditorDocument document;
    /*Constructor************************/
    public PluginInputLink(Element vd, Locale locale,
                           EditorDocument _doc, EditorFrame _frame) 
        throws TransformerException {
        super(vd, locale);
        // path
        Element pathEl = DOMUtil.getUniqueTagNS(vd, InputLinkFactory.XML_NAMESPACE, "path");
        String path = DOMUtil.getElementString(pathEl, false);
        String pathName = DOMUtil.getLocalizedElementParameterNS
            (pathEl, InputLinkFactory.XML_NAMESPACE, "name",
             locale, false);
        EditorDocument[] docs = _doc.followPath(path, pathName, true);
        document = docs[0];
        // plugin
        String pluginName = DOMUtil.getElementStringNS(vd, InputLinkFactory.XML_NAMESPACE, "plugin");
        PluginManager pluginManager
            = _frame.getSession().getApplication().getPluginManager();
        EditorPlugin ep = pluginManager.loadPlugin(pluginName);
        editorComponent = ep.createComponent(_frame);
        editorComponent.setDocument(document);
        component = editorComponent.getComponent();
        // listener
        EditorComponent[] root = _frame.getComponents(EditorFrame.COMPONENT_ROOT);
        if (root.length>0 &&
            root[0] instanceof ComponentListener)
            editorComponent.addComponentListener((ComponentListener)root[0]);
        // border
        if (component instanceof JComponent) {
            Element b = DOMUtil.getUniqueTag(vd, "border");
            if (b!=null)
                ((JComponent)component).setBorder(ContainerComponent.createBorder(b));
        }
        // scroll
        Element sp = DOMUtil.getUniqueTag(vd, "scrollpane");
        if (sp!=null)
            component = ContainerComponent.createScrollPane(sp, component);
    }
    /*Widget**************************/
    public void setEnabled(boolean _e) {
        editorComponent.setEnabled(_e);
    }
    public boolean getEnabled() {
        return editorComponent.getEnabled();
    }
    public Component getComponent() {
        return component;
    }
    public void dispose() {
        component = null;
        if (editorComponent!=null) {
            editorComponent.dispose();
            editorComponent = null;
        }
        document = null;
        super.dispose();
    }
    /*InputLink****************************/
    public void reload() {
        editorComponent.setDocument(document);
    }
    public void apply() throws ParseException {
        editorComponent.flushDocument();
    }
    public void checkInput() throws ParseException {
        editorComponent.flushDocument();
    }
}
