/*
 *
 *  LeanEdit
 *  Copyright (c) 1999-2001 Max Kellermann (max.kellermann@epost.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: RadioField.java,v 1.4 2001/09/20 20:20:53 max Exp $
 *
 */

package org.leanedit.field;

import java.awt.Component;
import java.awt.event.ActionListener;
import java.text.ParseException;
import javax.swing.JRadioButton;
import javax.swing.Box;
import javax.swing.ButtonGroup;
import javax.swing.ButtonModel;
import org.w3c.dom.Element;
import make.xml.DOMUtil;
import make.swing.field.DefaultField;
import make.swing.il.InputLinkFactory;

class RadioField extends DefaultField {
    /*properties*************/
    protected Box box = Box.createHorizontalBox();
    protected ButtonGroup buttonGroup = new ButtonGroup();
    protected JRadioButton[] bts;
    /*Constructor*************/
    public RadioField(Element _s) {
	Element[] rbs = DOMUtil.getChildElementsNS(_s, InputLinkFactory.XML_NAMESPACE, "radiobutton");
	bts = new JRadioButton[rbs.length];
	for (int z=0; z<rbs.length; z++) {
	    Element rb = rbs[z];
	    bts[z] = new JRadioButton(rb.getAttributeNS(InputLinkFactory.XML_NAMESPACE, "caption"));
	    bts[z].addActionListener(this);
	    box.add(bts[z]);
	    buttonGroup.add(bts[z]);
	    String curValue = rb.getAttributeNS(InputLinkFactory.XML_NAMESPACE, "value");
	    ButtonModel bm = bts[z].getModel();
	    bm.setActionCommand(curValue);
	}
	setComponent(box);
    }
    /*Slot*********************************/
    public void setValue(Object v) throws ParseException {
	String ac = v==null?null:v.toString();
	Component[] cps = box.getComponents();
	for (int z=0; z<cps.length; z++) {
	    JRadioButton bt = (JRadioButton)cps[z];
	    buttonGroup.setSelected(bt.getModel(), 
				    bt.getActionCommand().equals(ac));
	}
    }
    public Object getValue() throws ParseException {
	ButtonModel selection = buttonGroup.getSelection();
	return selection==null?null:selection.getActionCommand();
    }
    public void setEnabled(boolean _e) {
	for (int z=0; z<bts.length; z++)
	    bts[z].setEnabled(_e);
    }
    public boolean getEnabled() {
	return bts.length==0 || bts[0].isEnabled();
    }
}
