/*
 *
 *  LeanEdit
 *  Copyright (c) 1999-2001 Max Kellermann (max.kellermann@epost.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: EditorModule.java,v 1.2 2001/09/20 20:20:53 max Exp $
 *
 */

package org.leanedit.module;

import org.leanedit.EditorApplication;
import org.leanedit.EditorSession;
import org.leanedit.EditorFrame;

public interface EditorModule {
    //public EditorModule()
    /**
     * @return the name of the module
     */
    String getName();
    /**
     * This is called after the object is created.
     */
    void install(EditorApplication app);
    /**
     * Uninstalls from the application, this is called
     * before the module is unloaded or the application exits.
     */
    void uninstall(EditorApplication app);
    /**
     * This is called whenever a new session is created.
     * Tells the module to install itself in the new session.
     */
    void install(EditorSession sess);
    /**
     * Uninstalls from a closed session.
     */
    void uninstall(EditorSession sess);
    /**
     * This is called whenever a new frame is created.
     * Tells the module to install itself in the new frame.
     */
    void install(EditorFrame frm);
    /**
     * Uninstalls from a closed frame.
     */
    void uninstall(EditorFrame frm);
    /**
     * Opens a dialog or frame. The key is module specific, most
     * keys will be strings that can be passed as parameter to LeanEdit.
     */
    boolean open(EditorSession sess, Object key);
}
