/*
 *
 *  LeanEdit
 *  Copyright (c) 1999-2001 Max Kellermann (max.kellermann@epost.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: EditorComponent.java,v 1.6 2001/09/20 20:20:53 max Exp $
 *
 */

package org.leanedit.plugins;

import java.awt.Component;
import java.text.ParseException;
import java.beans.PropertyChangeListener;
import make.swing.Widget;
import org.leanedit.EditorDocument;

public interface EditorComponent extends Widget {
    /**
     * name of the property that contains a pointer to the AWT component
     */
    String COMPONENT = "component";
    String DOCUMENT = "document";
    /**
     * tells the component to load a specific document
     */
     void setDocument(EditorDocument doc);
    /**
     * tells the component to write all changes from memory 
     * into the DOM structure
     * @exception ParseException is thrown when some input 
     *     cannot be parsed, e.g. non-numeric input in a 
     *     numeric field. LeanEdit and not this component is responsible
     *     for displaying the error message.
     */
     void flushDocument() throws ParseException;
    /**
     * notifies the component that it has gained the keyboard focus.
     * Many components may want to install hotkeys here or activate 
     * menu buttons or toolbar buttons.
     */
     void focusGained();
    /**
     * notifies the component that it has lost the keyboard focus. 
     * This is called before the new focused component is notified.
     */
     void focusLost();
    /**
     * installs a PropertyChangeListener. Every component must know 
     * the property "component". More properties may be plugin specific.
     */
     void addPropertyChangeListener(PropertyChangeListener l);
    /**
     * remove a PropertyChangeListener
     */
     void removePropertyChangeListener(PropertyChangeListener l);
    /**
     * installs a ComponentListener. ComponentListeners get notification
     * about some events happening in an EditorComponent.
     */
     void addComponentListener(ComponentListener l);
    /**
     * uninstalls a ComponentListener
     */
     void removeComponentListener(ComponentListener l);
}
