/*
 *
 *  LeanEdit
 *  Copyright (c) 1999-2001 Max Kellermann (max.kellermann@epost.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: EditorStyleSet.java,v 1.3 2001/09/20 20:20:53 max Exp $
 *
 */

package org.leanedit.plugins;

import java.awt.Color;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.Locale;
import java.text.NumberFormat;
import org.w3c.dom.Element;
import org.w3c.dom.Text;
import make.xml.DOMUtil;

public abstract class EditorStyleSet {
    /***properties******************************/
    private Hashtable styles = new Hashtable();
    protected Hashtable newStyles = new Hashtable();
    static private NumberFormat numberFormat = NumberFormat.getInstance(Locale.US);
    /***Constructor****************************/
    public EditorStyleSet(Element _styles) {
	if (_styles!=null)
	    loadStyles(_styles);
    }
    /***Info*******************************/
    public Hashtable getStyle(String name) {
	Hashtable style = (Hashtable)styles.get(name);
	if (style==null) {
	    style = new Hashtable();
	    putStyle(name, style);
	    newStyles.put(name, style);
	}
	return style;
    }
    public Enumeration getStyleNames() {
	return styles.keys();
    }
    /***StyleSet*****************************/
    public void putStyle(String name, Hashtable s) {
	styles.put(name, s);
    }
    protected Hashtable createStyle(String attr1, Object val1) {
	Hashtable ht = new Hashtable(1);
	ht.put(attr1, val1);
	return ht;
    }
    protected Hashtable createStyle(String attr1, Object val1,
				    String attr2, Object val2) {
	Hashtable ht = new Hashtable(2);
	ht.put(attr1, val1);
	ht.put(attr2, val2);
	return ht;
    }
    protected Hashtable createStyle(String attr1, Object val1,
				  String attr2, Object val2,
				  String attr3, Object val3) {
	Hashtable ht = new Hashtable(2);
	ht.put(attr1, val1);
	ht.put(attr2, val2);
	ht.put(attr3, val3);
	return ht;
    }
    public void loadStyles(Element styleset) {
	Element[] ss = DOMUtil.getChildElements(styleset, "style");
	for (int z=0; z<ss.length; z++) {
	    String name = ss[z].getAttribute("name");
	    putStyle(name, loadStyle(ss[z]));
	}
    }
    protected String getElementString(Element el) {
	return ((Text)el.getFirstChild()).getData().trim();
    }
    protected Integer getElementInteger(Element el) {
	return new Integer(getElementString(el));
    }
    protected Color getElementColor(Element el) {
	try {
	    return new Color(numberFormat.parse(el.getAttribute("red")).floatValue(),
			     numberFormat.parse(el.getAttribute("green")).floatValue(),
			     numberFormat.parse(el.getAttribute("blue")).floatValue());
	} catch (Exception e) {
	    throw new IllegalArgumentException("could not parse color: "+e);
	}
    }
    /***EditorStyleSet*************************/
    public abstract Hashtable loadStyle(Element style);
}
