/*
 *
 *  LeanEdit
 *  Copyright (c) 1999-2001 Max Kellermann (max.kellermann@epost.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: PluginLoader.java,v 1.2 2001/09/20 20:20:53 max Exp $
 *
 */

package org.leanedit.plugins;

public class PluginLoader {
    /*properties******************/
    protected String name, displayName, description;
    protected ClassLoader classLoader;
    protected String className;
    protected EditorPlugin instance;
    /*Constructor********************/
    public PluginLoader(String _name, String _dn, String _desc,
			ClassLoader _cl, String _cn) {
	name = _name; displayName = _dn;
	description = _desc;
	classLoader = _cl; className = _cn;
    }
    public PluginLoader(String _name, String _dn, String _desc,
			EditorPlugin _instance) {
	name = _name; displayName = _dn;
	description = _desc;
	instance = _instance;
    }
    public String toString() {
	return "[PluginLoader name="+name+" class="+className+"]";
    }
    /*PluginLoader**********************/
    /**
     * Returns a machine-readable name consisting of
     * 7-bit alphanumeric characters including underscore.
     * This name must be globally unique.
     */
    public String getName() {
	return name;
    }
    /**
     * Returns the name of this plugin presented to the user.
     */
    public String getDisplayName() {
	return displayName;
    }
    /**
     * Returns a short description.
     */
    public String getDescription() {
	return description;
    }
    /**
     * Loads the plugin class and instantiates an instance.
     */
    public EditorPlugin loadPlugin() 
	throws ClassNotFoundException, InstantiationException,
	       IllegalAccessException {
	// first check cache
	if (instance!=null)
	    return instance;
	// load the class
	Class cls = classLoader==null
	    ? Class.forName(className)
	    : classLoader.loadClass(className);

	// create a new instance
	instance = (EditorPlugin)cls.newInstance();
	return instance;
    }
}
