/*
 *
 *  LeanEdit
 *  Copyright (c) 1999-2001 Max Kellermann (max.kellermann@epost.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: PluginManager.java,v 1.7 2001/09/20 20:20:53 max Exp $
 *
 */

package org.leanedit.plugins;

import java.util.Hashtable;
import java.util.Enumeration;
import make.util.Debug;

/**
 * A PluginManager manages a list of PluginLoader objects.
 */
public class PluginManager {
    /*properties**********************************/
    protected Hashtable plugins = new Hashtable();
    /*PluginManager*****************************/
    /**
     * Registers a new plugin.
     */
    public void registerPlugin(PluginLoader pluginLoader) {
	String name = pluginLoader.getName();
	Object oldValue = plugins.put(name, pluginLoader);
	if (Debug.DEBUG && oldValue!=null) {
	    plugins.put(name, oldValue);
	    System.err.println("ERROR: tried to overwrite plugin "+name);
	}
    }
    /**
     * Unregisters an existing plugin.
     */
    public void unregisterPlugin(PluginLoader pluginLoader) {
	String name = pluginLoader.getName();
	Object oldValue = plugins.remove(name);
	if (Debug.DEBUG && oldValue!=pluginLoader) {
	    plugins.put(name, oldValue);
	    System.err.println("ERROR: trying to unregister plugin name="+
			       name+" pluginLoader="+pluginLoader+
			       " but oldValue="+oldValue);
	}
    }
    /**
     * Returns a list containing the names of all registered plugins
     */
    public Enumeration getPluginNames() {
	return plugins.keys();
    }
    /**
     * Loads the specified plugin and returns the EditorPlugin object.
     * Returns null if the requested plugin cannot be loaded.
     */
    public EditorPlugin loadPlugin(String name) {
	PluginLoader pluginLoader = (PluginLoader)plugins.get(name);
	if (pluginLoader==null)
	    return null;
	try {
	    return pluginLoader.loadPlugin();
	} catch (ClassNotFoundException e) {
	    Debug.report(e, "plugin "+pluginLoader);
	    return null;
	} catch (InstantiationException e) {
	    Debug.report(e, "plugin "+pluginLoader);
	    return null;
	} catch (IllegalAccessException e) {
	    Debug.report(e, "plugin "+pluginLoader);
	    return null;
	} catch (ClassCastException e) {
	    Debug.report(e, "plugin "+pluginLoader);
	    return null;
	}
    }
}
