/*
 *
 *  LeanEdit
 *  Copyright (c) 1999-2001 Max Kellermann (max.kellermann@epost.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: ViewSelector.java,v 1.8 2001/10/12 20:03:13 max Exp $
 *
 */

package org.leanedit.plugins;

import java.util.Vector;
import java.util.Enumeration;
import java.awt.Frame;
import java.awt.Component;
import java.awt.event.ActionEvent;
import javax.swing.Action;
import javax.swing.AbstractAction;
import javax.swing.JList;
import javax.swing.JScrollPane;
import javax.swing.AbstractListModel;
import org.w3c.dom.Element;
import make.swing.bt.FrameDescription;
import make.swing.panel.InterfacePanel;
import make.swing.panel.DialogInterfacePanel;
import make.swing.panel.PanelContent;
import org.leanedit.EditorDocumentView;

public class ViewSelector implements PanelContent {
    /*static******************************/
    protected static final FrameDescription frameDescription
	= FrameDescription.okCancelDescription;
    /*properties********************************/
    protected InterfacePanel interfacePanel;
    protected Vector views;
    protected JList list;
    protected JScrollPane scrollPane;
    protected int selectedIndex = -1;
    /*Constructor********************************/
    public ViewSelector(InterfacePanel _ip, String title, String prompt, 
			Enumeration _v) {
	interfacePanel = _ip;
	interfacePanel.setTitle(title);
	interfacePanel.addFrameDescription(frameDescription);
	interfacePanel.getButtonManager().installActions(actions);
	views = new Vector();
	while (_v.hasMoreElements())
	    views.addElement(_v.nextElement());
	// list
	list = new JList(new AbstractListModel() {
	    public int getSize() { return views.size(); }
	    public Object getElementAt(int index) { 
		return ((EditorDocumentView)views.elementAt(index)).getTitle();
	    }
	});
	scrollPane = new JScrollPane(list);
	interfacePanel.setContent(this);
    }
    public EditorDocumentView getSelectedView() {
	return selectedIndex<0?null 
	    :(EditorDocumentView)views.elementAt(selectedIndex);
    }
    public static EditorDocumentView selectView(Frame _frame, String title, 
						String prompt, 
						Enumeration views) {
	InterfacePanel ip = new DialogInterfacePanel(_frame);
	ViewSelector ps = new ViewSelector(ip, title, prompt, views);
	ip.setVisible(true);
	return ps.getSelectedView();
    }
    /*PanelContent**********************/
    public Component getComponent() {
	return scrollPane;
    }
    public boolean panelClosing() {
	return true;
    }
    public void panelClosed() {
	interfacePanel.removeFrameDescription(frameDescription);
	interfacePanel.getButtonManager().uninstallActions(actions);
    }
    /*Actions***************************/
    protected Action[] actions = new Action[]{
	new OkAction(),
	new CancelAction(),
    };
    class OkAction extends AbstractAction {
	OkAction() { super("ok"); }
        public void actionPerformed(ActionEvent evt) {
	    selectedIndex = list.getSelectedIndex();
	    interfacePanel.close();
	}
    }
    class CancelAction extends AbstractAction {
	CancelAction() { super("cancel"); }
        public void actionPerformed(ActionEvent evt) {
	    interfacePanel.close();
	}
    }
}
