/*
 *
 *  LeanEdit
 *  Copyright (c) 1999-2001 Max Kellermann (max.kellermann@epost.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: ChecklistSlot.java,v 1.2 2001/09/20 20:20:53 max Exp $
 *
 */

package org.leanedit.plugins.checklist;

import java.text.ParseException;
import org.w3c.dom.Element;
import make.datamodel.slot.Slot;

public class ChecklistSlot implements Slot {
    /*properties********************/
    protected Element container;
    protected String name;
    protected ElementDescription trueElement, falseElement;
    protected Element element;
    protected boolean value;
    /*Constructor*******************/
    public ChecklistSlot(Element _container, String _name,
			 ElementDescription _trueE,
			 ElementDescription _falseE) {
	container = _container;
	name = _name;
	trueElement = _trueE;
	falseElement = _falseE;
    }
    public ChecklistSlot(Element _container, String _name,
			 ElementDescription _trueE,
			 ElementDescription _falseE,
			 Element _element) {
	this(_container, _name,
	     _trueE, _falseE);
	element = _element;
	value = element!=null;
    }
    /*Slot*********************************/
    public void setValue(Object v) throws ParseException {
	if (v instanceof Boolean) {
	    boolean newValue = ((Boolean)v).booleanValue();
	    if (newValue==value)
		return;
	    if (element!=null) {
		container.removeChild(element);
		element = null;
	    }
	    ElementDescription desc = newValue
		? trueElement
		: falseElement;
	    if (desc!=null) {
		element = desc.createElement(container.getOwnerDocument(), name);
		if (element!=null)
		    container.appendChild(element);
	    }
	} else
	    throw new ParseException("invalid class: "+v.getClass(), 0);
    }
    public Object getValue() throws ParseException {
	return value ? Boolean.TRUE : Boolean.FALSE;
    }
}
