/*
 *
 *  LeanEdit
 *  Copyright (c) 1999-2001 Max Kellermann (max.kellermann@epost.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: ElementDescription.java,v 1.3 2001/09/20 20:20:53 max Exp $
 *
 */

package org.leanedit.plugins.checklist;

import java.util.Hashtable;
import java.util.Enumeration;
import org.w3c.dom.Element;
import org.w3c.dom.Document;
import javax.xml.transform.TransformerException;
import org.apache.xpath.XPath;
import make.util.Debug;
import make.xml.DOMUtil;
import make.xml.XPathUtil;

public class ElementDescription {
    /*properties******************/
    protected String tag, namePath;
    protected XPath nameXPath;
    protected Hashtable assign;
    /*Constructor*******************/
    public ElementDescription(String _tag, String _namePath, 
			      Hashtable _assign) 
	throws TransformerException {
	tag = _tag; namePath = _namePath;
	nameXPath = XPathUtil.parse(namePath);
	assign = _assign;
    }
    /*ElementDescription************/
    public String getTag() {
	return tag;
    }
    public Element createElement(Document doc, String name) {
	Element el = doc.createElement(tag);
	DOMUtil.setDeepString(el, namePath, name);
	for (Enumeration enum=assign.keys(); enum.hasMoreElements();) {
	    String path = (String)enum.nextElement();
	    String value = (String)assign.get(path);
	    DOMUtil.setDeepString(el, path, value);
	}
	return el;
    }
    public String getName(Element el) {
	if (nameXPath==null)
	    return null;
	try {
	    return XPathUtil.execute(nameXPath, el).str();
	} catch (TransformerException e) {
	    Debug.report(e);
	    return null;
	}
    }
}
