/*
 *
 *  LeanEdit
 *  Copyright (c) 1999-2001 Max Kellermann (max.kellermann@epost.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: ContainerCursor.java,v 1.3 2001/09/20 20:20:53 max Exp $
 *
 */

package org.leanedit.plugins.container;

import javax.swing.event.ListSelectionListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.ListSelectionModel;
import org.leanedit.EditorDocument;
import org.leanedit.plugins.AbstractEditorCursor;

public class ContainerCursor extends AbstractEditorCursor 
    implements ListSelectionListener {
    /***properties***************************/
    protected ContainerModel model;
    protected ListSelectionModel selectionModel;
    /***Constructor***************************/
    public ContainerCursor(ContainerModel _m, ListSelectionModel _sm) {
	model = _m; selectionModel = _sm;
	selectionModel.addListSelectionListener(this);
    }
    public void setModel(ContainerModel _cm) {
	model = _cm;
	if (getSelection().length>0)
	    adjustSelection(emptySelection);
    }
    /***EditorCursor***************************/
    public void setSelection(EditorDocument[] sel) {
	synchronized(selectionModel) {
	    selectionModel.setValueIsAdjusting(true);
	    selectionModel.clearSelection();
	    for (int z=0; z<sel.length; z++) {
		int idx = model.indexOfItem(sel[z]);
		if (idx>=0)
		    selectionModel.addSelectionInterval(idx, idx);
	    }
	    selectionModel.setValueIsAdjusting(false);
	}
    }
    /***ListSelectionListener********************/
    public void valueChanged(ListSelectionEvent evt) {
	if (evt.getValueIsAdjusting())
	    return;
	if (model==null) {
	    if (getSelection().length>0)
		adjustSelection(emptySelection);
	} else {
	    final int idx0 = selectionModel.getMinSelectionIndex();
	    if (idx0==-1) {
		adjustSelection(emptySelection);
		return;
	    }
	    final int idx1 = selectionModel.getMaxSelectionIndex();
	    int[] sel = new int[idx1-idx0+1];
	    int count = 0;
	    for (int z=idx0; z<=idx1; z++)
		if (selectionModel.isSelectedIndex(z))
		    sel[count++] = z;
	    EditorDocument[] docs = new EditorDocument[count];
	    for (int z=0; z<count; z++)
		docs[z] = model.getItem(sel[z]);
	    adjustSelection(docs);
	}
    }
}
