/*
 *
 *  LeanEdit
 *  Copyright (c) 1999-2001 Max Kellermann (max.kellermann@epost.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: DefaultContainerModel.java,v 1.4 2001/09/20 20:20:53 max Exp $
 *
 */

package org.leanedit.plugins.container;

import java.util.Vector;
import java.util.Enumeration;
import javax.swing.Action;
import javax.swing.event.ListDataListener;
import javax.swing.event.ListDataEvent;
import org.w3c.dom.Node;
import org.w3c.dom.Element;
import org.w3c.dom.traversal.NodeIterator;
import javax.xml.transform.TransformerException;
import org.apache.xpath.XPath;
import make.util.Debug;
import make.xml.XPathUtil;
import make.xml.DOMUtil;
import org.leanedit.EditorDocument;
import org.leanedit.plugins.EditorComponent;

public class DefaultContainerModel implements DynamicContainerModel {
    /***properties****************************/
    protected EditorComponent component;
    protected EditorDocument document;
    protected XPath rowPath;
    protected Vector items = new Vector();
    protected Vector listeners = new Vector();
    protected Action[] actions;
    /***Constructor********************************/
    public DefaultContainerModel(EditorComponent _ec) {
	component = _ec; 
    }
    public DefaultContainerModel(EditorComponent _ec, EditorDocument _d, 
				 Element vd) 
	throws TransformerException {
	this(_ec);
	setDocument(_d, vd);
    }
    public void setDocument(EditorDocument _d, Element vd)
	throws TransformerException {
	try {
	    document = _d;
	    rowPath = vd==null
		?null
		:XPathUtil.parse(DOMUtil.getElementString(vd, "path"));
	} finally {
	    reload();
	}
    }
    /***Fire***************************************/
    protected void fireContentsChanged(int idx0, int idx1) {
	ListDataEvent evt = new ListDataEvent
	    (this, ListDataEvent.CONTENTS_CHANGED, idx0, idx1);
	for (Enumeration enum=listeners.elements(); enum.hasMoreElements();)
	    ((ListDataListener)enum.nextElement()).contentsChanged(evt);
    }
    protected void fireIntervalAdded(int idx0, int idx1) {
	ListDataEvent evt = new ListDataEvent
	    (this, ListDataEvent.INTERVAL_ADDED, idx0, idx1);
	for (Enumeration enum=listeners.elements(); enum.hasMoreElements();)
	    ((ListDataListener)enum.nextElement()).intervalAdded(evt);
    }
    protected void fireIntervalRemoved(int idx0, int idx1) {
	ListDataEvent evt = new ListDataEvent
	    (this, ListDataEvent.INTERVAL_ADDED, idx0, idx1);
	for (Enumeration enum=listeners.elements(); enum.hasMoreElements();)
	    ((ListDataListener)enum.nextElement()).intervalRemoved(evt);
    }
    /***ContainerModel******************************/
    public int getItemCount() { return items.size(); }
    public EditorDocument getItem(int idx) {
	return (EditorDocument)items.elementAt(idx);
    }
    public int indexOfItem(EditorDocument doc) {
	return items.indexOf(doc);
    }
    public void addListDataListener(ListDataListener _l) {
	listeners.addElement(_l);
    }
    public void removeListDataListener(ListDataListener _l) {
	listeners.removeElement(_l);
    }
    /***DynamicContainerModel***************************/
    public void reload() {
	if (!items.isEmpty()) {
	    final int oldCount = items.size();
	    items.removeAllElements();
	    fireIntervalRemoved(0, oldCount-1);
	}
	if (document!=null && rowPath!=null) {
	    NodeIterator ni;
	    try {
		ni = XPathUtil.execute(rowPath, document.getNode()).nodeset();
	    } catch (TransformerException e) {
		Debug.report(e);
		return;
	    }
	    Node[] nl = DOMUtil.iteratorToArray(ni);
	    for (int z=0; z<nl.length; z++)
		items.addElement(new EditorDocument(nl[z], document));
	    if (!items.isEmpty())
		fireIntervalAdded(0, items.size()-1);
	}
    }
    public int insertItem(EditorDocument _doc, int idx) {
	if (idx>=getItemCount()) {
	    document.getNode().appendChild(_doc.getNode());
	    items.addElement(_doc);
	} else {
	    Node target = getItem(idx).getNode();
	    target.getParentNode().insertBefore(_doc.getNode(), target);
	    items.insertElementAt(_doc, idx);
	}
	fireIntervalAdded(idx, idx);
	return idx;
    }
    public boolean removeItem(int idx) {
	Node n = getItem(idx).getNode();
	n.getParentNode().removeChild(n);
	items.removeElementAt(idx);
	fireIntervalRemoved(idx, idx);
	return true;
    }
    public void itemChanged(int idx) {
	fireContentsChanged(idx, idx);
    }
    public int moveItem(int idx, int newIdx) {
	EditorDocument doc = getItem(idx);
	removeItem(idx);
	return insertItem(doc, newIdx);
    }
}
