/*
 *
 *  LeanEdit
 *  Copyright (c) 1999-2001 Max Kellermann (max.kellermann@epost.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: CountComponent.java,v 1.7 2001/12/22 12:14:59 max Exp $
 *
 */

package org.leanedit.plugins.count;

import java.text.MessageFormat;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JLabel;
import javax.swing.Timer;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.CharacterData;
import org.w3c.dom.Element;
import org.w3c.dom.traversal.NodeIterator;
import javax.xml.transform.TransformerException;
import org.apache.xpath.XPath;
import make.xml.DOMUtil;
import make.xml.XPathUtil;
import make.xml.DocumentType;
import make.xml.ElementType;
import org.leanedit.plugins.EditorComponent;
import org.leanedit.plugins.ComponentListener;
import org.leanedit.plugins.ComponentEvent;
import org.leanedit.plugins.DefaultEditorComponent;
import org.leanedit.EditorDocument;
import org.leanedit.EditorFrame;

public class CountComponent extends DefaultEditorComponent 
    implements ActionListener, ComponentListener {
    /*properties*****************************/
    // visual
    protected JLabel component = new JLabel();
    protected EditorComponent rootComponent;
    // data
    protected XPath selection;
    protected MessageFormat label;
    protected Object[] values = new Object[2];
    protected Timer timer = new Timer(1000, this);
    protected DocumentType documentType;
    /*constructor***************************/
    public CountComponent(EditorFrame _ef) {
        super(_ef);
        timer.setRepeats(false);
    }
    /*Internal*************************************/
    protected boolean count(Node n, int[] v, boolean skipRest) {
        // flow?
        boolean flowing = true;
        if (n instanceof Element) {
            ElementType et = documentType.getElementType(n.getNodeName());
            if (et==null || !et.isFlowing()) {
                flowing = false;
                skipRest = false;
            }
        }
        // children
        NodeList nl = n.getChildNodes();
        for (int z=0; z<nl.getLength(); z++)
            skipRest = count(nl.item(z), v, skipRest);
        // text
        if (n instanceof CharacterData) {
            String s = ((CharacterData)n).getData();
            char[] c = s.toCharArray();
            for (int z=0; z<c.length; z++) {
                if (Character.isLetterOrDigit(c[z]))
                    v[0]++;
                if (Character.isWhitespace(c[z])) {
                    skipRest = false;
                } else {
                    if (!skipRest)
                        v[1]++;
                    skipRest = true;
                }
            }
        }
        // skip?
        return skipRest && flowing;
    }
    protected synchronized void count() {
        EditorDocument doc = getDocument();
        if (doc==null || selection==null) {
            for (int z=0; z<values.length; z++)
                values[z] = new Integer(0);
        } else
            try {
                NodeIterator ni = XPathUtil.execute
                    (selection, doc.getNode()).nodeset();
                Node[] nl = DOMUtil.iteratorToArray(ni);
                int[] v = new int[]{0,0};
                for (int z=nl.length-1; z>=0; z--)
                    count(nl[z], v, false);
                values[0] = new Integer(v[0]);
                values[1] = new Integer(v[1]);
            } catch (TransformerException e) {
                e.printStackTrace();
                for (int z=0; z<values.length; z++)
                    values[z] = new Integer(0);
            }
    }
    protected synchronized void updateDisplay() {
        StringBuffer sb = new StringBuffer();
        label.format(values, sb, null);
        component.setText(sb.toString());
        if (component.getParent()!=null)
            component.getParent().validate();
    }
    /*EditorComponent********************************/
    public synchronized void setDocument(EditorDocument _doc) {
        super.setDocument(_doc);
        component.setText(null);
        if (rootComponent!=null) {
            rootComponent.removeComponentListener(this);
            rootComponent = null;
        }
        if (_doc==null) {
            documentType = null;
            selection = null;
            label = null;
            timer.stop();
        } else {
            documentType = _doc.getType();
            Element vd = (Element)getViewDescription("count");
            try {
                selection = XPathUtil.parse(DOMUtil.getElementString(vd, "selection"));
            } catch (TransformerException e) {
                e.printStackTrace();
            }
            String labelFormatString = DOMUtil.getLocalizedElementString
                (vd, "label", getSession().getLocale(), false);
            label = new MessageFormat(labelFormatString);
            timer.setRepeats(false);
            timer.start();
            rootComponent = editorFrame.getComponents(EditorFrame.COMPONENT_ROOT)[0];
            rootComponent.addComponentListener(this);
        }
        // validate
        if (component.getParent()!=null)
            component.getParent().validate();
    }
    public Component getComponent() { return component; }
    public void dispose() {
        timer.stop();
        timer = null;
        if (rootComponent!=null) {
            rootComponent.removeComponentListener(this);
            rootComponent = null;
        }
        super.dispose();
    }
    /*ActionListener********************************/
    public void actionPerformed(ActionEvent evt) {
        count();
        updateDisplay();
    }
    /*ComponentListener*****************************/
    public void focusGained(ComponentEvent evt) {}
    public void documentChanged(ComponentEvent evt) {
        if (timer.isRunning())
            timer.restart();
        else
            timer.start();
    }
}
