/*
 *
 *  LeanEdit
 *  Copyright (c) 1999-2001 Max Kellermann (max.kellermann@epost.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: FileNodeSelector.java,v 1.4 2001/09/20 20:20:53 max Exp $
 *
 */

package org.leanedit.plugins.link;

import java.io.File;
import java.io.IOException;
import org.w3c.dom.Node;
import org.w3c.dom.Element;
import org.xml.sax.SAXException;
import javax.swing.JFileChooser;
import javax.swing.filechooser.FileFilter;
import make.datamodel.embedded.EmbeddedManager;
import make.datamodel.embedded.FileEmbeddedManager;
import make.datamodel.embedded.ZipEmbeddedManager;
import make.io.Streams;
import org.leanedit.EditorFrame;

class XmlFileFilter extends FileFilter {
    /***FileFilter******************************/
    public boolean accept(File f) {
	if (f!=null) {
	    if (f.isDirectory()) return true;
	    String name = f.getName();
	    return name.endsWith(".xml") ||
		name.endsWith(".wml");
	}
	return false;
    }
    public String getDescription() {
	return "eXtensible Markup Language (*.xml)";
    }
}

class ZipFileFilter extends FileFilter {
    /***FileFilter******************************/
    public boolean accept(File f) {
	if (f!=null) {
	    if (f.isDirectory()) return true;
	    String name = f.getName();
	    return name.endsWith(".zip");
	}
	return false;
    }
    public String getDescription() {
	return "XML-ZIP-Dateien (*.zip)";
    }
}

public class FileNodeSelector implements NodeSelector {
    /***properties****************************/
    protected JFileChooser fileChooser = new JFileChooser();
    protected EditorFrame editorFrame;
    /***Constructor****************************/
    public FileNodeSelector(Element vd, EditorFrame _ef) {
	editorFrame = _ef;
	// filters
	FileFilter xmlFilter = new XmlFileFilter();
	fileChooser.addChoosableFileFilter(xmlFilter);
	fileChooser.addChoosableFileFilter(new ZipFileFilter());
	fileChooser.setFileFilter(xmlFilter);
    }
    public static EmbeddedManager createEmbeddedManager(File f) 
	throws IOException {
	if (f.getName().endsWith(".zip"))
	    return new ZipEmbeddedManager(f, null, Streams.defaultEncoding);
	else
	    return new FileEmbeddedManager(f, null, Streams.defaultEncoding);
    }
    /***NodeSelector**************************/
    public Node selectNode() throws IOException, SAXException {
	final int chooseRet = fileChooser.showOpenDialog
	    (editorFrame.getInterfacePanel().getFrame());
	if (chooseRet!=fileChooser.APPROVE_OPTION)
	    return null;
	File f = fileChooser.getSelectedFile();
	if (f.exists()) {
	    EmbeddedManager em = createEmbeddedManager(f);
	    return em.getDocument();
	} else
	    return null;
    }
}
