/*
 *
 *  LeanEdit
 *  Copyright (c) 1999-2001 Max Kellermann (max.kellermann@epost.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: ListComponent.java,v 1.5 2001/09/20 20:20:53 max Exp $
 *
 */

package org.leanedit.plugins.list;

import java.awt.Component;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import javax.swing.JList;
import org.w3c.dom.Element;
import javax.xml.transform.TransformerException;
import org.apache.xpath.XPath;
import make.xml.DOMUtil;
import make.xml.XPathUtil;
import org.leanedit.EditorDocument;
import org.leanedit.EditorFrame;
import org.leanedit.plugins.container.AbstractListComponent;

public class ListComponent extends AbstractListComponent 
    implements MouseListener {
    /***properties*****************************/
    protected XPath itemPath;
    protected ContainerListModel model = new ContainerListModel(this);
    protected JList list = new JList(model);
    /***Constructor***************************/
    public ListComponent(EditorFrame _ef) {
	super(_ef);
	registerInputComponent(list);
	list.addMouseListener(this);
	setModel(model, list.getSelectionModel());
    }
    protected synchronized void reload() {
	model.reload();
    }
    /***MouseListener*********************************/
    public void mouseClicked(MouseEvent evt) {
	if (evt.getClickCount()==2)
	    editElement(getSelectedIndex());
    }
    public void mouseEntered(MouseEvent evt) {}
    public void mouseExited(MouseEvent evt) {}
    public void mousePressed(MouseEvent evt) {}
    public void mouseReleased(MouseEvent evt) {}
    /***EditorComponent********************************/
    public synchronized void setDocument(EditorDocument _doc) {
	super.setDocument(_doc);
	// remove old
	try {
	    model.setDocument(null, null);
	} catch (TransformerException e) {
	    e.printStackTrace();
	}
	itemPath = null;
	// config
	Element vd = (Element)getViewDescription("list");
	if (vd!=null) {
	    loadActions(DOMUtil.getUniqueTag(vd, "actions"));
	    loadKeyBindings(list, DOMUtil.getUniqueTag(vd, "keybindings"));
	    // item
	    Element it = DOMUtil.getUniqueTag(vd, "item");
	    try {
		itemPath = it==null
		    ?null
		    :XPathUtil.parse(it.getAttribute("caption"));
	    } catch (TransformerException e) {
		itemPath = null;
		e.printStackTrace();
	    }
	    // model
	    try {
		model.setDocument(_doc, vd);
	    } catch (TransformerException e) {
		e.printStackTrace();
	    }
	}
    }
    public Component getComponent() { 
	return list;
    }
}
