/*
 *
 *  LeanEdit
 *  Copyright (c) 1999-2001 Max Kellermann (max.kellermann@epost.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: SearchComponent.java,v 1.8 2001/12/22 12:14:59 max Exp $
 *
 */

package org.leanedit.plugins.search;

import java.awt.Component;
import java.awt.BorderLayout;
import javax.swing.JPanel;
import org.w3c.dom.Node;
import org.w3c.dom.Element;
import javax.xml.transform.TransformerException;
import org.apache.xpath.XPath;
import org.apache.xpath.objects.XObject;
import make.util.Strings;
import make.util.Debug;
import make.datamodel.slot.Slot;
import make.xml.DOMUtil;
import make.xml.XPathUtil;
import make.swing.il.InputLink;
import make.swing.il.FieldInputLink;
import make.swing.il.InputLinkFactory;
import make.swing.field.Field;
import make.swing.field.FieldListener;
import make.swing.field.FieldEvent;
import org.leanedit.EditorDocument;
import org.leanedit.EditorFrame;
import org.leanedit.field.EditorInputLinkFactory;
import org.leanedit.plugins.DefaultEditorComponent;
import org.leanedit.plugins.EditorComponent;
import org.leanedit.plugins.EditorCursor;
import org.leanedit.plugins.EditorCursorComponent;

public class SearchComponent extends DefaultEditorComponent 
    implements FieldListener {
    static class SearchInputLinkFactory 
        extends EditorInputLinkFactory {
        public SearchInputLinkFactory(EditorFrame _frm, EditorDocument _doc) {
            super(_frm, _doc, null);
        }
        public Slot getSlot(Element vd) {
            return null;
        }
        public String getType(Element vd) {
            return vd.getAttribute("type");
        }
    }
    /*properties*****************************/
    protected InputLink inputLink;
    protected Field field;
    protected Component component, label;
    protected JPanel panel = new JPanel(new BorderLayout(), false);
    protected EditorCursorComponent cursorComponent;
    protected EditorCursor cursor;
    protected XPath matchPath;
    /*Constructor***************************/
    public SearchComponent(EditorFrame _ef) {
        super(_ef);
    }
    /*FieldListener***********************************/
    public void valueChanged(FieldEvent evt) {}
    public void focusGained(FieldEvent evt) {}
    public void focusLost(FieldEvent evt) {}
    public void actionPerformed(FieldEvent evt) {
        try {
            EditorDocument baseDocument = getDocument();
            Node baseNode = baseDocument.getNode();
            XObject result = XPathUtil.execute(matchPath, baseNode);
            Node[] nl = DOMUtil.iteratorToArray(result.nodeset());
            EditorDocument[] sel = new EditorDocument[nl.length];
            for (int z=0; z<sel.length; z++)
                sel[z] = new EditorDocument(nl[z], baseDocument);
            cursor.setSelection(sel);
        } catch (TransformerException e) {
            Debug.report(e, "while executing XPath");
        }
    }
    /*EditorComponent********************************/
    public synchronized void setDocument(EditorDocument _doc) {
        super.setDocument(_doc);
        // remove old
        unregisterInputComponent(panel);
        if (component!=null) {
            panel.remove(component);
            component = null;
        }
        if (label!=null) {
            panel.remove(label);
            label = null;
        }
        if (inputLink!=null) {
            if (field!=null) {
                field.removeFieldListener(this);
                field = null;
            }
            inputLink.dispose();
            inputLink = null;
        }
        cursor = null;
        matchPath = null;
        // create new
        if (_doc!=null) {
            InputLinkFactory inputLinkFactory
                = new SearchInputLinkFactory(getEditorFrame(), _doc);
            Element vd = (Element)getViewDescription("search");
            // create inputLink
            Element ild = DOMUtil.getUniqueTag(vd, "inputlink");
            inputLink = inputLinkFactory.createInputLink(ild, getSession().getLocale());
            if (inputLink!=null) {
                label = inputLink.getLabel();
                if (label!=null) {
                    panel.add("West", label);
                }
                component = inputLink.getComponent();
                panel.add("Center", component);
                if (inputLink instanceof FieldInputLink) {
                    field = ((FieldInputLink)inputLink).getField();
                    field.addFieldListener(this);
                }
            }
            // lookup cursor
            Element cursorEl = DOMUtil.getUniqueTag(vd, "cursor");
            String cursorPath = Strings.trim(DOMUtil.getElementString(cursorEl));
            String cursorName = Strings.trim(cursorEl.getAttribute("name"));
            EditorComponent[] ecs = getEditorFrame().getComponents(cursorPath);
            if (ecs.length==0)
                System.err.println("ERROR: cursor "+cursorPath+" not found");
            if (ecs[0] instanceof EditorCursorComponent) {
                cursorComponent = (EditorCursorComponent)ecs[0];
                cursor = cursorComponent.getCursor(cursorName);
            } else
                System.err.println("ERROR: component "+ecs[0]+" has no cursors");
            // parse match path
            String mp = DOMUtil.getElementString(vd, "match");
            try {
                matchPath = XPathUtil.parse(mp);
            } catch (TransformerException e) {
                Debug.report(e, "while parsing "+mp);
            }
        }
        registerInputComponent(panel);
        // validate
        if (panel.getParent()!=null)
            panel.getParent().validate();
    }
    public Component getComponent() { return panel; }
}
