/*
 *
 *  LeanEdit
 *  Copyright (c) 1999-2001 Max Kellermann (max.kellermann@epost.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: SliderComponent.java,v 1.4 2001/09/20 20:20:53 max Exp $
 *
 */

package org.leanedit.plugins.slider;

import java.text.ParseException;
import java.awt.Component;
import javax.swing.JSlider;
import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;
import org.w3c.dom.Node;
import org.w3c.dom.Element;
import make.xml.DOMUtil;
import org.leanedit.EditorDocument;
import org.leanedit.EditorFrame;
import org.leanedit.plugins.DefaultEditorComponent;
import org.leanedit.plugins.ComponentEvent;

/**
 * This is an example class which maps javax.swing.JSlider 
 * to a LeanEdit plugin.
 */
public class SliderComponent extends DefaultEditorComponent 
    implements ChangeListener {
    /***properties*****************************/
    protected JSlider component;
    /***Constructor***************************/
    public SliderComponent(EditorFrame _ef) {
	super(_ef);
	component = new JSlider(JSlider.HORIZONTAL, 0, 100, 0);
	component.addChangeListener(this);
    }
    /***EditorComponent********************************/
    public synchronized void setDocument(EditorDocument _doc) {
	super.setDocument(_doc);
	int min = 0, max = 100;
	Element config = (Element)getViewDescription("slider");
	if (config!=null) {
	    String minString = DOMUtil.getElementString(config, "min");
	    if (minString!=null)
		min = Integer.parseInt(minString);
	    String maxString = DOMUtil.getElementString(config, "max");
	    if (maxString!=null)
		max = Integer.parseInt(maxString);
	}
	component.setMinimum(min);
	component.setMaximum(max);
	Node node = _doc.getNode();
	String valueString;
	if (node instanceof Element)
	    valueString = DOMUtil.getElementString((Element)node);
	else 
	    valueString = node.getNodeValue();
	int value = 0;
	try {
	    if (valueString!=null)
		value = Integer.parseInt(valueString);
	} catch (NumberFormatException e) {
	}
	component.setValue(value);
    }
    public void flushDocument() throws ParseException {
	int value = component.getValue();
	Node node = getDocument().getNode();
	if (node instanceof Element)
	    DOMUtil.setElementString((Element)node, ""+value);
	else
	    node.setNodeValue(""+value);
    }
    public Component getComponent() { return component; }
    public void dispose() {
	component.removeChangeListener(this);
    }
    /***ChangeListener*****************************/
    public void stateChanged(ChangeEvent evt) {
	if (!component.getValueIsAdjusting()) {
	    try {
		flushDocument();
		fireDocumentChanged(new ComponentEvent(this, getDocument(),
						       getDocument().getNode()));
	    } catch (ParseException e) {
	    }
	}
    }
}
