/*
 *
 *  LeanEdit
 *  Copyright (c) 1999-2001 Max Kellermann (max.kellermann@epost.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: TabbedEditorComponent.java,v 1.8 2001/12/22 12:14:59 max Exp $
 *
 */

package org.leanedit.plugins.tabbed;

import java.awt.Component;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseEvent;
import java.util.Vector;
import java.util.Hashtable;
import javax.swing.Action;
import javax.swing.JTabbedPane;
import javax.swing.JPopupMenu;
import javax.swing.event.PopupMenuListener;
import javax.swing.event.PopupMenuEvent;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import javax.xml.transform.TransformerException;
import org.leanedit.EditorFrame;
import org.leanedit.EditorDocument;
import org.leanedit.plugins.EditorComponent;
import org.leanedit.plugins.ContainerComponent;
import make.xml.DOMUtil;
import make.swing.bt.PopupMenuDescription;

class TabbedEditorComponent extends ContainerComponent 
    implements MouseListener {
    /*properties*********************************/
    protected JTabbedPane tabbedPane = new JTabbedPane();
    protected Hashtable tabDescriptions = new Hashtable();
    protected PopupMenuDescription popupMenu;
    protected Hashtable tabs = new Hashtable();
    /*Constructor*******************************/
    public TabbedEditorComponent(EditorFrame _ef) {
        super(_ef);
        tabbedPane.addMouseListener(this);
    }
    protected void addTab(Element t, Child child) {
        tabs.put(child.getComponent(), child);
        String caption = DOMUtil.getElementString(t, "caption");
        String tooltip = DOMUtil.getElementString(t, "tooltip");
        if (caption==null) caption = "???";
        String captionImpl;
        if (caption.startsWith("{") && caption.endsWith("}"))
            try {
                captionImpl = child.execute
                    (caption.substring(1, caption.length()-1)).str();
            } catch (TransformerException e) {
                e.printStackTrace();
                captionImpl = caption;
            }
        else
            captionImpl = caption;
        String tooltipImpl;
        if (tooltip==null)
            tooltipImpl = null;
        else if (tooltip.startsWith("{") && tooltip.endsWith("}"))
            try {
                tooltipImpl = child.execute
                    (tooltip.substring(1, tooltip.length()-1)).str();
            } catch (TransformerException e) {
                e.printStackTrace();
                tooltipImpl = null;
            }
        else
            tooltipImpl = tooltip;
        tabbedPane.addTab(captionImpl, null, 
                          child.getComponent(), tooltipImpl);
    }
    protected void addTab(String path, String name, Node data) {
        Element t = (Element)tabDescriptions.get(path);
        if (t!=null) {
            EditorDocument ed = new EditorDocument(data, getDocument());
            ed.setName(name);
            Element component = DOMUtil.getUniqueTag(t, "component");
            Element[] cps = DOMUtil.getChildElements(component);
            for (int a=0; a<cps.length; a++)
                addTab(t, createEditor(null, cps[a], ed));
        }
    }
    protected Action loadAction(Element _a) {
        String tag = _a.getTagName();
        if (tag.equals("new-tab"))
            return new InsertElementAction(_a.getAttribute("tag"), _a);
        if (tag.equals("delete-tab"))
            return new DeleteTabAction(_a);
        /*if (tag.equals("edit-tab"))
          return new EditTabAction(_a);*/
        return super.loadAction(_a);
    }
    /*EditorComponent****************************/
    public Component getComponent() { return tabbedPane; }
    public void setDocument(EditorDocument _doc) {
        super.setDocument(_doc);
        // remove old
        Component[] cs = tabbedPane.getComponents();
        for (int z=0; z<cs.length; z++)
            tabbedPane.remove(cs[z]);
        tabDescriptions.clear();
        tabs.clear();
        popupMenu = null;
        // read view
        Element vd = (Element)getViewDescription("tabbed");
        loadActions(DOMUtil.getUniqueTag(vd, "actions"));
        Element tbs = DOMUtil.getUniqueTag(vd, "tabs");
        Element[] _tabs = DOMUtil.getChildElements(tbs);
        for (int z=0; z<_tabs.length; z++) {
            final Element t = _tabs[z];
            String path = DOMUtil.getDeepString(t, "component/editor/path");
            if (path!=null)
                tabDescriptions.put(path, t);
            Element component = DOMUtil.getUniqueTag(t, "component");
            Element[] cps = DOMUtil.getChildElements(component);
            for (int a=0; a<cps.length; a++) {
                Child[] children = createEditor(null, cps[a]);
                for (int i=0; i<children.length; i++)
                    addTab(t, children[i]);
            }
        }
        Element pm = DOMUtil.getUniqueTag(vd, "popup-menu");
        if (pm!=null)
            popupMenu = new PopupMenuDescription(getButtonContainer(), pm,
                                                 getClass(), getSession().getLocale());
    }
    /*MouseListener********************************/
    public void mouseClicked(MouseEvent evt) {}
    public void mousePressed(MouseEvent evt) {
        if ((evt.getModifiers()&evt.BUTTON3_MASK)!=0) {
            final Vector bts = new Vector(), cps = new Vector();
            JPopupMenu menu = new JPopupMenu();
            menu.addPopupMenuListener(new PopupMenuListener() {
                    public void popupMenuWillBecomeVisible(PopupMenuEvent evt2) {}
                    public void popupMenuWillBecomeInvisible(PopupMenuEvent evt2) {}
                    public void popupMenuCanceled(PopupMenuEvent evt2) {
                        getButtonManager().uninstallButtons(bts.elements());
                    }
                });
            popupMenu.install(menu, bts, cps);
            getButtonManager().installButtons(bts.elements());
            menu.show(evt.getComponent(), evt.getX(), evt.getY());
        }
    }
    public void mouseReleased(MouseEvent evt) {}
    public void mouseEntered(MouseEvent evt) {}
    public void mouseExited(MouseEvent evt) {}
    /*Actions****************************************/
    protected abstract class InsertNodeAction extends ConfigAction {
        public InsertNodeAction(String _ac, Element vd) { super(_ac, vd); }
        protected abstract Node createNode();
        protected abstract String getPath();
        public void actionPerformed(ActionEvent evt) {
            Node n = createNode();
            getDocument().getNode().appendChild(n);
            addTab(getPath(), null, n); // FIXME: fill out name
        }
    }
    class InsertElementAction extends InsertNodeAction {
        String tag;
        public InsertElementAction(String _tag, Element vd) { 
            super("insert-"+_tag, vd); 
            tag = _tag;
        }
        protected Node createNode() {
            return getXMLDocument().createElement(tag);
        }
        protected String getPath() { return tag; }
    }
    class DeleteTabAction extends ConfigAction {
        public DeleteTabAction(Element vd) { 
            super("delete-tab", vd); 
        }
        public void actionPerformed(ActionEvent evt) {
            synchronized(tabbedPane) {
                Component sel = tabbedPane.getSelectedComponent();
                if (sel==null)
                    return;
                Child ch = (Child)tabs.get(sel);
                if (ch==null)
                    return;
                EditorDocument doc = ch.getDocument();
                Node n = doc.getNode();
                n.getParentNode().removeChild(n);
                tabbedPane.remove(sel);
                tabs.remove(sel);
            }
        }
    }
}
