/*
 *
 *  LeanEdit
 *  Copyright (c) 1999-2001 Max Kellermann (max.kellermann@epost.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: TabbedComponent2.java,v 1.6 2001/10/03 21:10:28 max Exp $
 *
 */

package org.leanedit.plugins.tabbed2;

import java.awt.Component;
import java.awt.Frame;
import java.awt.event.MouseListener;
import java.awt.event.MouseEvent;
import java.util.Vector;
import javax.swing.JTabbedPane;
import javax.swing.JPopupMenu;
import javax.swing.DefaultListSelectionModel;
import javax.swing.event.PopupMenuListener;
import javax.swing.event.PopupMenuEvent;
import javax.swing.event.ListDataListener;
import javax.swing.event.ListDataEvent;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.dom.DOMResult;
import make.util.Debug;
import make.xml.DOMUtil;
import make.swing.SingleSelectionAdapter;
import org.leanedit.EditorFrame;
import org.leanedit.EditorDocument;
import org.leanedit.plugins.EditorComponent;
import org.leanedit.plugins.ComponentEvent;
import org.leanedit.plugins.container.AbstractListComponent;
import org.leanedit.plugins.container.DefaultContainerModel;

class TabbedComponent2 extends AbstractListComponent 
    implements MouseListener, ListDataListener {
    /***properties*********************************/
    protected DefaultContainerModel model = new DefaultContainerModel(this);
    protected TransformerFactory tFactory = TransformerFactory.newInstance();
    protected Transformer tabStyle;
    protected JTabbedPane tabbedPane = new JTabbedPane();
    protected Vector children = new Vector();
    /***Constructor*******************************/
    public TabbedComponent2(EditorFrame _ef) {
	super(_ef);
	model.addListDataListener(this);
	tabbedPane.addMouseListener(this);
	DefaultListSelectionModel dest = new DefaultListSelectionModel();
	SingleSelectionAdapter ssa = new SingleSelectionAdapter
	    (tabbedPane.getModel(), dest);
	setModel(model, dest);
    }
    /***Add***************************************/
    protected Element transformTab(EditorDocument _ed) {
	Document vdd = DOMUtil.createDocument();
	try {
	    Document newDoc = DOMUtil.createDocument();
	    newDoc.appendChild(newDoc.importNode(_ed.getNode(), true));
	    tabStyle.transform(new DOMSource(newDoc),
			       new DOMResult(vdd));
	} catch (Exception e) {
	    Debug.report(e);
	}
	return vdd.getDocumentElement();
    }
    protected void updateTab(int idx, Element vd) {
	String caption = DOMUtil.getElementString(vd, "caption");
	tabbedPane.setTitleAt(idx, caption==null || caption.equals("")?"[no title]":caption);
    }
    protected void updateTab(int idx) {
	updateTab(idx, transformTab(model.getItem(idx)));
    }
    protected void insertTab(EditorDocument _ed, int idx) {
	Element vd = transformTab(_ed);
	Child ch = createEditor(null, vd, _ed);
	children.insertElementAt(ch, idx);
	tabbedPane.insertTab("", null, ch.getComponent(), null, idx);
	updateTab(idx, vd);
    }
    /***EditorComponent****************************/
    public Component getComponent() { return tabbedPane; }
    public synchronized void setDocument(EditorDocument _doc) {
	super.setDocument(_doc);
	// remove old
	try {
	    model.setDocument(null, null);
	} catch (TransformerException e) {
	    e.printStackTrace();
	}
	tabStyle = null;
	// read view
	Element vd = (Element)getViewDescription("tabbed2");
	if (vd!=null) {
	    loadActions(DOMUtil.getUniqueTag(vd, "actions"));
	    try {
		Element tab = DOMUtil.getUniqueTag(vd, "tab");
		Document newDoc = DOMUtil.createDocument();
		newDoc.appendChild(newDoc.importNode(tab, true));
		tabStyle = tFactory.newTransformer(new DOMSource(newDoc));
	    } catch (TransformerException e) {
		Debug.report(e);
	    }
	    // model
	    try {
		model.setDocument(_doc, vd);
	    } catch (TransformerException e) {
		Debug.report(e);
	    }
	}
    }
    /***ComponentListener*****************************/
    public void documentChanged(ComponentEvent evt) {
	int idx = indexOf(evt.getDocument());
	Child ch = getChildAt(idx);
	itemChanged(ch.getEditorComponent(), ch.getDocument());
	super.documentChanged(evt);
    }
    /***ListDataListener******************************/
    public void contentsChanged(ListDataEvent evt) {
	for (int z=evt.getIndex0(); z<=evt.getIndex1(); z++)
	    updateTab(z);
    }
    public void intervalAdded(ListDataEvent evt) {
	for (int z=evt.getIndex0(); z<=evt.getIndex1(); z++)
	    insertTab(model.getItem(z), z);
    }
    public void intervalRemoved(ListDataEvent evt) {
	for (int z=evt.getIndex1(); z>=evt.getIndex0(); z--) {
	    children.removeElementAt(z);
	    tabbedPane.removeTabAt(z);
	    //TODO: super.remove
	}
    }
    /***MouseListener********************************/
    public void mouseClicked(MouseEvent evt) {}
    public void mousePressed(MouseEvent evt) {
	/*if ((evt.getModifiers()&evt.BUTTON3_MASK)!=0) {
	  final Vector bts = new Vector(), cps = new Vector();
	  JPopupMenu menu = new JPopupMenu();
	  menu.addPopupMenuListener(new PopupMenuListener() {
	  public void popupMenuWillBecomeVisible(PopupMenuEvent evt2) {}
	  public void popupMenuWillBecomeInvisible(PopupMenuEvent evt2) {}
	  public void popupMenuCanceled(PopupMenuEvent evt2) {
	  getButtonManager().uninstallButtons(bts.elements());
	  }
	  });
	  popupMenu.install(menu, bts, cps);
	  getButtonManager().installButtons(bts.elements());
	  menu.show(evt.getComponent(), evt.getX(), evt.getY());
	  }*/
    }
    public void mouseReleased(MouseEvent evt) {}
    public void mouseEntered(MouseEvent evt) {}
    public void mouseExited(MouseEvent evt) {}
}
