/*
 *
 *  LeanEdit
 *  Copyright (c) 1999-2001 Max Kellermann (max.kellermann@epost.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: AbstractColumn.java,v 1.7 2001/12/22 12:14:59 max Exp $
 *
 */

package org.leanedit.plugins.table;

import java.util.Map;
import java.util.Locale;
import org.w3c.dom.Node;
import org.w3c.dom.Element;
import make.xml.DOMUtil;
import make.xml.DocumentType;
import make.swing.cell.CellRenderer;
import make.swing.cell.TranslateCellRenderer;

public abstract class AbstractColumn implements Column {
    /*static******************************/
    public static Column createColumn(TableComponent table, Element cd,
                                      Locale locale) {
        String type = cd.getTagName();
        if (type.equals("column") || type.equals("xpathcolumn"))
            return new XPathColumn(table, cd, locale);
        else if (type.equals("choosecolumn"))
            return new ChooseColumn(table, cd, locale);
        else if (type.equals("nullcolumn"))
            return new NullColumn(cd, locale);
        else {
            System.err.println("WARNING: unknown table column type "+type);
            return new NullColumn(cd, locale);
        }
    }
    /*properties*********************/
    protected String caption;
    protected CellRenderer cellRenderer;
    /*Constructor*************************/
    public AbstractColumn(TableComponent table, Element c, Locale locale) {
        caption = DOMUtil.getLocalizedElementString(c, "caption", locale, false);
        Element rend = DOMUtil.getUniqueTag(c, "renderer");
        if (rend!=null) {
            Element[] rcs = DOMUtil.getChildElements(rend);
            if (rcs.length>0) {
                String type = rcs[0].getTagName();
                if (type.equals("translate-renderer")) {
                    String listName = rcs[0].getAttribute("list");
                    DocumentType docType = table.getDocument().getType();
                    Map map = (Map)docType.getList(listName);
                    cellRenderer = new TranslateCellRenderer
                        (table.getDefaultCellRenderer(), map);
                } else {
                    System.err.println("ERROR: unknown cell renderer "+type);
                }
            }
        }
    }
    /*Column*****************************/
    public String getCaption() { return caption; }
    public CellRenderer getCellRenderer(Node row) {
        return cellRenderer;
    }
}
