/*
 *
 *  LeanEdit
 *  Copyright (c) 1999-2001 Max Kellermann (max.kellermann@epost.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: ChooseColumn.java,v 1.6 2001/12/22 12:14:59 max Exp $
 *
 */

package org.leanedit.plugins.table;

import java.util.Locale;
import org.w3c.dom.Node;
import org.w3c.dom.Element;
import org.apache.xpath.XPath;
import make.xml.DOMUtil;
import make.xml.XPathUtil;
import make.swing.field.Field;

public class ChooseColumn extends AbstractColumn {
    /*CONST*****************************/
    protected static final NullColumn nullColumn = new NullColumn(null, null);
    /*properties*********************/
    protected XPath[] conditions;
    protected Column[] columns;
    /*Constructor*************************/
    public ChooseColumn(TableComponent table, Element c, Locale locale) {
        super(table, c, locale);
        Element chs = DOMUtil.getUniqueTag(c, "choose");
        Element[] wh = DOMUtil.getChildElements(chs);
        conditions = new XPath[wh.length];
        columns = new Column[wh.length];
        for (int z=0; z<wh.length; z++) {
            try {
                if ("when".equals(wh[z].getTagName()))
                    conditions[z] = XPathUtil.parse(wh[z].getAttribute("test"));
                columns[z] = createColumn(table, DOMUtil.getChildElements(wh[z])[0],
                                          locale);
            } catch (Exception e) {
                System.err.println("error while loading column "+z);
                e.printStackTrace();
            }
        }
    }
    /*Internal***************************/
    protected Column findColumn(Node row) {
        for (int z=0; z<conditions.length; z++)
            try {
                if (conditions[z]==null ||
                    XPathUtil.execute(conditions[z], row).bool())
                    return columns[z];
            } catch (Exception e) {
                e.printStackTrace();
            }
        return nullColumn;
    }
    /*Column*****************************/
    public boolean isEditable(Node row) { 
        return findColumn(row).isEditable(row);
    }
    public Object getValue(Node row) {
        return findColumn(row).getValue(row);
    }
    public void setValue(Node row, Object value) {
        findColumn(row).setValue(row, value);
    }
    public Field getField(Node row) {
        return findColumn(row).getField(row);
    }
}
