/*
 *
 *  LeanEdit
 *  Copyright (c) 1999-2001 Max Kellermann (max.kellermann@epost.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: XPathColumn.java,v 1.6 2001/12/22 12:14:59 max Exp $
 *
 */

package org.leanedit.plugins.table;

import java.util.Locale;
import org.w3c.dom.Node;
import org.w3c.dom.Element;
import javax.xml.transform.TransformerException;
import org.apache.xpath.XPath;
import make.util.Debug;
import make.xml.DOMUtil;
import make.xml.XPathUtil;
import make.swing.field.Field;
import make.swing.il.InputLink;
import make.swing.il.FieldInputLink;

public class XPathColumn extends AbstractColumn {
    /*properties*********************/
    protected XPath path;
    protected String pathString;
    protected Field field;
    protected boolean editable;
    /*Constructor*************************/
    public XPathColumn(TableComponent table, Element c, Locale locale) {
        super(table, c, locale);
        pathString = DOMUtil.getElementString(c, "path");
        try {
            path = XPathUtil.parse(DOMUtil.getElementString(c, "path"));
        } catch (TransformerException e) {
            Debug.report(e);
        }
        if (DOMUtil.childTagExists(c, "field")) {
            InputLink il = table.getInputLinkFactory().createInputLink(c, locale);
            if (il!=null)
                field = ((FieldInputLink)il).getField();
        }
        editable = !"no".equals(c.getAttribute("editable"));
    }
    /*Column*****************************/
    public boolean isEditable(Node row) { return editable; }
    public Object getValue(Node row) {
        try {
            return path==null
                ?null
                :XPathUtil.execute(path, row).str();
        } catch (TransformerException e) {
            Debug.report(e);
            return null;
        }
    }
    public void setValue(Node row, Object value) {
        DOMUtil.setDeepString(row, pathString, (String)value);
        //TODO: besser parsen
    }
    public Field getField(Node row) {
        return field;
    }
}
