/*
 *
 *  LeanEdit
 *  Copyright (c) 1999-2001 Max Kellermann (max.kellermann@epost.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: TextActions.java,v 1.11 2001/10/30 17:08:45 max Exp $
 *
 */

package org.leanedit.plugins.text;

import java.util.Hashtable;
import java.util.Vector;
import java.io.InputStream;
import java.io.Reader;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.util.Hashtable;
import java.util.Vector;
import java.util.Hashtable;
import java.util.Vector;
import java.awt.event.ActionEvent;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.UnsupportedFlavorException;
import javax.swing.Action;
import javax.swing.AbstractAction;
import javax.swing.JOptionPane;
import javax.swing.text.Segment;
import javax.swing.text.Position;
import javax.swing.text.BadLocationException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.dom.DOMResult;
import org.xml.sax.SAXException;
import org.w3c.dom.Node;
import org.w3c.dom.Text;
import org.w3c.dom.NodeList;
import make.util.Debug;
import make.io.Streams;
import make.xml.DOMUtil;
import make.xml.DocumentType;
import make.datamodel.embedded.EmbeddedManager;
import make.datamodel.embedded.MemoryEmbeddedManager;
import org.leanedit.EditorFrame;
import org.leanedit.plugins.text.tree.SwingDocument;
import org.leanedit.plugins.text.tree.ElementSwingElement;
import org.leanedit.plugins.text.dlg.DefaultEditorDialog;
import org.leanedit.plugins.text.view.XmlViewFactory;
import make.swing.ButtonManager;
import make.swing.text.TextComponent;
import make.swing.text.DefaultEditorKit;
import make.swing.text.Element;
import make.swing.text.Document;
import make.swing.text.DocumentEvent;
import make.swing.text.view.ViewFactory;

public class TextActions extends DefaultEditorKit {
    /*static*****************************************/
    protected static DataFlavor xmlFlavor
        = new DataFlavor("text/xml; charset="+Streams.defaultEncoding, "XML");
    protected static DataFlavor xmlJavaFlavor
        = new DataFlavor("application/x-java-serialized-object-leanedit; "+
                         "charset="+Streams.defaultEncoding, 
                         "XML mit Bildern");
    protected static final ViewFactory viewFactory
        = new XmlViewFactory();

    /*properties*********************************/
    private TextEditorComponent tec;
    protected EditorFrame editorFrame;
    protected Vector xmlActions = new Vector();
    protected boolean paintAll = false;

    /*Constructor********************************/
    public TextActions(TextEditorComponent _tec, EditorFrame _f) {
        tec = _tec; editorFrame = _f;
    }
    public void loadActions(org.w3c.dom.Element ac, ButtonManager bm) {
        synchronized(xmlActions) {
            if (bm!=null)
                bm.uninstallActions(xmlActions.elements());
            xmlActions.removeAllElements();
            org.w3c.dom.Element[] acs = DOMUtil.getChildElements(ac);
            for (int z=0; z<acs.length; z++) {
                Action a;
                String tag = acs[z].getTagName();
                if (tag.equals("formataction"))
                    a = new FormatStyleAction(acs[z]);
                else if (tag.equals("insertaction"))
                    a = new InsertTagAction(acs[z]);
                else if (tag.equals("embedaction"))
                    a = new EmbedTagAction(acs[z]);
                else if (tag.equals("insertdlgaction"))
                    a = new InsertDialogElementAction(acs[z]);
                else {
                    System.err.println("WARNING: unknown action type "+tag);
                    a = null;
                }
                if (a!=null)
                    xmlActions.addElement(a);
            }
            if (bm!=null)
                bm.installActions(xmlActions.elements());
        }
    }

    /*Options*********************************/
    public void setPaintAll(boolean pa) {
        if (paintAll!=pa) {
            paintAll = pa;
            getTextComponent().getUI().rebuild();
        }
    }
    public boolean getPaintAll() { return paintAll; }

    /*Paste helpers*******************************/
    protected Node findPasteSourceNode(Node root) {
        Node n = null;
        NodeList nl = root.getChildNodes();
        for (int z=0; z<nl.getLength(); z++) {
            n = nl.item(z);
            if (!(n instanceof Text))
                break;
        }
        return n;
    }
    protected boolean pasteEmbeddedManager(Document document1, int pos,
                                           EmbeddedManager src)
        throws IOException, SAXException, BadLocationException {
        SwingDocument document = (SwingDocument)document1;
        EmbeddedManager sem = document.getEmbeddedManager();
        // retrieve document
        Node pastedNode = src.getDocument();
        if (pastedNode==null)
            return false;
        // copy images
        String[] names = src.getNames();
        for (int z=0; z<names.length; z++)
            if (!sem.contains(names[z]))
                sem.replaceEntry(names[z], src.getEntry(names[z]));
        // FIXME: check for same root tag
        // find child node
        Node sourceNode = findPasteSourceNode(pastedNode);
        if (sourceNode==null)
            return false;
        // paste
        org.w3c.dom.Document xdoc = document.getXmlDocument();
        try {
            document.beginUpdate();
            document.paste(pos, DOMUtil.cloneNode(xdoc, sourceNode, true));
        } finally {
            document.endUpdate();
        }
        return true;
    }
    public boolean pasteDOM(Document document1, int pos,
                            org.w3c.dom.Document sdoc)
        throws BadLocationException {
        SwingDocument document = (SwingDocument)document1;
        org.w3c.dom.Element del = sdoc.getDocumentElement();
        if (del==null)
            return false;
        // FIXME: check for same root tag
        // find child node
        Node sourceNode = findPasteSourceNode(del);
        if (sourceNode==null)
            return false;
        // paste
        org.w3c.dom.Document xdoc = document.getXmlDocument();
        try {
            document.beginUpdate();
            document.paste(pos, DOMUtil.cloneNode(xdoc, sourceNode, true));
        } finally {
            document.endUpdate();
        }
        return true;
    }
    public boolean pastePlainText(Document document, int pos, Segment segment)
        throws BadLocationException {
        StringBuffer sb = new StringBuffer();
        for (int z=0; z<segment.count; z++) {
            final char ch = segment.array[segment.offset+z];
            if (Character.isISOControl(ch))
                sb.append(' ');
            else
                sb.append(ch);
        }
        if (sb.length()==0)
            return false;
        document.insertString(pos, sb.toString(), null);
        return true;
    }


    /*DefaultEditorKit*******************************/
    public void installActions(ButtonManager bm) {
        super.installActions(bm);
        bm.installActions(actions);
        synchronized(xmlActions) {
            bm.installActions(xmlActions.elements());
        }
    }
    public void uninstallActions(ButtonManager bm) {
        super.uninstallActions(bm);
        bm.uninstallActions(actions);
        synchronized(xmlActions) {
            bm.uninstallActions(xmlActions.elements());
        }
    }
    public Transferable copy(Document document1, int startPos, int endPos)
        throws BadLocationException {
        SwingDocument document = (SwingDocument)document1;

        // plain
        Segment plainSeg = new Segment();
        document.getText(startPos, endPos-startPos, plainSeg);
        final String plainString = new String
            (plainSeg.array, plainSeg.offset, plainSeg.count);

        // xml
        final org.w3c.dom.Node xdoc = document.copy(startPos, endPos);
        MemoryEmbeddedManager mem = new MemoryEmbeddedManager("[Clipboard]");
        final DocumentType docType = document.getDocumentType();
        if (docType==null) {
            mem = null;
        } else {
            try {
                mem.replaceDocument(xdoc);
            } catch (IOException e) {
                throw new InternalError(e.toString());
            }
            EmbeddedManager sem = document.getEmbeddedManager();
            String[] names = docType.getEmbeddedNames(xdoc);
            if (names.length==0)
                mem = null;
            else {
                int count = 0;
                for (int z=0; z<names.length; z++)
                    try {
                        mem.replaceEntry(names[z], sem.getEntry(names[z]));
                        count++;
                    } catch (IOException e) {
                    }
                if (count==0) mem = null;
            }
        } 
        final MemoryEmbeddedManager fem = mem;

        // create the result
        return new Transferable() {
                protected final DataFlavor[] flavors1 = new DataFlavor[] {
                    xmlFlavor,
                    DataFlavor.stringFlavor,
                };
                protected final DataFlavor[] flavors2 = new DataFlavor[] {
                    xmlFlavor,
                    DataFlavor.stringFlavor,
                    xmlJavaFlavor,
                };
		// internal
                protected int findFlavor(DataFlavor fl) {
                    for (int z=0; z<flavors2.length; z++)
                        if (flavors2[z].isMimeTypeEqual(fl))
                            return z;
                    return -1;
                }
		// Transferable
                public DataFlavor[] getTransferDataFlavors() {
                    return fem==null ? flavors1 : flavors2;
                }
                public boolean isDataFlavorSupported(DataFlavor flavor) {
                    int idx = findFlavor(flavor);
                    return idx!=-1 && (idx!=2 || fem!=null);
                }
                public Object getTransferData(DataFlavor flavor) 
                    throws UnsupportedFlavorException, IOException {
                    switch (findFlavor(flavor)) {
                    case 0: {
                        org.w3c.dom.Document xd = xdoc.getOwnerDocument();
                        byte[] b = DOMUtil.toByteArray(xd, Streams.defaultEncoding);
                        return new ByteArrayInputStream(b);
                    }
                    case 1: return plainString;
                    case 2: return fem;
                    default: return null;
                    }
                }
            };
    }
    public boolean paste(Document document, int pos, Transferable transferable)
        throws UnsupportedFlavorException, IOException, BadLocationException {

        DataFlavor[] dataFlavors = transferable.getTransferDataFlavors();

        // make.datamodel.embedded.EmbeddedManager
        if (transferable.isDataFlavorSupported(xmlJavaFlavor)) {
            EmbeddedManager embeddedManager = 
                (EmbeddedManager)transferable.getTransferData(xmlJavaFlavor);
            try {
                if (pasteEmbeddedManager(document, pos, embeddedManager))
                    return true;
            } catch (SAXException e) {
                Debug.report(e);
            }
        }

        // text/xml
        if (transferable.isDataFlavorSupported(xmlFlavor)) {
            try {
                Reader reader = xmlFlavor.getReaderForText(transferable);
                org.w3c.dom.Document cdoc = DOMUtil.parse(reader);
                if (pasteDOM(document, pos, cdoc))
                    return true;
            } catch (SAXException e) {
                Debug.report(e);
            }
        }

        // text/plain
        for (int i=0; i<dataFlavors.length; i++) {
            if (dataFlavors[i].getRepresentationClass()==java.io.InputStream.class && 
                dataFlavors[i].isMimeTypeEqual("text/plain")) {
                StringBuffer sb = new StringBuffer();
                Streams.append(sb, dataFlavors[i].getReaderForText(transferable));
                char[] data = new char[sb.length()];
                if (data.length==0)
                    continue;
                sb.getChars(0, data.length, data, 0);
                if (pastePlainText(document, pos, new Segment(data, 0, data.length)))
                    return true;
            }
        }

        // java.lang.String
        if (transferable.isDataFlavorSupported(DataFlavor.stringFlavor)) {
            String data = (String)transferable.getTransferData(DataFlavor.stringFlavor);
            document.insertString(pos, data, null);
            return true;
        }

        return false;
    }
    public Clipboard getClipboard() {
        return tec.getSession().getClipboard();
    }

    /*EditorKit**********************************/
    public ViewFactory getViewFactory() {
        return viewFactory;
    }
    public Document createDefaultDocument() {
        throw new InternalError();
    }

    /*Actions******************************************/
    private final Action[] actions = {
        //new InsertNoteAction(),
        new PaintAllAction(),
    };
    protected org.w3c.dom.Node createNodeInstance(String style) {
        Document _doc = editor.getDocument();
        SwingDocument doc = (SwingDocument)_doc;
        org.w3c.dom.Document xdoc = doc.getXmlDocument();
        Hashtable ht = doc.getStyle(style);
        if (ht!=null) {
            // clone?
            org.w3c.dom.Element nc =
                (org.w3c.dom.Element)ht.get(TextEditorStyleSet.NEW_CLONE);
            if (nc!=null)
                return DOMUtil.cloneNode(xdoc, nc, true);
            // transform?
            org.w3c.dom.Element xslt
                = (org.w3c.dom.Element)ht.get(TextEditorStyleSet.NEW_XSLT);
            if (xslt!=null) {
                DOMSource docSource = new DOMSource(xdoc);
                DOMSource styleSource = new DOMSource(xslt);
                org.w3c.dom.Document dest = DOMUtil.createDocument();
                DOMResult target = new DOMResult(dest);
                try {
                    TransformerFactory tFactory
                        = TransformerFactory.newInstance();
                    Transformer transformer = tFactory.newTransformer(styleSource);
                    transformer.transform(docSource, target);
                } catch (Exception e) {
                    e.printStackTrace();
                }
                org.w3c.dom.Element result1 = dest.getDocumentElement();
                if (result1!=null)
                    return DOMUtil.cloneNode(xdoc, result1, true);
            }
        }
        // create empty element
        return xdoc.createElement(style);
    }
    protected Element createElementInstance(String style) {
        Document _doc = editor.getDocument();
        SwingDocument doc = (SwingDocument)_doc;
        org.w3c.dom.Node n = createNodeInstance(style);
        return n==null
            ?null
            :doc.getSwingElementFactory().create(null, n);
    }
    final class FormatStyleAction extends DocumentAction {
        protected String style;
        public FormatStyleAction(org.w3c.dom.Element _el) {
            super(_el.getAttributeNode("command")==null
                  ?"format-"+_el.getAttribute("style")
                  :_el.getAttribute("command"));
            style = _el.getAttribute("style");
        }
        public void actionPerformed(Document doc, int start, int end, ActionEvent evt2) 
            throws BadLocationException {
            if (start==end) {
                if (style.equals(getCurrentFormat()))
                    setCurrentFormat(null);
                else {
                    boolean ok = false;
                    try {
                        ElementSwingElement root
                            = (ElementSwingElement)doc.getRootElement();
                        if (root.canInsertTag(start, Position.Bias.Forward, style)) {
                            setCurrentFormat(style);
                            ok = true;
                        }
                    } catch (BadLocationException e) {
                    }
                    if (!ok) {
                        JOptionPane.showMessageDialog
                            (tec.getFrame(),
                             "Die Formatierung kann an dieser Stelle nicht benutzt werden.",
                             "Formatierung", JOptionPane.WARNING_MESSAGE);
                        setCurrentFormat(null);
                    }
                }
            } else {
                Element root = doc.getRootElement();
                try {
                    doc.beginUpdate();
                    DocumentEvent evt = new DocumentEvent(doc);
                    if (root.isDeepStyle(start, Position.Bias.Forward, style) ||
                        root.isDeepStyle(end, Position.Bias.Backward, style))
                        root.removeStyle(style, start, end, evt);
                    else {
                        Element st = createElementInstance(style);
                        root.embedSplit(st, start, end, evt);
                    }
                    if (evt.isValid())
                        doc.fireDocumentChanged(evt);
                } finally {
                    doc.endUpdate();
                }
            }
        }
    }
    protected class InsertTagAction extends InsertElementAction {
        protected String style;
        public InsertTagAction(org.w3c.dom.Element _el) {
            super(_el.getAttributeNode("command")==null
                  ?"insert-"+_el.getAttribute("style")
                  :_el.getAttribute("command"));
            style = _el.getAttribute("style");
        }
        protected Element createElement(Document adoc) {
            SwingDocument doc = (SwingDocument)adoc;
            return createElementInstance(style);
        }
    }
    final class EmbedTagAction extends EmbedElementAction {
        protected String style;
        public EmbedTagAction(org.w3c.dom.Element _el) {
            super(_el.getAttributeNode("command")==null
                  ?"embed-"+_el.getAttribute("style")
                  :_el.getAttribute("command"));
            style = _el.getAttribute("style");
        }
        public Element createElement(Document adoc) {
            SwingDocument doc = (SwingDocument)adoc;
            return createElementInstance(style);
        }
    }
    final class InsertDialogElementAction extends DocumentAction {
        protected String style;
        public InsertDialogElementAction(org.w3c.dom.Element _el) {
            super(_el.getAttributeNode("command")==null
                  ?"insert-"+_el.getAttribute("style")
                  :_el.getAttribute("command"));
            style = _el.getAttribute("style");
        }
        public void actionPerformed(Document doc, int start, int end, ActionEvent evt2) 
            throws BadLocationException {
            Hashtable ht = doc.getStyle(style);
            if (ht==null) {
                beep(); return;
            }
            org.w3c.dom.Element dinfo
                = (org.w3c.dom.Element)ht.get(TextEditorStyleSet.EDITOR_DIALOG);
            if (dinfo==null) {
                beep(); return;
            }
            ElementSwingElement root = (ElementSwingElement)doc.getRootElement();
            if (root.canInsertTag(start, Position.Bias.Forward, style))
                DefaultEditorDialog.createNewElement
                    (tec.getEditorFrame(), 
                     (SwingDocument)doc, style, dinfo, start, end);
            else
                JOptionPane.showMessageDialog
                    (null, "Das Element kann an dieser Stelle nicht eingef\u00fcgt werden.",
                     "Einf\u00fcgen", JOptionPane.ERROR_MESSAGE);
        }
    }
    public class PaintAllAction extends AbstractAction {
        public PaintAllAction() { super(paintAllAction); }
        public void actionPerformed(ActionEvent e) {
            XmlTextPane ed = (XmlTextPane)getTextComponent();
            setPaintAll(!getPaintAll());
        }
    }
}
