/*
 *
 *  LeanEdit
 *  Copyright (c) 1999-2001 Max Kellermann (max.kellermann@epost.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: TextEditorComponent.java,v 1.5 2001/10/17 22:35:06 max Exp $
 *
 */

package org.leanedit.plugins.text;

import java.awt.Component;
import javax.swing.JScrollPane;
import org.w3c.dom.Element;
import org.leanedit.plugins.text.tree.SwingDocument;
import org.leanedit.plugins.DefaultEditorComponent;
import org.leanedit.EditorDocument;
import org.leanedit.EditorFrame;
import make.xml.DOMUtil;
import make.util.Debug;
import make.swing.ButtonManager;
import make.swing.text.DocumentListener;
import make.swing.text.DocumentEvent;
import make.swing.text.Document;

public class TextEditorComponent extends DefaultEditorComponent 
    implements DocumentListener {
    class ViewDescription {
	public boolean hasScroller;
	public Element actions;
	public TextEditorStyleSet styleSet;
	public ViewDescription(Element vd) {
	    hasScroller = !DOMUtil.childTagExists(vd, "noscroll");
	    actions = DOMUtil.getUniqueTag(vd, "actions");
	    Element styles = DOMUtil.getUniqueTag(vd, "styles");
	    styleSet = new TextEditorStyleSet(styles);
	}
    }
    /*properties*****************************/
    // visual
    protected Component component;
    protected XmlTextPane bodyEditor;
    protected TextEditorStyleSet styleSet;
    protected boolean hasScroller = true;
    protected boolean hasFocus = false;
    // data
    protected SwingDocument bodyDocument;
    protected TextActions textActions;
    /*Constructor***************************/
    public TextEditorComponent(EditorFrame _ef) {
	super(_ef);
	textActions = new TextActions(this, _ef);
	if (Debug.LEAK_DEBUG)
	    System.err.println("+txe "+this);
    }
    protected void finalize() throws Throwable {
	super.finalize();
	if (Debug.LEAK_DEBUG)
	    System.err.println("-txe "+this);
    }
    /*DocumentListener*******************************/
    public void documentChanged(DocumentEvent evt) {
	fireDocumentChanged();
    }
    /*EditorComponent********************************/
    public void setDocument(EditorDocument _doc) {
	super.setDocument(_doc);
	// read view
	Object vd = getViewDescription("text");
	if (vd instanceof Element) {
	    vd = new ViewDescription((Element)vd);
	    putViewDescription("text", vd);
	}
	if (vd==null) {
	    styleSet = new TextEditorStyleSet(null);
	} else {
	    ViewDescription d = (ViewDescription)vd;
	    hasScroller = d.hasScroller;
	    styleSet = d.styleSet;
	    ButtonManager bm = hasFocus
		?getEditorFrame().getInterfacePanel().getButtonManager()
		:null;
	    textActions.loadActions(d.actions, bm);
	}
	// create swingDocument
	if (bodyDocument!=null)
	    bodyDocument.removeDocumentListener(this);
	bodyDocument = new SwingDocument(document, styleSet);
	bodyDocument.addDocumentListener(this);
	if (bodyEditor==null) {
	    final Component oldComponent = component;
	    bodyEditor = new XmlTextPane(textActions, bodyDocument, 
					 styleSet, editorFrame);
	    registerInputComponent(bodyEditor);
	    if (hasScroller) {
		JScrollPane scrollPane = new JScrollPane(bodyEditor);
		scrollPane.getViewport().putClientProperty("EnableWindowBlit", Boolean.TRUE);
		component = scrollPane;
	    } else
		component = bodyEditor;
	    bodyEditor.setDocument(bodyDocument);
	    firePropertyChange(COMPONENT, oldComponent, component);
	} else {
	    bodyEditor.setDocument(bodyDocument);
	}
    }
    public Component getComponent() { return component; }
    public void focusGained() {
	super.focusGained();
	bodyEditor.setButtonManager(getEditorFrame().getInterfacePanel().getButtonManager());
	hasFocus = true;
    }
    public void focusLost() {
	super.focusLost();
	bodyEditor.setButtonManager(null);
	hasFocus = false;
    }
    public void dispose() {
	bodyEditor.setButtonManager(null);
	component = null;
	bodyEditor = null;
	styleSet = null;
	bodyDocument = null;
	super.dispose();
    }
}
