/*
 *
 *  LeanEdit
 *  Copyright (c) 1999-2001 Max Kellermann (max.kellermann@epost.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: XmlTextPane.java,v 1.4 2001/10/17 22:33:49 max Exp $
 *
 */

package org.leanedit.plugins.text;

import java.awt.Font;
import org.leanedit.plugins.text.tree.StyleElement;
import org.leanedit.plugins.text.tree.SwingText;
import org.leanedit.plugins.text.tree.SwingDocument;
import org.leanedit.EditorFrame;
import make.swing.text.TextComponent;
import make.swing.text.EditorKit;
import make.swing.text.Document;
import make.swing.text.Element;
import make.swing.text.TextUI;
import make.swing.text.BasicTextUI;

public class XmlTextPane extends TextComponent {
    protected class XmlTextUI extends BasicTextUI {
	protected XmlTextUI(EditorKit ek) {
	    super(ek);
	}
	public Font getFont(TextComponent t, Element el) {
	    return styles.getFont(el);
	}
    }
    /*properties**************************/
    protected EditorFrame editorFrame;
    protected TextEditorStyleSet styles;
    /*Constructor******************************/
    public XmlTextPane(EditorKit _ek, Document _doc, 
		       TextEditorStyleSet _ss, EditorFrame _ef) { 
	super(_ek);
	editorFrame = _ef;
	styles = _ss;
	setDocument(_doc);
    }
    /*info*********************************************/
    public EditorFrame getEditorFrame() { return editorFrame; }
    /*TextComponent************************************/
    protected TextUI createUI() {
	return new XmlTextUI(getEditorKit());
    }
    public void replaceSelection(String content, String style) {
	if (content!=null && style!=null) {
	    StyleElement st = new StyleElement((SwingDocument)getDocument(), style);
	    st.insertChild(new SwingText((SwingDocument)getDocument(), content, false), null);
	    super.replaceSelection(st);
	} else
	    super.replaceSelection(content, style);
    }
    /*Scrollable******************************************/
    public boolean getScrollableTracksViewportWidth() { return true; }
    public boolean getScrollableTracksViewportHeight() { return false; }
}
