/*
 *
 *  LeanEdit
 *  Copyright (c) 1999-2001 Max Kellermann (max.kellermann@epost.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: DefaultEditorDialog.java,v 1.14 2001/12/22 12:14:59 max Exp $
 *
 */

package org.leanedit.plugins.text.dlg;

import java.util.Locale;
import java.text.ParseException;
import java.awt.Frame;
import java.awt.Component;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import javax.swing.JTextField;
import javax.swing.text.BadLocationException;
import make.util.Strings;
import make.util.Debug;
import make.xml.DOMUtil;
import make.xml.ElementType;
import make.xml.AttributeType;
import make.datamodel.embedded.EmbeddedManager;
import make.datamodel.slot.Slot;
import make.datamodel.slot.SlotContainer;
import make.swing.field.Field;
import make.swing.il.InputLink;
import make.swing.il.InputLinkFactory;
import make.swing.il.DefaultInputLink;
import make.swing.il.PanelInputLink;
import make.swing.text.DocumentEvent;
import make.swing.panel.InterfacePanel;
import make.swing.panel.DialogInterfacePanel;
import org.leanedit.EditorFrame;
import org.leanedit.EditorDocument;
import org.leanedit.field.EditorInputLinkFactory;
import org.leanedit.plugins.text.tree.SwingDocument;
import org.leanedit.plugins.text.tree.SwingText;
import org.leanedit.plugins.text.tree.ElementSwingElement;

public class DefaultEditorDialog extends EditorDialog {
    /*Editors***********************************/
    public class AttributeSlot implements Slot {
        /*properties*****************************/
        protected ElementSwingElement element;
        protected String attributeName;
        protected AttributeType type;
        /*Constructor*****************************/
        public AttributeSlot(ElementSwingElement _el, String _an) {
            element = _el; attributeName = _an;
            ElementType et = element.getElementType();
            if (et!=null)
                type = et.getAttribute(_an);
        }
        /*Slot**********************************/
        public void setValue(Object v) throws ParseException {
            if (Debug.DEBUG && documentEvent==null && !inCheck)
                throw new InternalError("setValue without documentEvent");
            if (v==null && type!=null && type.isRequired())
                throw new ParseException("Bitte geben Sie hier einen Wert ein.", 0);
            if (!inCheck)
                element.setAttribute(attributeName, v==null?null:v.toString(),
                                     documentEvent);
        }
        public Object getValue() throws ParseException {
            return element.getAttribute(attributeName);
        }
    }
    class ContentEditor extends DefaultInputLink {
        /*properties**************/
        protected JTextField field = new JTextField(20);
        protected String value;
        /*Constructor************/
        public ContentEditor(Element vd, Locale locale) {
            super(vd, locale);
            reload();
        }
        /*Widget****************************/
        public void setEnabled(boolean _e) {}
        public boolean getEnabled() { return true; }
        public Component getComponent() { return field; }
        /*InputLink*********************/
        public void reload() {
            if (newPosition==-1 || (newSelection!=-1 && newSelection!=newPosition)) {
                field.setText("[Markierung]");
                field.setEnabled(false);
            } else {
                field.setText("");
                field.setEnabled(true);
            }
        }
        public void apply() throws ParseException {
            if (field.isEnabled()) {
                ElementSwingElement el = getElement();
                SwingDocument doc = (SwingDocument)el.getDocument();
                SwingText txt = new SwingText(doc.getXmlDocument().createTextNode(value),
                                              doc, false);
                el.insertChild(txt, null);
                if (documentEvent!=null)
                    documentEvent.createContainerChange(el).childAdded(el.indexOf(txt));
                field.setEnabled(false);
            }
        }
        public void checkInput() throws ParseException {
            if (isNew()) {
                value = Strings.trim(field.getText());
                if (value==null)
                    throw new ParseException
                        ("Bitte geben Sie bei hier einen Text ein.", 0);
            }
        }
    }
    class AttributeSlotContainer implements SlotContainer {
        public Slot getSlot(Object key) {
            if (key instanceof String)
                return new AttributeSlot(getElement(), (String)key);
            else
                return null;
        }
    }
    class Factory extends EditorInputLinkFactory implements SlotContainer {
        public Factory(EditorFrame frame, SlotContainer _sc) {
            super(frame, getEditorDocument(), _sc);
        }
        /*InputLinkFactory*********************/
        public InputLink createInputLink(Element vd, Locale locale) {
            String type = getType(vd);
            if (type.equals("content"))
                return new ContentEditor(vd, locale);
            else
                return super.createInputLink(vd, locale);
        }
        public Slot getSlot(Object key) {
            if (key instanceof String)
                return new AttributeSlot(getElement(), (String)key);
            else
                return null;
        }
    }
    /*properties******************************/
    protected EmbeddedManager embeddedManager;
    protected EditorFrame frame;
    protected EditorDocument document;
    protected DocumentEvent documentEvent;
    protected boolean inCheck = false;
    protected InputLink inputLink;
    /*Constructor******************************/
    public DefaultEditorDialog(InterfacePanel _ip, EditorFrame _ef, 
                               SwingDocument _doc, 
                               String _tag, Element dinfo, 
                               int _np, int _ns) {
        super(_ip, 
              Strings.trim(dinfo.getAttribute("newtitle"))==null
              ?dinfo.getAttribute("title"):dinfo.getAttribute("newtitle"),
              _doc, _tag, _np, _ns);
        frame = _ef;
        document = _doc.getEditorDocument();
        embeddedManager = _doc.getEmbeddedManager();
        createForm(dinfo);
    }
    public DefaultEditorDialog(InterfacePanel _ip, EditorFrame _ef, 
                               ElementSwingElement _el, Element dinfo) {
        super(_ip, 
              dinfo.getAttribute("title"), _el);
        frame = _ef;
        document = ((SwingDocument)_el.getDocument()).getEditorDocument();
        embeddedManager = ((SwingDocument)_el.getDocument()).getEmbeddedManager();
        createForm(dinfo);
    }
    /*Data*****************************************/
    protected boolean applyChanges() {
        if (documentEvent!=null)
            return false;
        ElementSwingElement el = getElement();
        // 1. check
        try {
            inCheck = true;
            inputLink.checkInput();
        } catch (ParseException e) {
            inputErrorMsg(e.getLocalizedMessage());
            return false;
        } finally {
            inCheck = false;
        }
        // 2. set values
        SwingDocument doc = (SwingDocument)el.getDocument();
        try {
            doc.beginUpdate();
            documentEvent = new DocumentEvent(doc);
            try {
                inputLink.apply();
            } catch (ParseException e) {
            }
            if (isNew()) {
                try {
                    ElementType eet = getElement().getElementType();
                    if (newPosition==newSelection || (eet!=null && eet.isEmpty())) {
                        if (!doc.getRootElement().pasteElementAt(newPosition, el, 
                                                                 documentEvent)) {
                            errorMsg("Hier koennen Sie dieses Element nicht einfuegen.", "Fehler");
                            return false; //TODO
                        }
                    } else {
                        if (!doc.getRootElement().embedSplit(el, newPosition, 
                                                             newSelection, documentEvent)) {
                            inputErrorMsg("Hier koennen Sie dieses Element nicht einfuegen.");
                            doc.endUpdate();
                            return false; //TODO: Attribute, undo rueckgaengig
                        }
                    }
                } catch (BadLocationException e) {
                    e.printStackTrace(); return false;
                }
            }
            if (documentEvent.isValid())
                doc.fireDocumentChanged(documentEvent);
        } finally {
            documentEvent = null;
            doc.endUpdate();
        }
        return true;
    }
    /*Form************************************/
    protected synchronized void createForm(Element dinfo) {
        ElementSwingElement element = getElement();
        Factory factory = new Factory(frame, new AttributeSlotContainer());
        inputLink = new PanelInputLink
            (dinfo,
             DOMUtil.getChildElementsNS(dinfo, InputLinkFactory.XML_NAMESPACE), 
             frame.getSession().getLocale(),
             factory);
        inputLink.install(interfacePanel.getButtonManager());
        // TODO:firstField aktivieren
        /*if (firstField!=null)
          firstField.requestFocus();*/
    }
    /*PanelContent**********************/
    public Component getComponent() {
        return inputLink.getComponent();
    }
    public void panelClosed() {
        inputLink.uninstall(interfacePanel.getButtonManager());
        super.panelClosed();
    }
    /*Static***************************************/
    public static void createNewElement(EditorFrame _f, 
                                        SwingDocument _doc, String _tag,
                                        Element dinfo, int _np, int _ns) {
        InterfacePanel ip = new DialogInterfacePanel
            (_f.getInterfacePanel().getFrame());
        DefaultEditorDialog gre = new DefaultEditorDialog
            (ip, _f, _doc, _tag, dinfo, _np, _ns);
        ip.setContent(gre);
        ip.setVisible(true);
    }
    public static void editElement(EditorFrame _f, 
                                   ElementSwingElement _el, Element dinfo) {
        InterfacePanel ip = new DialogInterfacePanel
            (_f.getInterfacePanel().getFrame());
        DefaultEditorDialog gre = new DefaultEditorDialog
            (ip, _f, _el, dinfo);
        ip.setContent(gre);
        ip.setVisible(true);
    }
}
