/*
 *
 *  LeanEdit
 *  Copyright (c) 1999-2001 Max Kellermann (max.kellermann@epost.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: EditorDialog.java,v 1.6 2001/12/22 09:02:53 max Exp $
 *
 */

package org.leanedit.plugins.text.dlg;

import java.awt.Component;
import java.awt.event.ActionEvent;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import javax.swing.Action;
import javax.swing.AbstractAction;
import javax.swing.JButton;
import javax.swing.JOptionPane;
import make.xml.ElementType;
import make.swing.bt.FrameDescription;
import make.swing.text.DocumentEvent;
import make.swing.panel.InterfacePanel;
import make.swing.panel.PanelContent;
import org.leanedit.EditorDocument;
import org.leanedit.plugins.text.tree.ElementSwingElement;
import org.leanedit.plugins.text.tree.SwingDocument;

public abstract class EditorDialog implements PanelContent {
    /*static*****************************/
    protected static final FrameDescription frameDescription
        = FrameDescription.okCancelDescription;
    /*properties***************************/
    protected InterfacePanel interfacePanel;
    private ElementSwingElement element;
    protected int newPosition = -1, newSelection = -1;
    protected JButton defaultButton;
    /*Constructor*************************/
    private EditorDialog(InterfacePanel _ip, String title) {
        interfacePanel = _ip;
        interfacePanel.setTitle(title);
        interfacePanel.addFrameDescription(frameDescription);
        interfacePanel.getButtonManager().installActions(actions);
    }
    protected EditorDialog(InterfacePanel _ip, String title, ElementSwingElement _el) {
        this(_ip, title);
        element = _el;
    }
    protected EditorDialog(InterfacePanel _ip, String title, SwingDocument doc, 
                           String tag, int _np, int _ns) {
        this(_ip, title);
        element = (ElementSwingElement)doc.getSwingElementFactory().create
            (doc.getRootElement(), doc.getXmlDocument().createElement(tag)); 
        newPosition = _np; newSelection = _ns;
    }
    protected EditorDialog(InterfacePanel _ip, String title, SwingDocument doc, 
                           String tag, int _np) {
        this(_ip, title, doc, tag, _np, -1);
    }
    /*Info***********************************/
    public boolean isNew() { return newPosition!=-1; }
    public EditorDocument getEditorDocument() {
        return ((SwingDocument)element.getDocument()).getEditorDocument();
    }
    /*Data************************************/
    protected void fireDefaultDocumentChanged() {
        DocumentEvent evt = new DocumentEvent(element.getDocument());
        evt.createParentChange(element);
        element.getDocument().fireDocumentChanged(evt);
    }
    protected String getAttribute(String name) {
        return element.getAttribute(name);
    }
    protected String getAttribute(String name, String def) {
        String val = getAttribute(name);
        return val==null?def:val;
    }
    protected void setAttribute(String name, String val, DocumentEvent evt) {
        element.setAttribute(name, val, evt);
    }
    protected ElementSwingElement getElement() { return element; }
    protected ElementType getElementType() { return element.getElementType(); }
    /*EditorDialog************************/
    protected abstract boolean applyChanges();
    /*PanelContent**********************/
    public abstract Component getComponent();
    public boolean panelClosing() {
        return true;
    }
    public void panelClosed() {
        interfacePanel.removeFrameDescription(frameDescription);
        interfacePanel.getButtonManager().uninstallActions(actions);
    }
    /*Message*****************************************/
    public void errorMsg(String msg, String title) {
        JOptionPane.showMessageDialog(interfacePanel.getFrame(), 
                                      msg, title,
                                      JOptionPane.ERROR_MESSAGE);
    }
    public void inputErrorMsg(String msg) {
        errorMsg(msg, "Eingabefehler");
    }
    /*Actions***************************/
    protected Action[] actions = new Action[]{
        new OkAction(),
        new CancelAction(),
    };
    class OkAction extends AbstractAction implements Runnable {
        OkAction() { super("ok"); }
        public void actionPerformed(ActionEvent evt) {
            new Thread(this, "ok").start();
        }
        public void run() {
            if (applyChanges())
                interfacePanel.close();
        }
    }
    class CancelAction extends AbstractAction {
        CancelAction() { super("cancel"); }
        public void actionPerformed(ActionEvent evt) {
            interfacePanel.close();
        }
    }
}
