/*
 *
 *  LeanEdit
 *  Copyright (c) 1999-2001 Max Kellermann (max.kellermann@epost.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: NoteElement.java,v 1.3 2001/10/30 20:55:33 max Exp $
 *
 */

package org.leanedit.plugins.text.tree;

import java.text.MessageFormat;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.text.Position;
import javax.swing.text.BadLocationException;
import make.swing.text.Element;
import make.swing.text.DocumentEvent;
import make.swing.text.Document;
import make.swing.text.SplitNotSupportedException;
import make.swing.text.TextComponent;

public class NoteElement extends ElementSwingElement {
    /***properties******************************/
    FixedText prefix, suffix;
    /***Constructor********************************/
    public NoteElement(org.w3c.dom.Element _el, SwingDocument _doc) {
	super(_el, _doc, 0);
	prefix = new FixedText(_doc, "[");
	suffix = new FixedText(_doc, "]");
	insertChild(prefix, 0);
	insertChild(suffix, null);
	prefix.setStyle(_doc.getStyle("emphasized"));
	suffix.setStyle(_doc.getStyle("emphasized"));
	leadingSpace = false; endingSpace = false;
    }
    /***Menu*******************************************/
    public void fillMenu(TextComponent editor, int context, 
                         int pos, Position.Bias bias, JPopupMenu menu) 
        throws BadLocationException {
        String menuItemLabel = MessageFormat.format
            (TextComponent.resources.getString("remove.text"),
             new Object[]{ TextComponent.resources.getString("note.word") });
        JMenuItem menuItem = new JMenuItem(menuItemLabel);
        menuItem.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent evt2) {
                    Document doc = getDocument();
                    try {
                        doc.beginUpdate();
                        Element parent = getParent();
                        DocumentEvent evt = new DocumentEvent(doc);
                        final int idx = parent.indexOf(NoteElement.this);
                        evt.removed(parent, getStartOffset(), getLength());
                        parent.removeChild(idx);
                        evt.createContainerChange(parent).childRemoved(idx);
                        getDocument().fireDocumentChanged(evt);
                    } finally {
                        doc.endUpdate();
                    }
                }
            });
        menu.add(menuItem);
	super.fillMenu(editor, context, pos, bias, menu);
    }
    /***Element****************************************/
    public boolean childConservationPossible(Element child) {
	return false;
    }
    public boolean canInsertAt(int pos) throws BadLocationException {
	return super.canInsertAt(pos) &&
	    pos>getStartOffset() && pos<getEndOffset();
    }
    protected Element createEmptyInstance(org.w3c.dom.Element el) {
	return new NoteElement(el, (SwingDocument)getDocument());
    }
    public Element split(int offset, DocumentEvent evt) 
	throws SplitNotSupportedException {
	throw new SplitNotSupportedException(this, offset);
    }
    public void remove(int offset, int length, DocumentEvent evt) 
	throws BadLocationException {
	if (offset==getStartOffset() && length==1)
	    offset++;
	if (offset==(getEndOffset()-1) && length==1)
	    offset--;
	if (length>=(getLength()-1) &&
	    (offset==prefix.getEndOffset() || (offset+length)==suffix.getStartOffset())) {
	    final Element parent = getParent();
	    final int idx = parent.indexOf(this);
	    parent.removeChild(idx);
	    evt.createContainerChange(parent).childRemoved(idx);
	    evt.removed(parent, offset, length);
	} else {
	    if (offset==getStartOffset()) {
		offset = getStartOffset()+1; length--;
	    }
	    if ((offset+length)==getEndOffset())
		length--;
	    super.remove(offset, length, evt);
	}
    }
    public boolean insertString(int offset, String str, DocumentEvent evt) 
	throws BadLocationException {
	return offset>=prefix.getEndOffset() && offset<=suffix.getStartOffset()
	    ?super.insertString(offset, str, evt)
	    :false;
    }
}
