/*
 *
 *  LeanEdit
 *  Copyright (c) 1999-2001 Max Kellermann (max.kellermann@epost.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: NumberedList.java,v 1.3 2001/09/20 20:20:53 max Exp $
 *
 */

package org.leanedit.plugins.text.tree;

import make.swing.text.Element;
import make.swing.text.DocumentEvent;

public class NumberedList extends Section {
    /***properties*******************************/
    protected int first = -1;
    protected char type = 0;
    protected String suffix = null;
    /***Constructor*********************************/
    public NumberedList(org.w3c.dom.Element _el, SwingDocument _doc) {
	super(_el, _doc, 0);
    } 
    /***NumberedList*******************************/
    protected void syncNumberedList() {
	org.w3c.dom.Element el = getXmlElement();
	try {
	    first = Integer.parseInt(el.getAttribute("first"));
	} catch (NumberFormatException e) {
	    first = 1;
	}
	String t = el.getAttribute("type");
	type = t==null || t.length()==0?'1':t.charAt(0);
	suffix = el.getAttribute("suffix");
    }
    public void invalidateNumberedList() {
	first = -1; type = 0; suffix = null;
    }
    public String formatIndex(int idx) {
	if (first==-1)
	    syncNumberedList();
	String s;
	int n = first+idx;
	switch (type) {
	case 'a':
	    s = "";
	    while (n>0) {
		s = (char)('a'+(n%26)-1) + s;
		n /= 26;
	    }
	    break;
	case 'A':
	    s = "";
	    while (n>0) {
		s = (char)('A'+(n%26)-1) + s;
		n /= 26;
	    }
	    break;
	default:
	    s = ""+n;
	}
	if (suffix!=null)
	    s += suffix;
	return s;
    }
    /***Format****************************************/
    protected Element createEmptyInstance(org.w3c.dom.Element el) {
	return new NumberedList(el, (SwingDocument)getDocument());
    }
    /***ElementSwingElement*****************************/
    public void setAttribute(String name, String value, DocumentEvent evt) {
	super.setAttribute(name, value, evt);
	invalidateNumberedList();
    }
} 
