/*
 *
 *  LeanEdit
 *  Copyright (c) 1999-2001 Max Kellermann (max.kellermann@epost.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: StyleElement.java,v 1.4 2001/10/30 20:56:13 max Exp $
 *
 */

package org.leanedit.plugins.text.tree;

import java.text.MessageFormat;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.text.Position;
import javax.swing.text.BadLocationException;
import make.swing.text.Element;
import make.swing.text.Document;
import make.swing.text.DocumentEvent;
import make.swing.text.TextComponent;

public class StyleElement extends ElementSwingElement {
    /***Constructor********************************/
    public StyleElement(org.w3c.dom.Element _el, SwingDocument _doc) {
	super(_el, _doc, 0);
    }
    public StyleElement(SwingDocument _doc, String tag) {
	super(((NodeSwingElement)_doc.getRootElement()).getXmlDocument().createElement(tag), 
	      _doc, 0);
    }
    /***Format****************************************/
    public boolean isStyle(String style) {
	return getTagName().equals(style);
    }
    protected Element createEmptyInstance(org.w3c.dom.Element el) {
	return new StyleElement(el, (SwingDocument)getDocument());
    }
    /***Menu*******************************************/
    public void fillMenu(TextComponent editor, int context, 
                         int pos, Position.Bias bias, JPopupMenu menu) 
        throws BadLocationException {
        final String tag = getTagName();
        // remove style
        String styleName = (String)getAttribute((Object)"title");
        if (styleName==null)
            styleName = '<'+tag+'>';
        String menuItemLabel = MessageFormat.format
            (TextComponent.resources.getString("remove_style.text"),
             new Object[]{ styleName });
        JMenuItem menuItem = new JMenuItem(menuItemLabel);
        menuItem.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent evte) {
                    Document doc = getDocument();
                    try {
                        doc.beginUpdate();
                        Element parent = getParent();
                        final int idx = parent.indexOf(StyleElement.this);
                        int first=idx, last=idx;
                        final int cnt1 = parent.getElementCount()-1;
                        while (first>0 && 
                               (parent.getElementAtIndex(--first).getLength()==0 ||
                                parent.getElementAtIndex(first).isStyle(tag)));
                        while (last<cnt1 && 
                               (parent.getElementAtIndex(++last).getLength()==0 ||
                                parent.getElementAtIndex(last).isStyle(tag)));
                        DocumentEvent evt = new DocumentEvent(doc);
                        Element root = doc.getRootElement();
                        root.removeStyle
                            (tag, 
                             parent.getElementAtIndex(first).getStartOffset(),
                             parent.getElementAtIndex(last).getEndOffset(),
                             evt);
                        if (evt.isValid())
                            doc.fireDocumentChanged(evt);
                    } catch (BadLocationException e) {
                        e.printStackTrace();
                    } finally {
                        doc.endUpdate();
                    }
                }
            });
        menu.add(menuItem);
        super.fillMenu(editor, context, pos, bias, menu);
   }
    /***Info****************************************/
    public String toString() {
	return "["+super.toString()+": style='"+getXmlNode().getNodeName()+"']";
    }
}
