/*
 *
 *  LeanEdit
 *  Copyright (c) 1999-2001 Max Kellermann (max.kellermann@epost.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: SwingEntityElement.java,v 1.3 2001/09/20 20:20:53 max Exp $
 *
 */

package org.leanedit.plugins.text.tree;

import javax.swing.text.BadLocationException;
import org.w3c.dom.Node;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Entity;
import org.w3c.dom.EntityReference;
import make.swing.text.Element;
import make.swing.text.Document;
import make.swing.text.DocumentEvent;
import make.swing.text.SplitNotSupportedException;

public class SwingEntityElement extends NodeSwingElement {
    /***properties*******************************/
    protected Entity entity;
    /***Constructor********************************/
    public SwingEntityElement(Entity _el, SwingDocument _doc) {
	super(_el, _doc, 0, "entity");
    }
    public SwingEntityElement(EntityReference _el, SwingDocument _doc) {
	super(_el, _doc, 0, "entity");
    }
    protected void loadChildren() {
	if (getXmlNode() instanceof Entity) {
	    entity = (Entity)getXmlNode();
	} else {
	    EntityReference ref = (EntityReference)getXmlNode();
	    NamedNodeMap nnm = ref.getOwnerDocument().getDoctype().getEntities();
	    entity = (Entity)nnm.getNamedItem(ref.getNodeName());
	}
	loadChildren(entity.getChildNodes());
    }
    /***Format****************************************/
    protected Element createEmptyInstance() {
	throw new InternalError("can't split entity");
    }
    /***Edit******************************************/
    public boolean embedSplit(Element parent, int start, int end,
			      DocumentEvent evt) {
	return false;
    }
    public Element split(int offset, DocumentEvent evt) 
	throws SplitNotSupportedException {
	throw new SplitNotSupportedException(this, offset);
    }
    public boolean insertString(int offset, String str, DocumentEvent evt) 
	throws BadLocationException {
	return false;
    }
    public void remove(int offset, int length, DocumentEvent evt) 
	throws BadLocationException {
    }
    /***NodeSwingElement*****************************/
    public Node copyXmlNode(org.w3c.dom.Document doc, int start, int end) 
	throws BadLocationException { 
	return doc.createEntityReference(entity.getNodeName());
    }
    public boolean pasteNode(int pos, Node n, DocumentEvent evt) 
	throws BadLocationException {
	return false;
    }
}
