/*
 *
 *  LeanEdit
 *  Copyright (c) 1999-2001 Max Kellermann (max.kellermann@epost.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: TableCellElement.java,v 1.3 2001/09/20 20:20:53 max Exp $
 *
 */

package org.leanedit.plugins.text.tree;

import org.w3c.dom.Text;
import javax.swing.text.BadLocationException;
import make.util.Debug;
import make.swing.text.Element;
import make.swing.text.Document;
import make.swing.text.DocumentEvent;
import make.swing.text.ContainerChange;

public class TableCellElement extends ElementSwingElement {
    public static class CellEnd extends Element {
	public CellEnd(Document _doc) {
	    super(_doc, 1, true);
	}
	/***Element*********************************/
	public String getName() { return "CellEnd"; }
	public void getText(StringBuffer sb, int offset, int length) {
	    if (length>=1)
		sb.append("\t");
	}
	protected int insertSpaceChild(int index, DocumentEvent evt,
				       ContainerChange chg) {
	    throw new InternalError();
	}
	/***Format****************************************/
	protected Element createEmptyInstance() {
	    throw new IllegalStateException();
	}
    }
    /***properties*********************************/
    protected CellEnd cellEnd;
    /***Constructor*************************/
    public TableCellElement(org.w3c.dom.Element _e, SwingDocument _doc) {
	super(_e, _doc, 0);
	insertChild(cellEnd = new CellEnd(_doc), null);
    }
    /***Info******************************************/
    public boolean childConservationPossible(Element child) {
	if (child==cellEnd)
	    return false;
	return super.childConservationPossible(child);
    }
    public boolean canInsertAt(int pos) throws BadLocationException {
	return super.canInsertAt(pos) &&
	    pos<=cellEnd.getStartOffset();
    }
    public boolean ignoreOuterSpaces() { return true; }
    /***Format****************************************/
    protected Element createEmptyInstance(org.w3c.dom.Element el) { 
	throw new IllegalStateException(); 
    }
    /***Element*****************************************/
    public void removeChild(int index) {
	if (Debug.DEBUG && index>=(getElementCount()-1))
	    throw new InternalError("tried to remove cellEnd");
	super.removeChild(index);
    }
    public boolean insertString(int offset, String str, DocumentEvent evt) 
	throws BadLocationException {
	if (offset>=getEndOffset())
	    return false;
	else
	    return super.insertString(offset, str, evt);
    }
}
