/*
 *
 *  LeanEdit
 *  Copyright (c) 1999-2001 Max Kellermann (max.kellermann@epost.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: TableRowView.java,v 1.3 2001/09/20 20:20:53 max Exp $
 *
 */

package org.leanedit.plugins.text.view;

import javax.swing.text.BadLocationException;
import org.leanedit.plugins.text.tree.ElementSwingElement;
import org.leanedit.plugins.text.tree.TableRowElement;
import make.swing.text.Element;
import make.swing.text.view.View;
import make.swing.text.view.ViewFactory;
import make.swing.text.view.BoxView;
import make.swing.text.view.EmptyView;
import make.swing.text.view.VerticalView;

class TableRowView extends BoxView implements VerticalView {
    /***properties************************************/
    protected TableView table;
    protected TableRowElement tableRow;
    protected ElementSwingElement[] cells;
    /***Constructor************************************/
    public TableRowView(TableRowElement _el, int axis) {
	super(_el, axis);
	tableRow = _el;
    }
    protected void loadChildren(ViewFactory f) {
	table = (TableView)getParent();
	cells = table.extractCells(tableRow);
	View[] v = new View[cells.length];
	int start = getStartOffset();
	for (int z=0; z<v.length; z++) {
	    ElementSwingElement el = cells[z];
	    if (el==null)
		v[z] = new EmptyView(getElement(), start);
	    else {
		v[z] = f.create(el);
		start = v[z].getEndOffset();
	    }
	}
	replace(0, 0, v);
    }
    /***CompositeView***************************************/
    protected void layoutMajorAxis(int targetSpan, int axis, int[] offsets, int[] spans) {
	final int count = cells.length;
	final int width = Math.max(40, targetSpan / count);
	for (int z=0; z<count; z++) {
	    offsets[z] = z*width;
	    spans[z] = Math.min(width, (int)getView(z).getMaximumSpan(axis));
	}
    }
    protected void layoutMinorAxis(int targetSpan, int axis, int[] offsets, int[] spans) {
	final int count = cells.length;
	for (int z=0; z<count; z++) {
	    offsets[z] = 0;
	    spans[z] = Math.min(targetSpan, (int)getView(z).getMaximumSpan(axis));
	}
    }
    protected int getNextNorthSouthVisualPositionFrom(int pos, int direction)
	throws BadLocationException {
	if (pos==-1) {
	    final int x = getMagicX(pos)-allocX;
	    final int column = table.getColumnIndexAt(x);
	    View v = getView(column);
	    return v.getNextVisualPositionFrom(-1, direction);
	} else {
	    final int vIndex = getViewIndexAtPosition(pos);
	    View v = getView(vIndex);
	    return v.getNextVisualPositionFrom(pos, direction);
	}
    }
}
