/*
 *
 *  LeanEdit
 *  Copyright (c) 1999-2001 Max Kellermann (max.kellermann@epost.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: TableView.java,v 1.3 2001/09/20 20:20:53 max Exp $
 *
 */

package org.leanedit.plugins.text.view;

import java.awt.Rectangle;
import java.awt.Graphics;
import java.awt.Shape;
import org.leanedit.plugins.text.tree.ElementSwingElement;
import org.leanedit.plugins.text.tree.TableElement;
import make.swing.text.Element;
import make.swing.text.view.View;
import make.swing.text.view.ViewFactory;
import make.swing.text.view.BoxView;
import make.swing.text.view.VerticalView;
import make.xml.DOMUtil;

public class TableView extends BoxView implements VerticalView {
    /***properties***********************************/
    protected int rowAxis;
    protected int nColumns = 1;
    /***Constructor**********************************/
    public TableView(TableElement _el, int _axis) {
	super(_el, _axis);
	rowAxis = _axis==X_AXIS ? Y_AXIS : X_AXIS;
	//minorCursorWrap = false;
	nColumns = 1;
	org.w3c.dom.Element[] rw = DOMUtil.getChildElements(_el.getXmlElement());
	for (int z=0; z<rw.length; z++) {
	    int cc = DOMUtil.getChildElements(rw[z]).length;
	    if (cc>nColumns) nColumns = cc;
	}
    }
    protected void loadChildren(ViewFactory f) {
	super.loadChildren(f);
    }
    /***Column Info**********************************/
    public ElementSwingElement[] extractCells(ElementSwingElement row) {
	ElementSwingElement[] c = new ElementSwingElement[nColumns];
	for (int z=0; z<nColumns; z++)
	    c[z] = z<row.getElementCount()
		?(ElementSwingElement)row.getElementAtIndex(z)
		:null;
	return c;
    }
    public int getColumnIndexAt(int x) {
	int c = x*nColumns/width;
	return Math.max(0, Math.min(nColumns-1, c));
    }
    /***View**************************************/
    public void paint(Graphics g, Shape allocation) {
	Rectangle rect = getInsideAllocation(allocation);
	g.setColor(java.awt.Color.lightGray);
	// frame
	g.drawRect(rect.x-1, rect.y, rect.width+2, rect.height-1);
	// vertical grid
	for (int x=1; x<nColumns; x++) {
	    final int gx = rect.width*x/nColumns;
	    g.drawLine(gx, rect.y, gx, rect.y+rect.height-1);
	}
	// horizontal grid
	for (int y=1; y<getViewCount(); y++)
	    g.drawLine(rect.x, rect.y+yOffsets[y], rect.x+rect.width, rect.y+yOffsets[y]);
	super.paint(g, allocation);
    }
    /***CompositeView***************************************/
    protected void layoutMinorAxis(int targetSpan, int axis, int[] offsets, int[] spans) {
	int count = getViewCount();
	for (int z=0; z<count; z++) {
	    offsets[z] = 0;
	    spans[z] = targetSpan;
	}
    }
    protected View getViewAtPosition(int pos, Rectangle a) {
	if (pos<getStartOffset() || pos>getEndOffset())
	    return null;
	int idx = getViewIndexAtPosition(pos);
	if (a!=null)
	    childAllocation(idx, a);
	return getView(idx);
    }
    protected int getViewIndexAtPosition(int pos) {
	final int count = getViewCount();
	if (count==0 || pos<getView(0).getEndOffset()) return 0;
	if (pos>=getView(count-1).getStartOffset()) return count-1;
	for (int z=count-1; z>0; z--) {
	    View v = getView(z);
	    if (pos>=v.getStartOffset() && pos<v.getEndOffset())
		return z;
	}
	throw new IllegalStateException("view not found at "+pos);
    }
}
