/*
 *
 *  LeanEdit
 *  Copyright (c) 1999-2001 Max Kellermann (max.kellermann@epost.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: XmlViewFactory.java,v 1.4 2001/09/20 20:20:53 max Exp $
 *
 */

package org.leanedit.plugins.text.view;

import make.swing.text.Element;
import make.swing.text.view.ViewFactory;
import make.swing.text.view.View;
import make.swing.text.view.StyleView;
import make.swing.text.view.UnknownView;
import make.swing.text.view.EmptyView;
import make.swing.text.view.NullView;
import org.leanedit.plugins.text.tree.SwingBody;
import org.leanedit.plugins.text.tree.Paragraph;
import org.leanedit.plugins.text.tree.SwingText;
import org.leanedit.plugins.text.tree.FixedText;
import org.leanedit.plugins.text.tree.StyleElement;
import org.leanedit.plugins.text.tree.LinkElement;
import org.leanedit.plugins.text.tree.UnknownSwingElement;
import org.leanedit.plugins.text.tree.ImageElement;
import org.leanedit.plugins.text.tree.SwingEntityElement;
import org.leanedit.plugins.text.tree.TableElement;
import org.leanedit.plugins.text.tree.TableRowElement;
import org.leanedit.plugins.text.tree.TableCellElement;
import org.leanedit.plugins.text.tree.NoteElement;
import org.leanedit.plugins.text.tree.UnorderedList;
import org.leanedit.plugins.text.tree.NumberedList;
import org.leanedit.plugins.text.tree.Section;
import org.leanedit.plugins.text.tree.BreakElement;

public class XmlViewFactory implements ViewFactory {
    public View create(Element elem) {
	if (elem instanceof SwingBody)
	    return new BodyView(elem);
	else if (elem instanceof Section || elem instanceof TableCellElement)
	    return new SectionView(elem);
	else if (elem instanceof Paragraph) {
	    Element parent = elem.getParent();
	    if (parent instanceof UnorderedList)
		return new BulletParagraphView(elem, this);
	    else if (parent instanceof NumberedList)
		return new NumberedParagraphView(elem, this);
	    else
		return new ParagraphView(elem, this);
	} else if (elem instanceof SwingText)
	    return new TextView((SwingText)elem);
	else if (elem instanceof StyleElement ||
		 elem instanceof SwingEntityElement ||
		 elem instanceof LinkElement ||
		 elem instanceof NoteElement)
	    return new StyleView(elem);
	else if (elem instanceof ImageElement)
	    return new ImageView((ImageElement)elem);
	else if (elem instanceof TableElement)
	    return new TableView((TableElement)elem, View.Y_AXIS);
	else if (elem instanceof TableRowElement)
	    return new TableRowView((TableRowElement)elem, View.X_AXIS);
	else if (elem instanceof UnknownSwingElement)
	    return new UnknownView((UnknownSwingElement)elem, this);
	else if (elem instanceof FixedText)
	    return new FixedTextView((FixedText)elem);
	else if (elem instanceof BreakElement)
	    return new BreakView((BreakElement)elem);
	else if (elem instanceof TableCellElement.CellEnd)
	    return new EmptyView(elem, elem.getStartOffset());
	else
	    return new NullView(elem);
    }
}
