/*
 *
 *  LeanEdit
 *  Copyright (c) 1999-2001 Max Kellermann (max.kellermann@epost.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: TreeEditorComponent.java,v 1.4 2001/09/21 20:13:26 max Exp $
 *
 */

package org.leanedit.plugins.tree;

import java.awt.Component;
import javax.swing.JTree;
import javax.swing.JScrollPane;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;
import javax.swing.event.TreeModelListener;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;
import org.w3c.dom.Attr;
import make.util.Strings;
import org.leanedit.EditorFrame;
import org.leanedit.EditorDocument;
import org.leanedit.plugins.DefaultEditorComponent;

class TreeEditorComponent extends DefaultEditorComponent 
    implements TreeModel {
    protected class TreeEditor extends JTree {
	public TreeEditor() {
	    super(TreeEditorComponent.this);
	}
	public String convertValueToText(Object value, boolean selected,
					 boolean expanded, boolean leaf, int row,
					 boolean hasFocus) {
	    Node node = (Node)value;
	    if (node instanceof Text || node instanceof Attr) {
		return Strings.normalize(node.getNodeValue());
	    } else if (node instanceof Element) {
		StringBuffer sb = new StringBuffer('<'+node.getNodeName());
		NamedNodeMap nnm = node.getAttributes();
		for (int z=0; z<nnm.getLength(); z++) {
		    Attr attr = (Attr)nnm.item(z);
		    sb.append(" "+attr.getName()+"=\""+attr.getValue()+"\"");
		}
		if (!node.hasChildNodes())
		    sb.append("/");
		sb.append(">");
		return sb.toString();
	    } else
		return node.getNodeName();
	}
    }
    /*properties*********************************/
    protected JScrollPane scrollPane;
    protected JTree tree;
    /*Constructor*******************************/
    public TreeEditorComponent(EditorFrame _ef) {
	super(_ef);
    }
    /*EditorComponent****************************/
    public void setDocument(EditorDocument doc) {
	if (tree!=null)
	    unregisterInputComponent(tree);
	super.setDocument(doc);
	if (tree==null) {
	    tree = new TreeEditor();
	    registerInputComponent(tree);
	    scrollPane = new JScrollPane(tree);
	    scrollPane.getViewport().putClientProperty("EnableWindowBlit", Boolean.TRUE);
	} else
	    tree.setModel(this);
    }
    public Component getComponent() {
	return scrollPane;
    }
    /*TreeModel********************************/
    public Object getRoot() {
	return document.getNode();
    }
    public Object getChild(Object parent, int index) {
	return ((Node)parent).getChildNodes().item(index);
    }
    public int getChildCount(Object parent) {
	return ((Node)parent).getChildNodes().getLength();
    }
    public boolean isLeaf(Object node) {
	return !((Node)node).hasChildNodes();
    }
    public void valueForPathChanged(TreePath path, Object newValue) {}
    public int getIndexOfChild(Object parent, Object child) {
	NodeList nl = ((Node)parent).getChildNodes();
	for (int z=0; z<nl.getLength(); z++)
	    if (nl.item(z)==child)
		return z;
	return -1;
    }
    public void addTreeModelListener(TreeModelListener l) {}
    public void removeTreeModelListener(TreeModelListener l) {}
}
