/*
 *
 *  LeanEdit
 *  Copyright (c) 1999-2001 Max Kellermann (max.kellermann@epost.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: SchemaChooser.java,v 1.6 2002/03/14 19:16:38 max Exp $
 *
 */

package org.leanedit.schema;

import java.awt.Frame;
import java.awt.Component;
import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.util.Collection;
import java.util.Vector;
import javax.swing.Action;
import javax.swing.AbstractAction;
import javax.swing.JList;
import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.JPanel;
import javax.swing.ListCellRenderer;
import make.xml.DocumentType;
import make.swing.bt.FrameDescription;
import make.swing.panel.PanelContent;
import make.swing.panel.InterfacePanel;
import make.swing.panel.DialogInterfacePanel;

public class SchemaChooser 
    implements PanelContent, ListCellRenderer {
    /*static******************************/
    public static final FrameDescription frameDescription
        = FrameDescription.okCancelDescription;
    /*properties*************************/
    protected InterfacePanel interfacePanel;
    protected JPanel panel = new JPanel(new BorderLayout(), false);
    protected JList list;
    protected DocumentType schema;
    protected ListCellRenderer oldRenderer;
    /*Constructor************************/
    protected SchemaChooser(InterfacePanel _ip,
                            Collection schemas) {
        interfacePanel = _ip;
        panel.add("North", new JLabel("Please choose a schema from the list:"));
        list = new JList(new Vector(schemas));
        oldRenderer = list.getCellRenderer();
        list.setCellRenderer(this);
        panel.add("Center", new JScrollPane(list));
        interfacePanel.getButtonManager().installActions(actions);
        interfacePanel.addFrameDescription(frameDescription);
        interfacePanel.setContent(this);
    }
    public DocumentType getSchema() {
        return schema;
    }
    /*PanelContent**********************/
    public Component getComponent() {
        return panel;
    }
    public boolean panelClosing() {
        return true;
    }
    public void panelClosed() {
        interfacePanel.removeFrameDescription(frameDescription);
        interfacePanel.getButtonManager().uninstallActions(actions);
    }
    /*ListCellRenderer*********************/
    public Component getListCellRendererComponent(JList list,
                                                  Object value,
                                                  int index,
                                                  boolean isSelected,
                                                  boolean cellHasFocus) {
        value = ((DocumentType)value).getName();
        return oldRenderer.getListCellRendererComponent
            (list, value, index, isSelected, cellHasFocus);
    }
    /*Actions**************************/
    protected final Action[] actions = {
        new OkAction(),
    };
    class OkAction extends AbstractAction {
        OkAction() { super("ok"); }
        public void actionPerformed(ActionEvent evt) {
            schema = (DocumentType)list.getSelectedValue();
            if (schema!=null)
                interfacePanel.close();
        }
    }
    /*Static******************************/
    public static DocumentType chooseSchema(Frame frame,
                                            Collection schemas) {
        InterfacePanel ip = new DialogInterfacePanel(frame);
        SchemaChooser dlg = new SchemaChooser
            (ip, schemas);
        ip.setVisible(true);
        return dlg.getSchema();
    }
}
